/* Evolution.f -- translated by f2c (version 20020621). */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>

#define SEED     12345
#define RANDMAX  2147483648.0

/* Table of constant values */

#define N0 20

#define abs(x)     ((x) >= 0 ? (x) : - (x))
#define dabs(x)    ((double) abs(x))
#define min(a, b)  ((a) <= (b) ? (a) : (b))
#define max(a, b)  ((a) >= (b) ? (a) : (b))
#define dmin(a, b) ((double) min(a, b))
#define dmax(a, b) ((double) max(a, b))

/* Subroutine                                                               */ 

/* ************************************************************************ */
/* **                                                                    ** */
/* **    EVOLUTIONSPROGRAMM                                              ** */
/* **    ZUR EXTREMA-SUCHE BEI DER FUNKTION FKT(PAR)                     ** */
/* **                                                                    ** */
/* **    INPUT :                                                         ** */
/* **       FKT      : NAME DER REAL-FUNKTION                            ** */
/* **                  (FUNKTION FKT(PAR), WOBEI PAR IPAR-DIMENSIONAL    ** */
/* **       IPAR     : ANZAHL DER PARAMETER   (MAX.20)                   ** */
/* **       PAR0     : STARTWERTE DER PARAMETER   PAR0(IPAR)             ** */
/* **       DPAR0    : ANFANGSSCHRITTWEITEN       DPAR0(IPAR)            ** */
/* **       EPS      : GEWUENSCHTE GENAUIGKEIT (IN DPAR0)                ** */
/* **       IOPT     : OPTION (0:MINIMUM, 1:MAXIMUM)                     ** */
/* **       MAXST    : MAX NUMBER OF STEPS                               ** */
/* **                                                                    ** */
/* **    OUTPUT :                                                        ** */
/* **       PAR      : PARAMETERSATZ FUER DAS EXTREMUM   PAR(IPAR)       ** */
/* **       FEXT     : FUNKTIONSWERT AM EXTREMUM                         ** */
/* **                                                                    ** */
/* **                                                                    ** */
/* ************************************************************************ */

int evolution(double (*fkt)(), int *ipar, float *par0, float *dpar0, 
	      float *eps, int *iopt, int *maxst, float *par, float *fext)
{
  /* System generated locals */

  int i_1, i_2;
  float r_0;
  
  /* Builtin functions */  
  /* Local variables   */

  static float b;
  static int i;
  static float r[N0], f1, f2, ff;
  static int ija, iii;
  static float par1[N0], par2[N0];
  static int isum, itot, iflag;
  static float delta;
  static int idopp, ntest, ineinh;
  static float rexmin;

  /* Subroutine */ 
  
  extern double ranown(int *);

  /* Initialize th random number generator */
  
  srand(SEED);

  /* Fortran I/O blocks    */
  /* Parameter adjustments */

  --par;
  --dpar0;
  --par0;
  
  /* Function Body */
  
  idopp = 3;
  rexmin = 4.0f;
  ntest = 4;
  
  /* SCHUTZABFRAGE */
    
  if(*ipar > N0) {
    printf("*** ERROR IN EVOL: IPAR > N0 = 20 *** ABEND *** \n");
    
    goto L50;
  }
  
  /* VORBESETZUNGEN */
    
  isum = 0;
  ineinh = 0;
  ija = 0;
  itot = 0;
  iflag = 0;
  b = 1.0f;
  f1 = (*fkt)(&par0[1]);
  i_1 = *ipar;
  
  for(i = 1; i <= i_1; ++i) {
    par1[i - 1] = par0[i];
    
    /* L10: */
  }
  
  /* ***  RANDOM WALK BEREICH ANFANG *** */

 L100:
  
  /* BEI IDOPP ERFOLGREICHEN VERSUCHEN (IN GLEICHER RICHTUNG) */
  /* HINTEREINANDER SCHRITTWEITE VERDOPPELN                   */

  if(ija >= idopp) {
    b *= 2.0f;
    ija = 0;
  }
  
  /* RANDOM STEP (NUR WENN LETZTER VERSUCH ERFOLGLOS, SONST */
  /* IN GLEICHER RICHTUNG WEITER)                           */
    
  i_1 = *ipar;
  
  for(i = 1; i <= i_1; ++i) {
    if(iflag == 0) {
      i_2 = isum * *ipar + i;
      
      // r[i - 1] = (ranown(&i_2) - 0.5f) * 2.0f;

      r[i - 1] = (rand() / RANDMAX - 0.5f) * 2.0f;

      // printf("ranown = %f \n", ranown(&i_2));

      /*

      printf("random = %f \n", rand() / RANDMAX);
      printf("r[%d]  = %f \n", i - 1, r[i - 1]);

      **/
    }
    
    delta = dpar0[i] * b * r[i - 1];
    par2[i - 1] = par1[i - 1] + delta;
    
    /* L20: */
  }
  
  ++itot;
  ++isum;

  /* NEUER WERT */
    
  f2 = (*fkt)(par2);

  /* FEHLERFAKTOR (FUER ZUFAELLIG SEHR GUTE WERTE) */
    
  if(dabs(f1) > 1e-6f) {
    ff = (r_0 = (f1 - f2) / f1, dabs(r_0)) * rexmin;
  }

  /* PRINT EVERY TENTH STEP */
    
  iii  = isum / 10.0f;
  iii *= 10.0f;
  
  /*

  if(iii == isum) {
    printf("ISUM = %d \n", isum);
    printf("B    = %f \n", b);
    printf("F1   = %f \n", f1);
    printf("F2   = %f \n", f2);
  }

  **/

  /* STOP IF TOO MANY STEPS */
    
  if(isum >= *maxst) {
    goto L200;
  }

  /* VERBESSERT? */
    
  if((*iopt == 0 && f2 < f1) || (*iopt == 1 && f2 > f1)) {
    /* JA : NEUE WERTE UEBERNEHMEN, FLAG SETZEN */
    
    ++ija;
    ineinh = 0;
    iflag = 1;
    i_1 = *ipar;
    
    for(i = 1; i <= i_1; ++i) {
      par1[i - 1] = par2[i - 1];

      /* L30: */
    }
    
    f1 = f2;
  } else {
    /* NEIN : BEI ZUVIELEN ERFOLGLOSEN VERSUCHEN  */
    /* NTEST*IPAR ERFOLGLOS HINTEREINANDER        */
    /* SCHRITTWEITE VERKLEINERN                   */
    /* ALLG. FLAG RESET                           */
    /* SOWIE EV. LETZTE VERDOPPLUNG ZURUECKNEHMEN */

    if(iflag == 1 && ija == 0) {
      b *= 0.5f;
    }
    
    ineinh += dmax(1.0f, ff);
    iflag = 0;
    ija = 0;
	
    if(ineinh >= ntest * *ipar) {
      b *= 0.5f;
      ineinh = 0;
      itot = 0;
      
      /* ABBRUCH BEDINGUNG */
  
      if(b < *eps) {
	goto L200;
      }
    }
  }
  
  goto L100;
  
  /* ***  ENDE RANDOM WALK BEREICH *** */
 
 L200:
  
  /* ENDWERTE BESETZEN */
  
  i_1 = *ipar;
  
  for(i = 1; i <= i_1; ++i) {
    par[i] = par1[i - 1];

    /* L40: */
  }
  
  *fext = f1;
 
 L50:
  
  return(0);
} /* evolution */


