#include<stdio.h>
#include<stdlib.h>
#include<math.h>

double func(float *x);

int main() 
{
  int i, j, n = 11, np = 11, iter = 1000;
  float xi[36], fret, ftol = 1.0E-6;  

  // float pin[3] = {0.0, 0.0, 0.0};

  float pin[11], p[11];

  extern int powell(float *, float *, int *, int *, float *, int *, 
		    float *, float *);

  for(i = 0; i < 6; i++ ) {
    for(j = 0; j < 6; j++) {
      xi[i + j * 6] = (i == j ? 2.0 : - 2.0);
    }
  }

  pin[0]  = 0.22;
  pin[1]  = 0.24;
  pin[2]  = 0.45;
  pin[3]  = 1.82;
  pin[4]  = 0.52;
  pin[5]  = 60.0 * (M_PI / 180.0);
  pin[6]  = 0.01;
  pin[7]  = 0.01;
  pin[8]  = 0.00;
  pin[9]  = 0.00;
  pin[10] = 0.00;

  fret = func(pin);

  printf("Initial function value: %f \n", fret);

  powell(pin, xi, &n, &np, &ftol, &iter, p, &fret);

  printf("Iterations: %d \n", iter);
  printf("Minimum found at: \n");

  for(i = 0; i < 10; i++ ) 
    printf("p[%d] = %f \n", i, p[i]);
  
  fret = func(p);

  printf("Minimum function value: %f \n", fret);

  printf("p[0]  = alpha   = %g \n", p[0]);
  printf("p[1]  = epsilon = %g \n", p[1]);
  printf("p[2]  = kappa   = %g \n", p[2]);
  printf("p[3]  = RE      = %g \n", p[3]);
  printf("p[4]  = RC      = %g \n", p[4]);
  printf("p[5]  = theta   = %g \n", p[5]);
  printf("p[6]  = sigma   = %g \n", p[6]);
  printf("p[7]  = gamma   = %g \n", p[7]);
  printf("p[8]  = phi     = %g \n", p[8]);
  printf("p[9]  = yp1     = %g \n", p[9]);
  printf("p[10] = yp2     = %g \n", p[10]);
  printf("p[5]  = %g \n", p[5] * (180.0 / M_PI));
  printf("p[8]  = %g \n", p[8] * (180.0 / M_PI));

  return(0); 
}

/*

double func(float *x) 
{
  int i;
  double func = 0.0;
  
  for(i = 0; i < 3; i++) {
    func += (x[i] - 1.0) * (x[i] - 1.0);
  }
  
  return(func);
}

*/

double func(float *x)
{
  int i, j = 0;    
  
  extern double cos(), sin(), pow();

  double A        = x[0];
  double lambda   = x[1];
  double kappa    = x[2];
  double re       = x[3];
  double xc       = x[4];
  double theta    = x[5];
  double sigma    = x[6];  
  double gamma    = x[7];
  double phi      = x[8];
  double yp1      = x[9];
  double yp2      = x[10];
  double delta1   = 1.0 - A;
  double delta2   = 2.0 * A;
  double Epsilon1 = 1.0 - lambda;
  double Epsilon2 = 1.0 + lambda;
  double function = 0.0;
  double Yi[5][2], s2[5], alfa[5];
  double y[5][2], yn[5][2];
  double temp1, temp2;

  y[0][0] =   0.00;
  y[0][1] =   0.00;
  y[1][0] =   1.40;
  y[1][1] =   0.52;
  y[2][0] = - 1.20;
  y[2][1] =   1.48;        
  y[3][0] = - 1.54;
  y[3][1] = - 0.82;
  y[4][0] =   0.16;
  y[4][1] = - 1.88;

  // yp1 = yp2 = 0.0;

  for(i = 0; i < 5; i++) {
    yn[i][0] =   (y[i][0] - yp1) * cos(theta) + (y[i][1] - yp2) * sin(theta);
    yn[i][1] = - (y[i][0] - yp1) * sin(theta) + (y[i][1] - yp2) * cos(theta);
  }
  
  for(i = 0; i < 5; i++) {
    s2[i] = Epsilon1 * yn[i][0] * yn[i][0] + Epsilon2 * yn[i][1] * yn[i][1]
      + xc * xc;
  }

  for(i = 0; i < 5; i++) {
    alfa[i] = (kappa * re * re) / (pow(s2[i], delta1) * pow(xc, delta2));   
  }

  for(i = 0; i < 5; i++) {
    Yi[i][0] = yn[i][0] * (1.0 - sigma - gamma * cos(phi) - Epsilon1 * alfa[i])
      - gamma * yn[i][1] * sin(phi);
    Yi[i][1] = yn[i][1] * (1.0 - sigma + gamma * cos(phi) - Epsilon2 * alfa[i])
      - gamma * yn[i][0] * sin(phi);
  }

  yp1 = (Yi[0][0] + Yi[1][0] + Yi[2][0] + Yi[3][0] + Yi[4][0]) / 5.0;
  yp2 = (Yi[0][1] + Yi[1][1] + Yi[2][1] + Yi[3][1] + Yi[4][1]) / 5.0;

  // printf("yp1 = %g \n", yp1);
  // printf("yp2 = %g \n", yp2);

  for(i = 0; i < 5; i++) {
    for(j = 0; j < 5; j++) {
      temp1 = (Yi[i][0] - Yi[j][0]) * (Yi[i][0] - Yi[j][0]);
      temp2 = (Yi[i][1] - Yi[j][1]) * (Yi[i][1] - Yi[j][1]);
      
      function += (temp1 + temp2) / 10.0;
    } 
  }      

  // printf("F = %g \n", function);

  return(function);
} /* func */
