#include<stdio.h>
#include<stdlib.h>
#include<math.h>

double func(float *x);
double func2(float *x);

double yp1 = 0.0;
double yp2 = 0.0;

int main() 
{
  int i, n = 11, iopt = 0, iter = 5000;
  float fret, ftol = 1.0E-6;  
  float par0[11], pd[11], par[11];

  int evolution(double (*fkt)(), int *ipar, float *par0, float *dpar0, 
		float *eps, int *iopt, int *maxst, float *par, float *fext);

  par0[0]  = 0.267606;
  par0[1]  = 0.450704;
  par0[2]  = 1.00141;
  par0[3]  = 1.99549;
  par0[4]  = 0.732394;
  par0[5]  = 30.4225 * (M_PI / 180.0);
  par0[6]  = 0.00;
  par0[7]  = 0.112676;
  par0[8]  = 15.2113 * (M_PI / 180.0);

  pd[0]  = -0.0001;
  pd[1]  = -0.0001;
  pd[2]  = -0.0001;
  pd[3]  = -0.0001;
  pd[4]  = -0.0001;
  pd[5]  = -0.0001;
  pd[6]  = -0.0001;
  pd[7]  = -0.0001;
  pd[8]  = -0.0001;

  fret = func(par0);

  printf("Initial function value: %f \n", fret);

  evolution(func, &n, par0, pd, &ftol, &iopt, &iter, par, &fret);

  printf("Iterations: %d \n", iter);
  printf("Minimum found at: \n");

  for(i = 0; i < 9; i++ ) 
    printf("p[%d] = %f \n", i, par[i]);
  
  fret = func(par);

  printf("Minimum function value: %f \n", fret);

  printf("p[0] = alpha   = %g \n", par[0]);
  printf("p[1] = epsilon = %g \n", par[1]);
  printf("p[2] = kappa   = %g \n", par[2]);
  printf("p[3] = RE      = %g \n", par[3]);
  printf("p[4] = RC      = %g \n", par[4]);
  printf("p[5] = theta   = %g \n", par[5]);
  printf("p[6] = sigma   = %g \n", par[6]);
  printf("p[7] = gamma   = %g \n", par[7]);
  printf("p[8] = phi     = %g \n", par[8]);
  printf("p[5] = %g \n", par[5] * (180.0 / M_PI));
  printf("p[8] = %g \n", par[8] * (180.0 / M_PI));

  fret = func2(par);
    
  return(0); 
}

double func(float *x)
{
  int i, j = 0;    
  
  extern double cos(), sin(), pow();

  double A        = x[0];
  double lambda   = x[1];
  double kappa    = x[2];
  double re       = x[3];
  double xc       = x[4];
  double theta    = x[5];
  double sigma    = x[6];  
  double gamma    = x[7];
  double phi      = x[8];
  double delta1   = 1.0 - A;
  double delta2   = 2.0 * A;
  double Epsilon1 = 1.0 - lambda;
  double Epsilon2 = 1.0 + lambda;
  double function = 0.0;
  double Yi[5][2], s2[5], alfa[5];
  double y[5][2], yn[5][2];
  double temp1, temp2;

  y[0][0] =   0.00;
  y[0][1] =   0.00;
  y[1][0] =   1.40;
  y[1][1] =   0.52;
  y[2][0] = - 1.20;
  y[2][1] =   1.48;        
  y[3][0] = - 1.54;
  y[3][1] = - 0.82;
  y[4][0] =   0.16;
  y[4][1] = - 1.88;

  // yp1 = yp2 = 0.0;

  for(i = 0; i < 5; i++) {
    yn[i][0] =   y[i][0] * cos(theta) + y[i][1] * sin(theta);
    yn[i][1] = - y[i][0] * sin(theta) + y[i][1] * cos(theta);
  }
  
  for(i = 0; i < 5; i++) {
    s2[i] = Epsilon1 * yn[i][0] * yn[i][0] + Epsilon2 * yn[i][1] * yn[i][1]
      + xc * xc;
  }

  for(i = 0; i < 5; i++) {
    alfa[i] = (kappa * re * re) / (pow(s2[i], delta1) * pow(xc, delta2));   
  }

  for(i = 0; i < 5; i++) {
    Yi[i][0] = yn[i][0] * (1.0 - sigma - gamma * cos(2.0 * phi) 
			   - Epsilon1 * alfa[i])
      - gamma * yn[i][1] * sin(2.0 * phi);
    Yi[i][1] = yn[i][1] * (1.0 - sigma + gamma * cos(2.0 * phi) 
			   - Epsilon2 * alfa[i])
      - gamma * yn[i][0] * sin(2.0 * phi);
  }

  // yp1 = (Yi[0][0] + Yi[1][0] + Yi[2][0] + Yi[3][0] + Yi[4][0]) / 4.0;
  // yp2 = (Yi[0][1] + Yi[1][1] + Yi[2][1] + Yi[3][1] + Yi[4][1]) / 4.0;

  yp1 = Yi[1][0];
  yp2 = Yi[1][1];

  // printf("yp1 = %g \n", yp1);
  // printf("yp2 = %g \n", yp2);

  /*

  if(x[0] < 0.0 || x[0] > 0.5) {
    function += pow(10.0, 2);
  } else {
    if(x[1] < 0.0 || x[1] > 1.0) {
      function += pow(10.0, 2);
    } else {
      if(x[2] < 0.0) {
        function += pow(10.0, 2.0); 
      } else {
        if(x[3] < 0.0) {
	  function += pow(10.0, 2.0); 
        } else {
          if(x[4] < 0.0 || x[4] >= x[3]) {
            function += pow(10.0, 2.0); 
          } else {
            if(x[6] < 0.0) {
              function += pow(10.0, 2.0); 
            } else {
              if(x[7] < 0.0) {
                function += pow(10.0, 2.0);
              } else {
                for(i = 0; i < 5; i++) {
                  for(j = 0; j < 5; j++) {

  **/

  temp1 = (Yi[i][0] - Yi[j][0]) * (Yi[i][0] - Yi[j][0]);
  temp2 = (Yi[i][1] - Yi[j][1]) * (Yi[i][1] - Yi[j][1]);

  function += (temp1 + temp2) / 8.0;

  /*                  	
                  }       
                }     
              }
            }
          }
        }
      }
    }
  }

  **/

  // printf("F = %g \n", function);

  return(function);
} /* func */

double func2(float *x)
{
  int i, j = 0;    
  
  extern double cos(), sin(), pow();

  double A        = x[0];
  double lambda   = x[1];
  double kappa    = x[2];
  double re       = x[3];
  double xc       = x[4];
  double theta    = x[5];
  double sigma    = x[6];  
  double gamma    = x[7];
  double phi      = x[8];
  double delta1   = 1.0 - A;
  double delta2   = 2.0 * A;
  double Epsilon1 = 1.0 - lambda;
  double Epsilon2 = 1.0 + lambda;
  double function = 0.0;
  double Yi[5][2], s2[5], alfa[5];
  double y[5][2], yn[5][2];
  double temp1, temp2;

  y[0][0] =   0.00;
  y[0][1] =   0.00;
  y[1][0] =   1.40;
  y[1][1] =   0.52;
  y[2][0] = - 1.20;
  y[2][1] =   1.48;        
  y[3][0] = - 1.54;
  y[3][1] = - 0.82;
  y[4][0] =   0.16;
  y[4][1] = - 1.88;

  // yp1 = yp2 = 0.0;

  for(i = 0; i < 5; i++) {
    yn[i][0] =   y[i][0] * cos(theta) + y[i][1] * sin(theta);
    yn[i][1] = - y[i][0] * sin(theta) + y[i][1] * cos(theta);
  }
  
  for(i = 0; i < 5; i++) {
    s2[i] = Epsilon1 * yn[i][0] * yn[i][0] + Epsilon2 * yn[i][1] * yn[i][1]
      + xc * xc;
  }

  for(i = 0; i < 5; i++) {
    alfa[i] = (kappa * re * re) / (pow(s2[i], delta1) * pow(xc, delta2));   
  }

  for(i = 0; i < 5; i++) {
    Yi[i][0] = yn[i][0] * (1.0 - sigma - gamma * cos(2.0 * phi) 
			   - Epsilon1 * alfa[i])
      - gamma * yn[i][1] * sin(2.0 * phi);
    Yi[i][1] = yn[i][1] * (1.0 - sigma + gamma * cos(2.0 * phi) 
			   - Epsilon2 * alfa[i])
      - gamma * yn[i][0] * sin(2.0 * phi);
  }

  yp1 = (Yi[0][0] + Yi[1][0] + Yi[2][0] + Yi[3][0] + Yi[4][0]) / 4.0;
  yp2 = (Yi[0][1] + Yi[1][1] + Yi[2][1] + Yi[3][1] + Yi[4][1]) / 4.0;

  printf("yp1 = %g \n", yp1);
  printf("yp2 = %g \n", yp2);

  if(x[0] < 0.0 || x[0] > 0.5) {
    function += pow(10.0, 2);
  } else {
    if(x[1] < 0.0 || x[1] > 1.0) {
      function += pow(10.0, 2);
    } else {
      if(x[2] < 0.0) {
        function += pow(10.0, 2.0); 
      } else {
        if(x[3] < 0.0) {
	  function += pow(10.0, 2.0); 
        } else {
          if(x[4] < 0.0 || x[4] >= x[3]) {
            function += pow(10.0, 2.0); 
          } else {
            if(x[6] < 0.0) {
              function += pow(10.0, 2.0); 
            } else {
              if(x[7] < 0.0) {
                function += pow(10.0, 2.0);
              } else {
                for(i = 0; i < 5; i++) {
                  for(j = 0; j < 5; j++) {
                    temp1 = (Yi[i][0] - Yi[j][0]) * (Yi[i][0] - Yi[j][0]);
                    temp2 = (Yi[i][1] - Yi[j][1]) * (Yi[i][1] - Yi[j][1]);
                    
		    function += (temp1 + temp2) / 8.0;
                  }       
                }     
              }
            }
          }
        }
      }
    }
  }

  // printf("F = %g \n", function);

  return(function);
} /* func2 */
