/* Monte_Carlo.f -- translated by f2c (version 20020621). */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define FALSE 0
#define TRUE  1

/* Subroutine                                                              */ 
/*                                                                         */
/* MONTE CARLO INTEGRATION OF FUNCTION FKT(X,Y)                            */
/*                                                                         */
/*      FKT(X<Y)      REAL FUNCTION                                        */
/*      LIMIT(X<Y)    LOGICAL FUNCTION                                     */
/*                    .TRUE. DEFINES INTEGRATION REGION                    */
/*      XI,XE,YI,YE   REGION FOR MONTE CARLO VARIATION                     */
/*      N             NUMBER OF POINTS                                     */
/*      VALINT        VALUE OF INTEGRAL                                    */
/*      NSUM          NUMBER OF POINTS USED, I.E. INSIDE REGION DEFINED BY */
/*                    LIMIT                                                */
/* *********************************************************************** */

int mcint(double (*fkt)(), int (*limit)(), float *xi, float *xe, 
	  float *yi, float *ye, int *n, float *valint, int *nsum)
{
  /* System generated locals */

  int i_1, i_2;

  /* Local variables */
  
  static int i;
  static float x, y, dxdy;
  extern double ranown(int *);

  /* **************************** */
  /* CALCULATE DX DY = REGION / N */
    
  dxdy = (*xe - *xi) * (*ye - *yi) / *n;

  /* INIT INTEGRAL VALUE */

  *valint = 0.0f;
  *nsum = 0;
  
  /* LOOP FOR INTEGRATION */
    
  i_1 = *n;
  
  for(i = 1; i <= i_1; ++i) {
    /* GENERATE POINT */
    
    i_2 = i << 1;
    x = ranown(&i_2);
    i_2 = (i << 1) + 1;
    y = ranown(&i_2);
    
    /* SCALING */
    
    x = x * (*xe - *xi) + *xi;
    y = y * (*ye - *yi) + *yi;
    
    /* ADD TO INTEGRAL IF INSIDE REGION */
    
    if((*limit)(&x, &y)) {
      *valint += (*fkt)(&x, &y);
      ++(*nsum);
    }
    
    /* L100: */
  }
  
  /* SCALE INTEGRAL */
  
  *valint *= dxdy;
  
  return(0);
} /* mcint */

int limit(float *x, float *y)
{
  /* System generated locals */

  int ret_val;
  
  /* Builtin functions */
  
  double sqrt(double);
  
  /* Local variables */
  
  static float r;
  
  r = sqrt(*x * *x + *y * *y);
  
  ret_val = r < 1.0f;
  
  return(ret_val);
} /* limit */

/*

int limit(float *x, float *y)
{
  if(*x > *y) {
    return(FALSE);
  } else { 
    return(TRUE);
  }
}

int limit(float *x, float *y, float *xi, float *yi, float *xe, float *ye)
{
  if((*x > *xe || *y > *ye) || (*x < *xi || *y < *yi)){
    return(FALSE);
  } else { 
    return(TRUE);
  }
}

*/

