/* pikaia.f -- translated by f2c (version 20020621). */

#include <stdio.h>
#include <math.h>

#define FALSE_ 0
#define TRUE_  1

#define min(a, b)  ((a) <= (b) ? (a) : (b))
#define max(a, b)  ((a) >= (b) ? (a) : (b))
#define dmin(a, b) (double)min(a, b)
#define dmax(a, b) (double)max(a, b)

/* Common Block Declarations */

struct {
    int iseed;
} rnseed;

/* Table of constant values */

static int c_2 = 2;
static int c_12 = 12;
static int c_32 = 32;
static float c_b115 = 10.0f;

/* Main program */ 

/* ====================================================================== */
/*     Sample driver program for pikaia.f                                 */
/* ====================================================================== */

// int main(void)

int xpikaia(void)
{
  /* Local variables */
  
  static float f;
  static int i_;
  static float x[2];
  static int seed;
  static float ctrl[12];
  static int status;
  
  /* Subroutines */ 

  extern int pikaia(double (*func)(), int *, float *, float *, float *, 
		    int *); 
  extern int rninit(int *);

  /* twod is an example fitness function, a smooth 2-d landscape */

  extern double twod();
    
 L1:

  /* First, initialize the random-number generator */
  
  printf("Random number seed (I*4)? ");
  scanf("%d", &seed);
  printf("\n");
  
  rninit(&seed);

  /*     Set control variables (use defaults) */
  
  for(i_ = 1; i_ <= c_12; ++i_) {
    ctrl[i_ - 1] = -1.0f;
  }
  
  ctrl[1] = 50.0f;
  
  /*     Now call pikaia */
  
  pikaia(twod, &c_2, ctrl, x, &f, &status);
  
  /* Print the results */

  printf("status: %d \n", status);
  printf("     x: \n");

  for(i_ = 1; i_ <= c_2; i_++) {
    printf("%f \n", x[i_ - 1]);
  }

  printf("     f: %f \n", f);
  printf("  ctrl: \n");
  
  for(i_ = 1; i_ <= c_12; i_++) {
    printf("%f \n", ctrl[i_ - 1]);
  }

  goto L1;
  
  // return(0);
} /* MAIN */

/* ********************************************************************* */

/* Function */ 

/* ===================================================================== */
/*     Compute sample fitness function (2-d landscape)                   */
/* ===================================================================== */

/* Input     - *n, *x                                   */
/* Output    - twod                                     */
/* Constants - pi = 3.1415926536, sigma2 = 0.15, nn = 9 */
/* Local     - rr                                       */

double twod(int *n, float *x)
{
  /* System generated locals */
  
  int nn = 9;
  float pi = 3.1415926536f, sigma2 = 0.15f;
  float ret_val, r_1, r_2;
  
  /* Subroutine */ 

  double sqrt(double), cos(double), exp(double);
  
  /* Local variables */
  
  static float rr;
  
  /* Parameter adjustments */
  
  --x;
  
  /* Function Body */
  
  if(x[1] > 1.0f || x[2] > 1.0f) {
    return(0);
  }

  /* Computing 2nd power */
  
  r_1 = 0.5f - x[1];
  
  /* Computing 2nd power */
  
  r_2 = 0.5f - x[2];
  rr = sqrt(r_1 * r_1 + r_2 * r_2);
  
  /* Computing 2nd power */
  
  r_1 = cos(rr * ((float) nn) * pi);

  /* Computing 2nd power */
  
  r_2 = rr;
  ret_val = r_1 * r_1 * exp(-(r_2 * r_2) / sigma2);
  
  return(ret_val);
} /* twod */

/* ********************************************************************* */

/* Subroutine */ 

/* ===================================================================== */
/*     Return the next pseudo-random deviate from a sequence which is    */
/*     uniformly distributed in the interval [0,1]                       */

/*     Uses the function ran0, the "minimal standard" random number      */
/*     generator of Park and Miller (Comm. ACM 31, 1192-1201, Oct 1988;  */
/*     Comm. ACM 36 No. 7, 105-110, July 1993).                          */
/* ===================================================================== */

/* Input  - none  */
/* Output - urand */
/* Local  - none  */

double urand(void)
{
  /* System generated locals */
  
  float ret_val;
  
  /* Local variables */
  
  extern double ran0(int *);
  
  /*     Common block to make iseed visible to rninit (and to save */
  /*     it between calls) */
  
  ret_val = ran0(&rnseed.iseed);
  
  return(ret_val);
} /* urand */

/* ********************************************************************* */

/* Subroutine */ 

/* ===================================================================== */
/*     Initialize random number generator urand with given seed          */
/* ===================================================================== */

/* Input  - *seed */
/* Output - none  */
/* Local  - none  */

/* Common block to communicate with urand */
/* rnseed.iseed                           */

int rninit(int *seed)
{
  /* Set the seed value */
  
  rnseed.iseed = *seed;
  
  if(rnseed.iseed <= 0) {
    rnseed.iseed = 123456;
  }
  return(0);
} /* rninit */


/* ********************************************************************* */

/* Subroutine */ 

/* ===================================================================== */
/*     "Minimal standard" pseudo-random number generator of Park and     */
/*     Miller.  Returns a uniform random deviate r s.t. 0 < r < 1.0.     */
/*     Set seed to any non-zero int value to initialize a sequence,      */
/*     then do not change seed between calls for successive deviates     */
/*     in the sequence.                                                  */

/*     References:                                                       */
/*        Park, S. and Miller, K., "Random Number Generators: Good Ones  */
/*        are Hard to Find", Comm. ACM 31, 1192-1201 (Oct. 1988)         */
/*        Park, S. and Miller, K., in "Remarks on Choosing and Imple-    */
/*           menting Random Number Generators", Comm. ACM 36 No. 7,      */
/*           105-110 (July 1993)                                         */
/* ===================================================================== */

/* Input/Output - *seed                                           */
/* Output       - ran0                                            */
/* Constants    - A = 48271, M = 2147483647, Q = 44488, R = 3399, */
/*                SCALE = 1.0/M, EPS = 1.2e-7, RNMX = 1.0 - EPS   */
/* Local        - j                                               */

double ran0(int *seed)
{
  /* *** Declaration section *** */
  
  int A = 48271, M = 2147483647, Q = 44488, R = 3399;
  float SCALE = 1.0 / ((float) M), EPS = 1.2e-7, RNMX = 1.0 - EPS;
  
  /* System generated locals */
  
  float ret_val, r_1;
  
  /* Local variables */
  
  static int j;
  
  /* *** Executable section *** */

  j = *seed / ((float) Q);
  *seed = ((float) A) * (*seed - j * ((float) Q)) - ((float) R) * j;
 
  if(*seed < 0) {
    *seed += M;
  }

  /* Computing MIN */
  
  r_1 = *seed * SCALE;
  ret_val = dmin(r_1, RNMX);
  
  return(ret_val);
} /* ran0 */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     Return int array p which indexes array a in increasing order.      */
/*     Array a is not disturbed.  The Quicksort algorithm is used.        */

/*     B. G. Knapp, 86/12/23                                              */

/*     Reference: N. Wirth, Algorithms and Data Structures,               */
/*     Prentice-Hall, 1986                                                */
/* ====================================================================== */

/* Input:    - *n, *a                                   */
/* Output:   - *p                                       */
/* Constants - LGN = log base 2 of maximum n;           */
/*             Q = smallest subfile to use quicksort on */
/* Local     - i_, j, l, m, r_, s, t, x, stackl, stackr */

int rqsort(int *n, float *a, int *p)
{
  /* System generated locals */
  
  int i_1, i_2;
  
  /* Local variables */
  
  static int i_, j, l, m, r_, s, t;
  static float x;
  static int stackl[32], stackr[32];
  
  /* Parameter adjustments */
  
  --p;
  --a;
  
  /* Function Body */  
  /* Initialize the stack */
  
  stackl[0] = 1;
  stackr[0] = *n;
  s = 1;

  /* Initialize the pointer array */
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    p[i_] = i_;
  }

 L2:
  
  if(s > 0) {
    l = stackl[s - 1];
    r_ = stackr[s - 1];
    --s;

  L3:
    
    if(r_ - l < 11) {
      /* Use straight insertion */
      
      i_1 = r_;
      
      for(i_ = l + 1; i_ <= i_1; ++i_) {
	t = p[i_];
	x = a[t];
	i_2 = l;
	
	for(j = i_ - 1; j >= i_2; --j) {
	  if(a[p[j]] <= x) {
	    goto L5;
	  }
	  
	  p[j + 1] = p[j];
	}
	
	j = l - 1;
      
      L5:
	
	p[j + 1] = t;
      }
    } else {
      /* Use quicksort, with pivot as median of a(l), a(m), a(r) */
      
      m = (l + r_) / 2;
      t = p[m];
      
      if(a[t] < a[p[l]]) {
	p[m] = p[l];
	p[l] = t;
	t = p[m];
      }
	
      if(a[t] > a[p[r_]]) {
	p[m] = p[r_];
	p[r_] = t;
	t = p[m];
	
	if(a[t] < a[p[l]]) {
	  p[m] = p[l];
	  p[l] = t;
	  t = p[m];
	}
      }
      
      /*           Partition */
      
      x = a[t];
      i_ = l + 1;
      j = r_ - 1;

    L7:
      
      if(i_ <= j) {
	
      L8:
	
	if(a[p[i_]] < x) {
	  ++i_;
	 
	  goto L8;
	}
	
      L9:
	
	if(x < a[p[j]]) {
	  --j;
	  
	  goto L9;
	}
	
	if(i_ <= j) {
	  t = p[i_];
	  p[i_] = p[j];
	  p[j] = t;
	  ++i_;
	  --j;
	}
	
	goto L7;
      }
      
      /* Stack the larger subfile */
      
      ++s;
      
      if(j - l > r_ - i_) {
	stackl[s - 1] = l;
	stackr[s - 1] = j;
	l = i_;
      } else {
	stackl[s - 1] = i_;
	stackr[s - 1] = r_;
	r_ = j;
      }
      
      goto L3;
    }
  
    goto L2;
  }
  
  return(0);
} /* rqsort */

/* *********************************************************************** */

/* Subroutine */ 

/* ======================================================================= */
/*     Optimization (maximization) of user-supplied "fitness" function     */
/*     ff  over n-dimensional parameter space  x  using a basic genetic    */
/*     algorithm method.                                                   */

/*     Paul Charbonneau & Barry Knapp                                      */
/*     High Altitude Observatory                                           */
/*     National Center for Atmospheric Research                            */
/*     Boulder CO 80307-3000                                               */
/*     <paulchar@hao.ucar.edu>                                             */
/*     <knapp@hao.ucar.edu>                                                */

/*     Version 1.2 [2002 April 3]                                          */

/*     Genetic algorithms are heuristic search techniques that             */
/*     incorporate in a computational setting, the biological notion       */
/*     of evolution by means of natural selection.  This subroutine        */
/*     implements the three basic operations of selection, crossover,      */
/*     and mutation, operating on "genotypes" encoded as strings.          */

/*     Version 1.2 differs from version 1.0 (December 1995) in that        */
/*     it includes (1) two-point crossover, (2) creep mutation, and        */
/*     (3) dynamical adjustment of the mutation rate based on metric       */
/*     distance in parameter space.                                        */

/*     References:                                                         */

/*        Charbonneau, Paul., "An introduction to gemetic algorithms for   */
/*        numerical optimization", NCAR Technical Note TN-450+IA           */
/*        (April 2002)                                                     */

/*        Charbonneau, Paul., "Release Notes for PIKAIA 1.2",              */
/*        NCAR Technical Note TN-451+STR (April 2002)                      */

/*        Charbonneau, Paul, and Knapp, Barry., "A User's Guide            */
/*        to PIKAIA 1.0", NCAR Technical Note TN-418+IA                    */
/*        (December 1995)                                                  */

/*        Goldberg, David E., Genetic Algorithms in Search, Optimization,  */
/*        & Machine Learning. Addison-Wesley, 1989.                        */

/*        Davis, Lawrence, ed.,  Handbook of Genetic Algorithms,           */
/*        Van Nostrand Reinhold, 1991.                                     */

/* ======================================================================= */
/*     USES: ff, urand, setctl, report, rnkpop, select, encode, decode,    */
/*           cross, mutate, genrep, stdrep, newpop, adjmut                 */

/*     Input - n, ff                                                       */

/*      o Int n is the parameter space dimension, i.e., the number         */
/*        of adjustable parameters.                                        */

/*      o Function  ff  is a user-supplied scalar function of n vari-      */
/*        ables, which must have the calling sequence f = ff(n,x), where   */
/*        x is a float parameter array of length n.  This function must    */
/*        be written so as to bound all parameters to the interval [0,1];  */
/*        that is, the user must determine a priori bounds for the para-   */
/*        meter space, and ff must use these bounds to perform the appro-  */
/*        priate scalings to recover true parameter values in the          */
/*        a priori ranges.                                                 */

/*        By convention, ff should return higher values for more optimal   */
/*        parameter values (i.e., individuals which are more "fit").       */
/*        For example, in fitting a function through data points, ff       */
/*        could return the inverse of chi**2.                              */

/*        In most cases initialization code will have to be written        */
/*        (either in a driver or in a separate subroutine) which loads     */
/*        in data values and communicates with ff via one or more labeled  */
/*        common blocks.  An example exercise driver and fitness function  */
/*        are provided in the accompanying file, xpkaia.f.                 */


/*      Input/Output - ctrl(12)                                            */

/*      o Array  ctrl  is an array of control flags and parameters, to     */
/*        control the genetic behavior of the algorithm, and also printed  */
/*        output.  A default value will be used for any control variable   */
/*        which is supplied with a value less than zero.  On exit, ctrl    */
/*        contains the actual values used as control variables.  The       */
/*        elements of ctrl and their defaults are:                         */

/*           ctrl( 1) - number of individuals in a population (default     */
/*                      is 100)                                            */
/*           ctrl( 2) - number of generations over which solution is       */
/*                      to evolve (default is 500)                         */
/*           ctrl( 3) - number of significant digits (i.e., number of      */
/*                      genes) retained in chromosomal encoding (default   */
/*                      is 6)  (Note: This number is limited by the        */
/*                      machine floating point precision.  Most 32-bit     */
/*                      floating point representations have only 6 full    */
/*                      digits of precision.  To achieve greater preci-    */
/*                      sion this routine could be converted to double     */
/*                      precision, but note that this would also require   */
/*                      a double precision random number generator, which  */
/*                      likely would not have more than 9 digits of        */
/*                      precision if it used 4-byte ints internally.)      */
/*           ctrl( 4) - crossover probability; must be  <= 1.0 (default    */
/*                      is 0.85). If crossover takes place, either one     */
/*                      or two splicing points are used, with equal        */
/*                      probabilities                                      */
/*           ctrl( 5) - mutation mode; 1/2/3/4/5 (default is 2)            */
/*                      1=one-point mutation, fixed rate                   */
/*                      2=one-point, adjustable rate based on fitness      */
/*                      3=one-point, adjustable rate based on distance     */
/*                      4=one-point+creep, fixed rate                      */
/*                      5=one-point+creep, adjustable rate based on        */
/*                      fitness                                            */
/*                      6=one-point+creep, adjustable rate based on        */
/*                      distance                                           */
/*           ctrl( 6) - initial mutation rate; should be small (default    */
/*                      is 0.005) (Note: the mutation rate is the proba-   */
/*                      bility that any one gene locus will mutate in      */
/*                      any one generation.)                               */
/*           ctrl( 7) - minimum mutation rate; must be >= 0.0 (default     */
/*                      is 0.0005)                                         */
/*           ctrl( 8) - maximum mutation rate; must be <= 1.0 (default     */
/*                      is 0.25)                                           */
/*           ctrl( 9) - relative fitness differential; range from 0        */
/*                      (none) to 1 (maximum).  (default is 1.)            */
/*           ctrl(10) - reproduction plan; 1/2/3=Full generational         */
/*                      replacement/Steady-state-replace-random/Steady-    */
/*                      state-replace-worst (default is 3)                 */
/*           ctrl(11) - elitism flag; 0/1=off/on (default is 0)            */
/*                      (Applies only to reproduction plans 1 and 2)       */
/*           ctrl(12) - printed output 0/1/2=None/Minimal/Verbose          */
/*                      (default is 0)                                     */

/*     Output - x(n), f, status                                            */

/*      o Array  x(1:n)  is the "fittest" (optimal) solution found,        */
/*        i.e., the solution which maximizes fitness function ff           */

/*      o Scalar  f  is the value of the fitness function at x             */

/*      o Int  status  is an indicator of the success or failure           */
/*        of the call to pikaia (0=success; non-zero=failure)              */

/*     Constants - NMAX = 32, PMAX = 128, DMAX = 6                         */

/*      o NMAX is the maximum number of adjustable parameters              */
/*        (n <= NMAX)                                                      */

/*      o PMAX is the maximum population (ctrl(1) <= PMAX)                 */

/*      o DMAX is the maximum number of Genes (digits) per Chromosome      */
/*        segement (parameter) (ctrl(3) <= DMAX)                           */

/*     Local variables - np, nd, ngen, imut, irep, ielite, ivrb, k, ip,    */
/*                       ig, ip1, ip2, new, newtot, pcross, pmut, pmutmn,  */
/*			 pmutmx, fdif, ph(NMAX,2), oldph(NMAX,PMAX),       */
/*                       newph(NMAX,PMAX), gn1(NMAX*DMAX), gn2(NMAX*DMAX)  */
/*                       ifit(PMAX), jfit(PMAX), fitns(PMAX)               */

/*     User-supplied uniform random number generator: urand                */

/*     Function urand should not take any arguments.  If the user wishes   */
/*     to be able to initialize urand, so that the same sequence of        */
/*     random numbers can be repeated, this capability could be imple-     */
/*     mented with a separate subroutine, and called from the user's       */
/*     driver program.  An example urand function (and initialization      */
/*     subroutine) which uses the function ran0 (the "minimal standard"    */
/*     random number generator of Park and Miller [Comm. ACM 31, 1192-     */
/*     1201, Oct 1988; Comm. ACM 36 No. 7, 105-110, July 1993]) is         */
/*     provided.                                                           */

int pikaia(double (*ff)(), int *n, float *ctrl, float *x, float *f, 
	   int *status)
{
  /* System generated locals */
  
  int i_1, i_2;
  
  /* Local variables */
  
  static int k, ig, nd;
  static int ip, np, ip1, ip2, new_;
  static int ngen, ifit[128], jfit[128], irep, ivrb, imut;
  static int ielite;
  static int newtot;
  static float fdif, pmut;
  static float pcross;
  static float pmutmn, pmutmx;
  
  /* gn1[NMAX*DMAX] is gn1[192] and gn2(NMAX*DMAX) is gn2[192] */  
  
  static int gn1[192], gn2[192];
  
  /* ph[NMAX][2] was [32][2] */;
  
  static float ph[64];	
  
  /* oldph[NMAX][PMAX] was [32][128] */;
  
  static float oldph[4096];	
  
  /* newph[NMAX][PMAX] was [32][128] */

  static float newph[4096];
  
  /* fitns[PMAX] is fitns[128] */
  
  static float fitns[128];
  
  /* Subroutines */ 
  
  extern int cross(int *, int *, float *, int *, int *);
  extern int decode(int *, int *, int *, float *);
  extern int encode(int *, int *, float *, int *);
  extern int select(int *, int *, float *, int *);
  extern int genrep(int *, int *, int *, int *, float *, float *); 
  extern int adjmut(int *, int *, int *, float *, float *, int *, 
		    float *, float *, float *, int *);
  extern int setctl(float *, int *, int *, int *, int *, float *, float *, 
		    float *, float *, int *, float *, int *, int *, int *, 
		    int *);
  extern int mutate(int *, int *, float *, int *, int *);
  extern int stdrep(double (*func)(), int *, int *, int *, int *, int *, 
		    float *, float *, float *, int *, int *, int *);
  extern int newpop(double (*func)(), int *, int *, int *, int *, 
		    float *, float *, int *, int *, float *, int *);
  extern int rnkpop(int *, float *, int *, int *); 
  extern int report(int *, int *, int *, int *, int *, float *, float *, 
		    int *, float *, int *, int *);
  extern double urand(void);
  
  /* Parameter adjustments */
  
  --x;
  --ctrl;
  
  /* Function Body */
  /* Set control variables from input and defaults */
    
  setctl(&ctrl[1], n, &np, &ngen, &nd, &pcross, &pmutmn, &pmutmx, &pmut, &
	 imut, &fdif, &irep, &ielite, &ivrb, status);
  
  if(*status != 0) {
    printf("Control vector (ctrl) argument(s) invalid \n");
    return(0);
  }

  /* Make sure locally-dimensioned arrays are big enough */
    
  if(*n > 32 || np > 128 || nd > 6) {
    printf(" Number of parameters, population, or genes too large");

    *status = -1;
    
    return(0);
  }
  
  /* Compute initial (random but bounded) phenotypes */
  
  i_1 = np;
  
  for(ip = 1; ip <= i_1; ++ip) {
    i_2 = *n;
    
    for(k = 1; k <= i_2; ++k) {
      oldph[k + (ip << 5) - 33] = urand();
    }
    
    fitns[ip - 1] = (*ff)(n, &oldph[(ip << 5) - 32]);
  }
  
  /* Rank initial population by fitness order */
    
  rnkpop(&np, fitns, ifit, jfit);
  
  /* Main Generation Loop */
  
  i_1 = ngen;
  
  for(ig = 1; ig <= i_1; ++ig) {
    /* Main Population Loop */
    
    newtot = 0;
    i_2 = np / 2;
	
    for(ip = 1; ip <= i_2; ++ip) {
      /* 1. pick two parents */
      
      select(&np, jfit, &fdif, &ip1);
    
    L21:
	
      select(&np, jfit, &fdif, &ip2);
      
      if(ip1 == ip2) {
	goto L21;
      }

      /* 2. encode parent phenotypes */
      
      encode(n, &nd, &oldph[(ip1 << 5) - 32], gn1);
      encode(n, &nd, &oldph[(ip2 << 5) - 32], gn2);
      
      /* 3. breed */
	 
      cross(n, &nd, &pcross, gn1, gn2);
      mutate(n, &nd, &pmut, gn1, &imut);
      mutate(n, &nd, &pmut, gn2, &imut);
      
      /* 4. decode offspring genotypes */
	 
      decode(n, &nd, gn1, ph);
      decode(n, &nd, gn2, &ph[32]);
      
      /* 5. insert into population */
	
      if(irep == 1) {
	genrep(&c_32, n, &np, &ip, ph, newph);
      } else {
	stdrep(ff, &c_32, n, &np, &irep, &ielite, ph, oldph, 
	       fitns, ifit, jfit, &new_);
	
	newtot += new_;
      }

      /* End of Main Population Loop */
    }
    
    /* if running full generational replacement: swap populations */
	
    if(irep == 1) {
      newpop(ff, &ielite, &c_32, n, &np, oldph, newph, ifit, 
	     jfit, fitns, &newtot);
    }

    /* adjust mutation rate? */
	
    if(imut == 2 || imut == 3 || imut == 5 || imut == 6) {
      adjmut(&c_32, n, &np, oldph, fitns, ifit, &pmutmn, &pmutmx, 
	     &pmut, &imut);
    }
    
    if(ivrb > 0) {
      report(&ivrb, &c_32, n, &np, &nd, oldph, fitns, ifit, &pmut, 
	     &ig, &newtot);
    }

    /* End of Main Generation Loop */
  }

  /* Return best phenotype and its fitness */
    
  i_1 = *n;
  
  for(k = 1; k <= i_1; ++k) {
    x[k] = oldph[k + (ifit[np - 1] << 5) - 33];
  }
  
  *f = fitns[ifit[np - 1] - 1];
  
  return(0);
} /* pikaia */

/* ******************************************************************** */

/* Subroutine */ 

/* ===================================================================  */
/*     Set control variables and flags from input and defaults          */
/* ===================================================================  */

/* Input        - n                                                     */
/* Input/Output - ctrl(12)                                              */
/* Output       - np, ngen, nd, imut, irep, ielite, ivrb, status,       */ 
/*                pcross, pmutmn, pmutmx, pmut, fdif                    */
/* Local        - i, DFAULT(12)                                         */ 
/*                DFAULT                                                */ 
/*                100, 500, 5, 0.85, 2, 0.005, 0.0005, 0.25, 1, 1, 1, 0 */

int setctl(float *ctrl, int *n, int *np, int *ngen, int *nd, float *pcross, 
	   float *pmutmn, float *pmutmx, float *pmut, int *imut, float *fdif, 
	   int *irep, int *ielite, int *ivrb, int *status)
{
  /* Initialized data */
  
  static float dfault[12] = {100.0f, 500.0f, 5.0f, 0.85f, 2.0f, 0.005f, 
			     5.0e-4f, 0.25f, 1.0f, 1.0f, 1.0f, 0.0f};

  static int i_;

  /* Parameter adjustments */
  
  --ctrl;
  
  /* Function Body */
  
  for(i_ = 1; i_ <= 12; ++i_) {
    if(ctrl[i_] < 0.f) {
      ctrl[i_] = dfault[i_ - 1];
    }
  }
  
  *np = ctrl[1];
  *ngen = ctrl[2];
  *nd = ctrl[3];
  *pcross = ctrl[4];
  *imut = ctrl[5];
  *pmut = ctrl[6];
  *pmutmn = ctrl[7];
  *pmutmx = ctrl[8];
  *fdif = ctrl[9];
  *irep = ctrl[10];
  *ielite = ctrl[11];
  *ivrb = ctrl[12];
  *status = 0;
  
  /* Print a header */
  
  if(*ivrb > 0) {
    printf("************************************************** \n");
    printf("* PIKAIA Genetic Algorithm Report \n");
    printf("************************************************** \n");
    printf("Number of Generations evolving: %d \n", *ngen);
    printf("Individuals per generation:     %d \n", *np);
    printf("Number of Chromosome segments:  %d \n", *n);
    printf("Length of Chromosome segments:  %d \n", *nd);
    printf("Crossover probability:          %lf \n", *pcross);
    printf("Initial mutation rate:          %lf \n", *pmut);
    printf("Minimum mutation rate:          %lf \n", *pmutmn);
    printf("Maximum mutation rate:          %lf \n", *pmutmx);
    printf("Relative fitness differential:  %lf \n", *fdif);

    if(*imut == 1) {
      printf("Uniform, Constant Rate             \n");
    }
    
    if(*imut == 2) {
      printf("Uniform, Variable Rate (F)         \n");
    }
    
    if(*imut == 3) {
      printf("Uniform, Variable Rate (D)         \n");
    }
	
    if(*imut == 4) {
      printf("Uniform + Creep, Constant Rate     \n");
    }
    
    if(*imut == 5) {
      printf("Uniform + Creep, Variable Rate (F) \n");
    }
	
    if(*imut == 6) {
      printf("Uniform + Creep, Variable Rate (D) \n");
    }

    printf("Mutation Mode:                       \n");
	
    if(*irep == 1) {
      printf("Full generational replacement      \n");
    }
    
    if(*irep == 2) {
      printf("Steady-state-replace-random        \n");
    }
    
    if(*irep == 3) {
      printf("Steady-state-replace-worst         \n");
    }

    printf("Reproduction Plan:                   \n");
    
  }
  
  /* Check some control values */
  
  if(*imut != 1 && *imut != 2 && *imut != 3 && 
     *imut != 4 && *imut != 5 && *imut != 6) {
    printf("ERROR: illegal value for imut (ctrl(5)     \n");
    
    *status = 5;
  }
  
  if(*fdif > 1.0f) {
    printf("ERROR: illegal value for fdif (ctrl(5)     \n");

    *status = 9;
  }
  
  if(*irep != 1 && *irep != 2 && *irep != 3) {
    printf("ERROR: illegal value for irep (ctrl(10))   \n");
  
    *status = 10;
  }
  
  if(*pcross > 1.0f || *pcross < 0.0f) {
    printf("ERROR: illegal value for pcross (ctrl(4))  \n");
    
    *status = 4;
  }
  
  if(*ielite != 0 && *ielite != 1) {
    printf("ERROR: illegal value for ielite (ctrl(11)) \n");
    
    *status = 11;
  }
  
  if(*irep == 1 && *imut == 1 && *pmut > .5f && *ielite == 0) {
    printf("WARNING: dangerously high value for pmut (ctrl(6));      \n");
    printf("(Should enforce elitism with ctrl(11) = 1.0)             \n");

  }
  
  if(*irep == 1 && *imut == 2 && *pmutmx > 0.5f && *ielite == 0) {
    printf("WARNING: dangerously high value for pmutmx (ctrl(8));    \n");
    printf("(Should enforce elitism with ctrl(11) = 1.0)             \n");
  }
  
  if(*fdif < 0.33f && *irep != 3) {
    printf("WARNING: dangerously low value of fdif (ctrl(9))         \n");
  }
  
  if(*np % 2 > 0) {
    --(*np);
    
    printf("WARNING: decreasing population size (ctrl(1)) to np = %d \n", *np);
  }
  
  return(0);
} /* setctl */

/* ******************************************************************** */

/* Subroutine */ 

/* Write generation report to standard output         */

/* Input  - np,ifit(np), ivrb, ndim, n, nd, ig, nnew, */
/*          oldph(ndim, np), fitns(np), pmut          */
/* Output - none                                      */
/* Local  - ndpwr, k, rpt, bestft = 0, pmutpv = 0     */

int report(int *ivrb, int *ndim, int *n, int *np, int *nd, float *oldph, 
	   float *fitns, int *ifit, float *pmut, int *ig, int *nnew)
{
  /* Initialized data */
  
  static float bestft = 0.0f;
  static float pmutpv = 0.0f;
  
  /* System generated locals */
  
  int oldph_dim1, oldph_offset, i_1, i_2, i_3, i_4;
  float r_1, r_2, r_3;
  
  /* Builtin functions */
  
  double pow_ri(float *, int *);
  int i_nint(float *);

  /* Local variables */
  
  static int k;
  static int rpt;
  static int ndpwr;
  
  /* Parameter adjustments */
  
  --ifit;
  --fitns;
  oldph_dim1 = *ndim;
  oldph_offset = 1 + oldph_dim1 * 1;
  oldph -= oldph_offset;

  /* Function Body */
  
  rpt = FALSE_;
  
  if(*pmut != pmutpv) {
    pmutpv = *pmut;
    rpt = TRUE_;
  }
  
  if(fitns[ifit[*np]] != bestft) {
    bestft = fitns[ifit[*np]];
    rpt = TRUE_;
  }
  
  if(rpt || *ivrb >= 2) {
    /* Power of 10 to make int genotypes for display */
    
    r_1 = pow_ri(&c_b115, nd);
    ndpwr = i_nint(&r_1);
    
    printf("%d \n", *ig);
    printf("%d \n", *nnew);
    printf("%lf \n", *pmut);
    printf("%lf \n", fitns[ifit[*np]]);
    printf("%lf \n", fitns[ifit[*np - 1]]);
    printf("%lf \n", fitns[ifit[*np / 2]]);
    
    i_1 = *n;
	
    for(k = 1; k <= i_1; ++k) {
      r_1 = ndpwr * oldph[k + ifit[*np] * oldph_dim1];
      i_2 = i_nint(&r_1);
      
      printf("%d \n", i_2);
      
      r_2 = ndpwr * oldph[k + ifit[*np - 1] * oldph_dim1];
      i_3 = i_nint(&r_2);
      
      printf("%d \n", i_3);
      
      r_3 = ndpwr * oldph[k + ifit[*np / 2] * oldph_dim1];
      i_4 = i_nint(&r_3);
      
      printf("%d \n", i_4);
    }
  }
  
  return(0);
} /* report */

/* ********************************************************************** */
/*                         GENETICS MODULE                                */
/* ********************************************************************** */

/*     ENCODE:    encodes phenotype into genotype                         */
/*                called by: PIKAIA                                       */

/*     DECODE:    decodes genotype into phenotype                         */
/*                called by: PIKAIA                                       */

/*     CROSS:     Breeds two offspring from two parents                   */
/*                called by: PIKAIA                                       */

/*     MUTATE:    Introduces random mutation in a genotype                */
/*                called by: PIKAIA                                       */

/*     ADJMUT:    Implements variable mutation rate                       */
/*                called by: PIKAIA                                       */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     encode phenotype parameters into int genotype                      */
/*     ph(k) are x,y coordinates [ 0 < x, y < 1 ]                         */
/* ====================================================================== */

/* Inputs - n, nd, ph(n)    */
/* Output - gn(n*nd)        */
/* Local  - ip, i, j, ii, z */

int encode(int *n, int *nd, float *ph, int *gn)
{
  /* System generated locals */
  
  int i_1;
  
  /* Builtin functions */
  
  double pow_ri(float *, int *);
  
  /* Local variables */
  
  static int i_, j;
  static float z_;
  static int ii, ip;
  
  /* Parameter adjustments */
  
  --ph;
  --gn;
  
  /* Function Body */
  
  z_ = pow_ri(&c_b115, nd);
  ii = 0;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    ip = (int) (ph[i_] * z_);
	
    for(j = *nd; j >= 1; --j) {
      gn[ii + j] = ip % 10;
      ip /= 10;
    }
    
    ii += *nd;
  }
  
  return(0);
} /* encode */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     decode genotype into phenotype parameters                          */
/*     ph(k) are x,y coordinates [ 0 < x, y < 1 ]                         */
/* ====================================================================== */

/* Inputs - n, nd, gn(n*nd) */
/* Output - *ph(n)          */
/* Local  - ip, i, j, ii, z */

int decode(int *n, int *nd, int *gn, float *ph)
{
  /* System generated locals */
  
  int i_1, i_2;
  
  /* Builtin functions */
  
  double pow_ri(float *, int *);
  
  /* Local variables */
  
  static int i_, j;
  static float z_;
  static int ii, ip;
  
  /* Parameter adjustments */
  
  --ph;
  --gn;
  
  /* Function Body */
  
  i_1 = -(*nd);
  z_ = pow_ri(&c_b115, &i_1);
  ii = 0;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    ip = 0;
    i_2 = *nd;
    
    for(j = 1; j <= i_2; ++j) {
      ip = ip * 10 + gn[ii + j];
    }
    
    ph[i_] = ip * z_;
    ii += *nd;
  }
  
  return(0);
} /* decode */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     breeds two parent chromosomes into two offspring chromosomes       */
/*     breeding occurs through crossover. If the crossover probability    */
/*     test yields true (crossover taking place), either one-point or     */
/*     two-point crossover is used, with equal probabilities.             */

/*     Compatibility with version 1.0: To enforce 100% use of one-point   */
/*     crossover, un-comment appropriate line in source code below        */
/* ====================================================================== */

/* Inputs       - n, nd, pcross           */
/* Input/Output - gn1(n*nd), gn2(n*nd)    */
/* Local        - i, ispl, ispl2, itmp, t */
/* Function     - urand                   */

int cross(int *n, int *nd, float *pcross, int *gn1, int *gn2)
{
  /* System generated locals */
  
  int i_1;
  
  /* Local variables */
  
  static int i_, t, ispl, itmp, ispl2;
  
  /* Subroutine */
  
  extern double urand(void);

  /* Parameter adjustments */
  
  --gn2;
  --gn1;
  
  /* Function Body */
  /* Use crossover probability to decide whether a crossover occurs */

  if(urand() < *pcross) {
    /* Compute first crossover point */
    
    ispl = (int) (urand() * *n * *nd) + 1;

    /* Now choose between one-point and two-point crossover */
	
    if(urand() < 0.5f) {
      ispl2 = *n * *nd;
    } else {
      ispl2 = (int) (urand() * *n * *nd) + 1;
      
      /* Un-comment following line to enforce one-point crossover */
      
      // ispl2 = *n * *nd 
      
      if(ispl2 < ispl) {
	itmp = ispl2;
	ispl2 = ispl;
	ispl = itmp;
      }
    }
    
    /* Swap genes from ispl to ispl2 */
    
    i_1 = ispl2;
    
    for(i_ = ispl; i_ <= i_1; ++i_) {
      t = gn2[i_];
      gn2[i_] = gn1[i_];
      gn1[i_] = t;
    }
  }
  
  return(0);
} /* cross */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     Mutations occur at rate pmut at all gene loci                      */
/*        imut=1    Uniform mutation, constant rate                       */
/*        imut=2    Uniform mutation, variable rate based on fitness      */
/*        imut=3    Uniform mutation, variable rate based on distance     */
/*        imut=4    Uniform or creep mutation, constant rate              */
/*        imut=5    Uniform or creep mutation, variable rate based on     */
/*                  fitness                                               */
/*        imut=6    Uniform or creep mutation, variable rate based on     */
/*                  distance                                              */
/* ====================================================================== */

/* Input        - n, nd, imut, pmut           */
/* Input/Output - gn(n*nd)                    */
/* Local        - - i, j, k, l, ist, inc, loc */
/* Function     - urand                       */

int mutate(int *n, int *nd, float *pmut, int *gn, int *imut)
{
  /* System generated locals */
  
  int i_1, i_2, i_3;
  float r_1;
  
  /* Builtin functions */
  
  int i_nint(float *);
  
  /* Local variables */
  
  static int i_, j, k, l, inc, loc, ist;
  
  /* Subroutine */
  
  extern double urand(void);

  /* Parameter adjustments */
  --gn;

  /* Function Body */
  /* Decide which type of mutation is to occur */    
  
  if(*imut >= 4 && urand() <= 0.5f) {
    /* CREEP MUTATION OPERATOR */
    /* Subject each locus to random +/- 1 increment at the rate pmut */
    
    i_1 = *n;
    
    for(i_ = 1; i_ <= i_1; ++i_) {
      i_2 = *nd;
      
      for(j = 1; j <= i_2; ++j) {
	if (urand() < *pmut) {
	  /* Construct int */
	  
	  loc = (i_ - 1) * *nd + j;
	  r_1 = urand();
	  inc = (i_nint(&r_1) << 1) - 1;
	  ist = (i_ - 1) * *nd + 1;
	  gn[loc] += inc;
	  
	  /* printf("%d, %d, %d \n", ist, loc, inc); */

	  /* This is where we carry over the one (up to two digits) */
	  /* first take care of decrement below 0 case */

	  if(inc < 0 && gn[loc] < 0) {
	    if(j == 1) {
	      gn[loc] = 0;
	    } else {
	      i_3 = ist + 1;
		
	      for(k = loc; k >= i_3; --k) {
		gn[k] = 9;
		--gn[k - 1];
		
		if(gn[k - 1] >= 0) {
		  goto L4;
		}
	      }
	      
	      /* we popped under 0.00000 lower bound; fix it up */
	      
	      if((float) gn[ist] < 0.0f) {
		i_3 = loc;
		
		for(l = ist; l <= i_3; ++l) {
		  gn[l] = 0;
		}
	      }

	    L4:
	      
	      ;
	    }
	  }
	  
	  if(inc > 0 && gn[loc] > 9) {
	    if(j == 1) {
	      gn[loc] = 9;
	    } else {
	      i_3 = ist + 1;
	      
	      for(k = loc; k >= i_3; --k) {
		gn[k] = 0;
		++gn[k - 1];
		
		if(gn[k - 1] <= 9) {
		  goto L7;
		}
	      }
	      
	      /* we popped over 9.99999 upper bound; fix it up */
	      
	      if(gn[ist] > 9) {
		i_3 = loc;
		
		for(l = ist; l <= i_3; ++l) {
		  gn[l] = 9;
		}
	      }
	    
	    L7:
	      
	      ;
	    }
	  }
	}
      }
    }
  } else {
    /* UNIFORM MUTATION OPERATOR */
    /* Subject each locus to random mutation at the rate pmut */
	
    i_1 = *n * *nd;
    
    for(i_ = 1; i_ <= i_1; ++i_) {
      if(urand() < *pmut) {
	gn[i_] = (int) (urand() * 10.f);
      }
    }
  }
  
  return(0);
} /* mutate */

/* ********************************************************************** */
/* Subroutine */ 

/* ====================================================================== */
/*     dynamical adjustment of mutation rate;                             */
/*        imut=2 or imut=5 : adjustment based on fitness differential     */
/*                           between best and median individuals          */
/*        imut=3 or imut=6 : adjustment based on metric distance          */
/*                           between best and median individuals          */
/* ====================================================================== */

/* Input        - n, ndim, np, ifit(np), imut                  */
/*                oldph(ndim,np), fitns(np), pmutmn, pmutmx    */
/* Input/Output - pmut                                         */
/* Local        - i, rdiflo = 0.05, rdifhi = 0.25, delta = 1.5 */

int adjmut(int *ndim, int *n, int *np, float *oldph, float *fitns, int *ifit, 
	   float *pmutmn, float *pmutmx, float *pmut, int *imut)
{
  /* System generated locals */
  
  int oldph_dim1, oldph_offset, i_1;
  float r_1, r_2;
  
  /* Builtin functions */
  
  double sqrt(double);
  
  /* Local variables */
  
  static int i_;
  static float rdif;
  
  /* Parameter adjustments */
  
  --ifit;
  --fitns;
  oldph_dim1 = *ndim;
  oldph_offset = 1 + oldph_dim1 * 1;
  oldph -= oldph_offset;
  
  /* Function Body */
  
  if (*imut == 2 || *imut == 5) {
    /* Adjustment based on fitness differential */
    
    rdif = (r_1 = fitns[ifit[*np]] - fitns[ifit[*np / 2]], 
	    fabs(r_1)) / (fitns[ifit[*np]] + fitns[ifit[*np / 2]]);
  } else if(*imut == 3 || *imut == 6) {
    /* Adjustment based on normalized metric distance */
    rdif = 0.0f;
    i_1 = *n;
    
    for(i_ = 1; i_ <= i_1; ++i_) {
      /* Computing 2nd power */
      
      r_1 = oldph[i_ + ifit[*np] * oldph_dim1] 
	- oldph[i_ + ifit[*np / 2] * oldph_dim1];
      rdif += r_1 * r_1;
    }
    
    rdif = sqrt(rdif) / (float) (*n);
  }
  
  if(rdif <= 0.05f) {
    /* Computing MIN */
    
    r_1 = *pmutmx, r_2 = *pmut * 1.5f;
    *pmut = dmin(r_1, r_2);
  } else if(rdif >= 0.25f) {
    /* Computing MAX */
    
    r_1 = *pmutmn, r_2 = *pmut / 1.5f;
    *pmut = dmax(r_1, r_2);
  }
  return(0);
} /* adjmut */

/* ********************************************************************** */
/*                       REPRODUCTION MODULE */
/* ********************************************************************** */

/*     SELECT:   Parent selection by roulette wheel algorithm             */
/*               called by: PIKAIA                                        */

/*     RNKPOP:   Ranks initial population                                 */
/*               called by: PIKAIA, NEWPOP                                */

/*     GENREP:   Inserts offspring into population, for full              */
/*               generational replacement                                 */
/*               called by: PIKAIA                                        */

/*     STDREP:   Inserts offspring into population, for steady-state      */
/*               reproduction                                             */
/*               called by: PIKAIA                                        */
/*               calls:     FF                                            */

/*     NEWPOP:   Replaces old generation with new generation              */
/*               called by: PIKAIA                                        */
/*               calls:     FF, RNKPOP                                    */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     Selects a parent from the population, using roulette wheel         */
/*     algorithm with the relative fitnesses of the phenotypes as         */
/*     the "hit" probabilities [see Davis 1991, chap. 1].                 */
/* ====================================================================== */

/*     USES     - urand               */
/*     Input    - np, jfit(np), fdif  */
/*     Output   - idad                */
/*     Local    - np1, i, dice, rtfit */
/*     Function - urand               */

int select(int *np, int *jfit, float *fdif, int * idad)
{
  /* System generated locals */
  
  int i_1;
  
  /* Local variables */
  
  static int i_, np1;
  static float dice;
  static float rtfit;
  
  /* Subroutine */

  extern double urand(void);
  
  /* Parameter adjustments */
  
  --jfit;
  
  /* Function Body */
  
  np1 = *np + 1;
  dice = urand() * *np * np1;
  rtfit = 0.f;
  i_1 = *np;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    rtfit = rtfit + np1 + *fdif * (np1 - (jfit[i_] << 1));
    
    if(rtfit >= dice) {
      *idad = i_;
      goto L2;
    }
  }

  /* Assert: loop will never exit by falling through */

 L2:
  
  return(0);
} /* select */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     Calls external sort routine to produce key index and rank order    */
/*     of input array arrin (which is not altered).                       */
/* ====================================================================== */

/* USES                     - rqsort           */
/* Input                    - narrin(n)        */
/* Output                   - indx(n), rank(n) */
/* Local                    - i                */
/* External sort subroutine - rqsort           */

int rnkpop(int *n, float *arrin, int *indx, int *rank)
{
  /* System generated locals */
  
  int i_1;
  
  /* Local variables */
  
  static int i_;
  
  /* Subroutine */ 
  
  extern int rqsort(int *, float *, int *);
  
  /* Parameter adjustments */
  
  --rank;
  --indx;
  --arrin;
  
  /* Function Body */
  /* Compute the key index */
  
  rqsort(n, &arrin[1], &indx[1]);
  
  /* ... and the rank order */
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    rank[indx[i_]] = *n - i_ + 1;
  }
  
  return(0);
} /* rnkpop */

/* *********************************************************************** */

/* Subroutine */ 

/* ======================================================================= */
/*     full generational replacement: accumulate offspring into new        */
/*     population array                                                    */
/* ======================================================================= */

/* Input  - ndim, n, np, ipph(ndim, 2) */
/* Output - newph(ndim, np)            */
/* Local  - i1, i2, k                  */

int genrep(int *ndim, int *n, int *np, int *ip, float *ph, float *newph)
{
  /* System generated locals */
  
  int ph_dim1, ph_offset, newph_dim1, newph_offset, i_1;
  
  /* Local variables */
  
  static int k, i1, i2;
  
  /* Parameter adjustments */
  
  ph_dim1 = *ndim;
  ph_offset = 1 + ph_dim1 * 1;
  ph -= ph_offset;
  newph_dim1 = *ndim;
  newph_offset = 1 + newph_dim1 * 1;
  newph -= newph_offset;

  /* Function Body */
  /* Insert one offspring pair into new population */
  
  i1 = (*ip << 1) - 1;
  i2 = i1 + 1;
  i_1 = *n;
  
  for(k = 1; k <= i_1; ++k) {
    newph[k + i1 * newph_dim1] = ph[k + ph_dim1];
    newph[k + i2 * newph_dim1] = ph[k + (ph_dim1 << 1)];
  }
  
  return(0);
} /* genrep */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     steady-state reproduction: insert offspring pair into population   */
/*     only if they are fit enough (replace-random if irep=2 or           */
/*     replace-worst if irep=3).                                          */
/* ====================================================================== */

/* USES              - ff, urand                                      */
/* Input             - ndim, n, np, irep, ielite, ff, ph(ndim, 2)     */
/* Input/Output      - oldph(ndim, np), fitns(np), ifit(np), jfit(np) */
/* Output            - nnew                                           */
/* Local             - i, j, k, i1, if1, fit                          */
/* External function - urand                                          */

int stdrep(double (*ff)(), int *ndim, int *n, int *np, int *irep, int *ielite, 
	   float *ph, float *oldph, float *fitns, int *ifit, int *jfit, 
	   int *nnew)
{
  /* System generated locals */
  
  int ph_dim1, ph_offset, oldph_dim1, oldph_offset, i_1;
  
  /* Local variables */
  
  static int i_, j, k, i1, if1;
  static float fit;

  /* Subroutine */
  
  extern double urand(void);
  
  /* Parameter adjustments */
  
  ph_dim1 = *ndim;
  ph_offset = 1 + ph_dim1 * 1;
  ph -= ph_offset;
  --jfit;
  --ifit;
  --fitns;
  oldph_dim1 = *ndim;
  oldph_offset = 1 + oldph_dim1 * 1;
  oldph -= oldph_offset;

  /* Function Body */
  
  *nnew = 0;
  
  for(j = 1; j <= 2; ++j) {
    /* 1. compute offspring fitness (with caller's fitness function) */
    
    fit = (*ff)(n, &ph[j * ph_dim1 + 1]);
    
    /* 2. if fit enough, insert in population */
	
    for(i_ = *np; i_ >= 1; --i_) {
      if(fit > fitns[ifit[i_]]) {
	/* make sure the phenotype is not already in the population */
	
	if(i_ < *np) {
	  i_1 = *n;
		
	  for(k = 1; k <= i_1; ++k) {
	    if(oldph[k + ifit[i_ + 1] * oldph_dim1] != ph[k + j * ph_dim1]) {
	      goto L6;
	    }
	  }
	  
	  goto L1;
	
	L6:
	  
	  ;
	}

	/* offspring is fit enough for insertion, and is unique */
	/* (i) insert phenotype at appropriate place in population */
		
	if(*irep == 3) {
	  i1 = 1;
	} else if(*ielite == 0 || i_ == *np) {
	  i1 = (int) (urand() * *np) + 1;
	} else {
	  i1 = (int) (urand() * (*np - 1)) + 1;
	}
		
	if1 = ifit[i1];
	fitns[if1] = fit;
	i_1 = *n;
	
	for(k = 1; k <= i_1; ++k) {
	  oldph[k + if1 * oldph_dim1] = ph[k + j * ph_dim1];
	}
	
	/* (ii) shift and update ranking arrays */
	
	if(i_ < i1) {
	  /* shift up */
	  
	  jfit[if1] = *np - i_;
	  i_1 = i_ + 1;
	  
	  for(k = i1 - 1; k >= i_1; --k) {
	    --jfit[ifit[k]];
	    ifit[k + 1] = ifit[k];
	  }
	  
	  ifit[i_ + 1] = if1;
	} else {
	  /* shift down */
	  
	  jfit[if1] = *np - i_ + 1;
	  i_1 = i_;
	  
	  for(k = i1 + 1; k <= i_1; ++k) {
	    ++jfit[ifit[k]];
	    ifit[k - 1] = ifit[k];
	    
	    /* L23: */
	  }
	
	  ifit[i_] = if1;
	}
	
	++(*nnew);
	goto L1;
      }
    }
    
  L1:
    
    ;
  }
  
  return(0);
} /* stdrep */

/* ********************************************************************** */

/* Subroutine */ 

/* ====================================================================== */
/*     replaces old population by new; recomputes fitnesses & ranks       */
/* ====================================================================== */

/* USES:        - ff, rnkpop                          */
/* Input        - ndim, np, n, ielite, ff             */
/* Input/Output - oldph(ndim, np), newph(ndim, np)    */
/* Output       - ifit(np), jfit(np), nnew, fitns(np) */
/* Local        - i, k                                */

int newpop(double (*ff)(), int *ielite, int *ndim, int *n, int *np, 
	   float *oldph, float *newph, int *ifit, int *jfit, float *fitns, 
	   int *nnew)
{
  /* System generated locals */
  
  int oldph_dim1, oldph_offset, newph_dim1, newph_offset, i_1, i_2;
  
  /* Local variables */
  
  static int i_, k;
  
  /* Subroutine */ 

  extern int rnkpop(int *, float *, int *, int *);
  
  /* Parameter adjustments */
  
  --fitns;
  --jfit;
  --ifit;
  newph_dim1 = *ndim;
  newph_offset = 1 + newph_dim1 * 1;
  newph -= newph_offset;
  oldph_dim1 = *ndim;
  oldph_offset = 1 + oldph_dim1 * 1;
  oldph -= oldph_offset;
  
  /* Function Body */
  
  *nnew = *np;
  
  /* if using elitism, introduce in new population fittest of old */
  /* population (if greater than fitness of the individual it is  */
  /* to replace)                                                  */
  
  if(*ielite == 1 && (*ff)(n, &newph[newph_dim1 + 1]) < fitns[ifit[*np]]) {
    i_1 = *n;
    
    for(k = 1; k <= i_1; ++k) {
      newph[k + newph_dim1] = oldph[k + ifit[*np] * oldph_dim1];
    }
	
    --(*nnew);
  }

  /* replace population */
    
  i_1 = *np;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    i_2 = *n;
	
    for(k = 1; k <= i_2; ++k) {
      oldph[k + i_ * oldph_dim1] = newph[k + i_ * newph_dim1];
    }

    /* get fitness using caller's fitness function */
    
    fitns[i_] = (*ff)(n, &oldph[i_ * oldph_dim1 + 1]);
  }
  
  /*     compute new population fitness rank order */
  
  rnkpop(np, &fitns[1], &ifit[1], &jfit[1]);
  
  return(0);
} /* newpop */

double pow_ri(float *ap, int *bp) 
{
  double pow, x;
  int n;
  unsigned long u;
  
  pow = 1;
  x = *ap;
  n = *bp;
  
  if(n != 0) {
    if(n < 0) {
      n = -n;
      x = 1/x;
    }
    
    for(u = n; ; ) {
      if(u & 01)
	pow *= x;
      if(u >>= 1)
	x *= x;
      else
	break;
    }
  }
  
  return(pow);
}

int i_nint(float *x)
{
  return((int)(*x >= 0 ? floor(*x + 0.5) : -floor(0.5 - *x)));
}

