#include <math.h>
#include <stdio.h>
#include <string.h>

#define XINC          1.0E-10
#define Epsilon       1.0E-20

#include "Simpson.c"  

/*
  
#include "Runge_Kutta.c"
#include "Integration.c" 

**/

/* Following functions are defined in math.h:

   log1p(x) = log(1.0 + x)
   hypot(x, y) = sqrt(x * x + y * y)

**/

/*

#define M_E             2.7182818284590452354
#define M_LOG2E         1.4426950408889634074
#define M_LOG10E        0.43429448190325182765
#define M_LN2           0.69314718055994530942
#define M_LN10          2.30258509299404568402
#define M_PI            3.14159265358979323846
#define M_PI_2          1.57079632679489661923
#define M_PI_4          0.78539816339744830962
#define M_1_PI          0.31830988618379067154
#define M_2_PI          0.63661977236758134308
#define M_2_SQRTPI      1.12837916709551257390
#define M_SQRT2         1.41421356237309504880
#define M_SQRT1_2       0.70710678118654752440  

**/  

double DeltaT_geom(VEC x); 
double Potential(double x, double y);
void draw_contour_td();

/*

extern void cntr(int *numx, int *numy, double *xmin, double *xmax, 
		 double *ymin, double *ymax, double *val, int *nvals, 
		 int *nchars, double *title, int *gridp);
  
**/

/* double square(double x) { return x * x; } */

double DeltaT_geom(VEC x)
{
  VEC x_new, y_new;
  double detA, DeltaT;

  x_new[0] = A1 * x[0] + D  * x[1];
  x_new[1] = D  * x[0] + A2 * x[1];

  detA = square(1.0 - parms[P_SIGMA]) - parms[P_GAMMA] * parms[P_GAMMA];

  y_new[0] = (  A2 * Y[0] - D  * Y[1]) / detA;
  y_new[1] = (- D  * Y[0] + A1 * Y[1]) / detA;
    
  DeltaT  = x_new[0] * x[0]     + x_new[1] * x[1];
  DeltaT -= x_new[0] * y_new[0] + x_new[1] * y_new[1];
  DeltaT -= Y[0]     * x[0]     + Y[1]     * x[1];
  DeltaT += y_new[0] * Y[0]     + y_new[1] * Y[1];
  DeltaT /= 2.0;
  
  return DeltaT;      
}

double Potential(double x, double y)
{
  /* int k; */

  double hs;
  VEC x_rot; 

  double Psi = 0.0, DeltaT, s2, radio, rad0, rad2, MDipol, MQuadrupol, RoT; 
  double Epsilon1 = 1.0 - parms[P_EPSILON];
  double Epsilon2 = 1.0 + parms[P_EPSILON];

  extern double TFunc();

  x_rot[0] =   x * cs + y * ss;
  x_rot[1] = - x * ss + y * cs; 

  radio = x_rot[0] * x_rot[0] + x_rot[1] * x_rot[1];
  rad2  = (x_rot[0] * x_rot[0] + x_rot[1] * x_rot[1]) / R_E2;
  rad0  = sqrt(rad2);

  if(rad2 == 0.0) {
    radio += Epsilon;
    rad2  += Epsilon; 
    rad0  = sqrt(rad2);
  }     
  
  switch(current_model) {
  case M_CHANG_REFSDAL:
    Psi = log(rad2) / 2.0;
    break;   
  case M_SING_ISOT_SPHERE:
    if(parms[P_EXP] != 0.0) {
      Psi = pow(rad2, parms[P_EXP] / 2.0) / parms[P_EXP];
    } else {
      Psi = log(rad2) / 2.0;
    }
    
    break;
  case M_NSING_ISOT_SPHERE:
    s2   = xc2 + rad2;
    Psi  = sqrt(s2) - xc + xc * log((2.0 * xc)/(xc + sqrt(s2))); 
    Psi *= parms[P_KAPPA]; 
    break;
  case M_TRANSP_SPHERE:
    Psi = 0.0;
    break;
  case M_ELLIPTICAL:
    s2 = xc2 + (Epsilon1 * x_rot[0] * x_rot[0]
                + Epsilon2 * x_rot[1] * x_rot[1]) / R_E2;
    
    if(parms[P_ALPHA] != 0.0) {
      Psi  = (pow((s2 / xc2), parms[P_ALPHA]) - 1.0) / (2.0 * parms[P_ALPHA]);
      Psi *= parms[P_KAPPA];        
    }else{
      Psi = parms[P_KAPPA] * log(s2 / xc2) / 2.0;
    }     
    
    break;
  case M_KING:    
    /* There is no analytical expression for the Potential of this Model */

    hs = (rad0 - XINC) / ((double) Nint);

    Psi = - parms[P_KAPPA] * Simpson(XINC, rad0, hs, (int) Nint, TFunc);   

    // Psi = - (parms[P_KAPPA] * DiLog(- rad2)) / 2.0;    
    
    break;
  case M_TRUNC_KING:
    /* There is no analytical expression for the Potential of this Model */

    /* Psi = 4.0 * parms[P_KAPPA2] *
       ((1.0 - sqrt(1.0 + radio)
       + log((1.0 + sqrt(1.0 + radio)) / 2.0)) / sqrt(1.0 + R_C2)
       + radio / (8.0 * (1.0 + R_C2)) - DiLog(- radio) / 8.0); */

    /* Psi = parms[P_KAPPA2] *
       ((4.0 * (1.0 - hypot(1.0, rad0)        
	       + log1p(hypot(1.0, rad0)) - M_LN2)) / hypot(1.0, xc)
	       + rad2 / (2.0 * (1.0 + xc2)) - DiLog(- rad2) / 2.0); */

    Psi = 0.0;
    break;
  case M_HUBBLE:
    /* There is no analytical expression for the Potential of this Model */

    /* Psi = 2.0 * parms[P_KAPPA] * (Simpson(XINC, radio)
             - log(1.0 + radio)); */

    hs = (rad2 - XINC) / ((double) Nint);

    Psi = 2.0 * parms[P_KAPPA] * (Simpson(XINC, rad2, hs, (int) Nint, TFunc) 
				  - log1p(rad2)); 
    break;
  case M_DE_VAUCOULEUR:
    /* There is no analytical expression for the Potential of this Model */
    
    Psi = 0.0;
    break;
  case M_SPIRAL:
    /* There is no analytical expression for the Potential of this Model */

    /* Psi = 2.0 * parms[P_KAPPA] * Integration_F(rad0); */

    hs = (rad2 - XINC) / ((double) Nint);

    Psi = 2.0 * parms[P_KAPPA] * 
      Simpson(XINC, rad2, hs, (int) Nint, TFunc) / (parms[P_ALPHA2] 
						    * parms[P_ALPHA2]); 
    break;
  case M_MULTIPOLE:
    MDipol     = (parms[P_E] * (parms[P_DX_QP] * x_rot[0]
                                + parms[P_DY_QP] * x_rot[1])) / radio;
    MQuadrupol = (R_E2 *
                  (parms[P_QXX] * x_rot[0] * x_rot[0]
                   + 2.0 * parms[P_QXY] * x_rot[0] * x_rot[1]
                   + parms[P_QYY] * x_rot[1] * x_rot[1])) / square(radio);
    
    Psi = parms[P_KAPPA0] * log(rad2) / 2.0 - (MDipol + MQuadrupol);
    break; 
  case M_ROTATION_LENS:
    RoT = (parms[P_SX] * x_rot[0] - parms[P_SY] * x_rot[1]) / parms[P_E];

    Psi = log(rad2) / 2.0 + 4.0 * RoT / rad2;
    break;
  case M_DOUBLE_LENS:
    Psi = 0.0;
    break;
  case M_UNIFORM_RING:
    Psi = 0.0;
    break; 
  }
      
  DeltaT = DeltaT_geom(x_rot) - R_E2 * Psi; 

  return(DeltaT);
}

void draw_contour_td()
{
  int i, j, kc = 50;
  double x, y, d[Nd];

  for(i = 0; i < kc; ++i) {
    for(j = 0; j < kc; ++j) {
      x = ((double) i);
      y = ((double) j);

      d[i + j * Np] = Potential(x, y);
    }
  }

  cntr(&c_M, &c_N, &c_10, &c_12, &c_10, &c_12, 
       &d[0], &c_3, &c_0, &c_b11, &c_0);  
}

