/* Powell22.f -- translated by f2c (version 20020621). */

#include<stdio.h>
#include<stdlib.h>

/* Common Block Declarations */

#define M  20
#define N  50

struct {
  int ncom;
  float pcom[N], xicom[N];
} fcom_0;

#define fcom fcom_0

#define abs(x)     ((x) >= 0 ? (x) : - (x))
#define dabs(x)    ((double) abs(x))
#define min(a, b)  ((a) <= (b) ? (a) : (b))
#define max(a, b)  ((a) >= (b) ? (a) : (b))
#define dmin(a, b) ((double) min(a, b))
#define dmax(a, b) ((double) max(a, b))

/* Table of constant values */

static float c_b13 = 1e-4f;
static float c_b19 = 1.0f;

/* Subroutine                                            */ 

/* Powell's Method for minimation of a given function    */
/* FUNC(X), where X(N), N max NMAX                       */
/* from Numerical Recipes, p.298ff., rutine POWELL       */
/* including subroutines LINMIN, FDIM, MNBRAK and BRENT  */

/* slightly changed by R.Kayser, Feb-01-1993             */
/* PIN(N) is now only the start vector (input)           */
/* P(N)   is the actual vector (output)                  */
/* ITMAX  is the maximum number of iterations (input)    */

int powell(float *pin, float *xi, int *n, int *np, float *ftol, int *itmax, 
	   float *p, float *fret)
{
    /* System generated locals */

    int xi_dim, xi_offset, i_1, i_2;
    float r_1, r_2;

    /* Builtin functions */

    /* Local variables */

    static int i, j;
    static float t, fp, pt[M], del, xit[M], ptt[M];
    static int ibig;
    static int iter;
    static float fptt;

    /* Subroutines */ 

    extern int linmin(float *, float *, int *, float *);
    extern double func(float *);

    /* Fortran I/O blocks */

    /* Parameter adjustments */

    --p;
    xi_dim = *np;
    xi_offset = 1 + xi_dim * 1;
    xi -= xi_offset;
    --pin;
    
    /* Function Body */
    
    i_1 = *n;
    
    for(i = 1; i <= i_1; ++i) {
      p[i] = pin[i];
    }
    
    *fret = func(&p[1]);
    i_1 = *n;
    
    for(j = 1; j <= i_1; ++j) {
      pt[j - 1] = p[j];

      /* L11: */
    }
    
    iter = 0;

 L1:

    ++iter;
    fp = *fret;
    ibig = 0;
    del = 0.0f;
    i_1 = *n;
    
    for(i = 1; i <= i_1; ++i) {
      i_2 = *n;
	
      for(j = 1; j <= i_2; ++j) {
	xit[j - 1] = xi[j + i * xi_dim];
	
	/* L12: */
      }
	
      linmin(&p[1], xit, n, fret);
      
      if((r_1 = fp - *fret, dabs(r_1)) > del) {
	del = (r_1 = fp - *fret, dabs(r_1));
	ibig = i;
      }
      
      /* L13: */
    }
    
    if((r_1 = fp - *fret, dabs(r_1)) * 2.0f <= *ftol * 
       (dabs(fp) + dabs(*fret))) {
      return(0);
    }
    
    if(iter == *itmax) {
      printf("POWELL exceeding maximum iterations. \n");
    }
    
    i_1 = *n;
    
    for(j = 1; j <= i_1; ++j) {
      ptt[j - 1] = p[j] * 2.0f - pt[j - 1];
      xit[j - 1] = p[j] - pt[j - 1];
      pt[j - 1] = p[j];

      /* L14: */
    }
    
    fptt = func(ptt);
    
    if(fptt >= fp) {
      goto L1;
    }

    /* Computing 2nd power */
    
    r_1 = fp - *fret - del;

    /* Computing 2nd power */
    
    r_2 = fp - fptt;
    t = (fp - *fret * 2.0f + fptt) * 2.0f * (r_1 * r_1) - del * (r_2 * r_2);
    
    if(t >= 0.0f) {
      goto L1;
    }
    
    linmin(&p[1], xit, n, fret);
    
    i_1 = *n;
    
    for(j = 1; j <= i_1; ++j) {
      xi[j + ibig * xi_dim] = xit[j - 1];
      
      /* L15: */
    }
    
    goto L1;
} /* powell */

/* Subroutine used by POWELL */

int linmin(float *p, float *xi, int *n, float *fret)
{
  /* System generated locals */
  
  int i_0;
  
  /* Local variables */
  
  static int j;
  static float fa, fb, ax, bx, fx, xx, xmin;
  
  /* Subroutines */ 
  
  extern double fdim();
  extern double brent(float *, float *, float *, double (*f)(), 
		      float *, float *);
  extern int mnbrak(float *, float *, float *, float *, float *, 
		    float *, double (*f)());
  
  /* Parameter adjustments */
  
  --xi;
  --p;
  
  /* Function Body */
  
  fcom.ncom = *n;
  i_0 = *n;
  
  for(j = 1; j <= i_0; ++j) {
    fcom.pcom[j - 1] = p[j];
    fcom.xicom[j - 1] = xi[j];

    /* L11: */
  }
  
  ax = 0.0f;
  xx = 1.0f;
  bx = 2.0f;
  
  mnbrak(&ax, &xx, &bx, &fa, &fx, &fb, fdim);
  
  *fret = brent(&ax, &xx, &bx, fdim, &c_b13, &xmin);
  
  i_0 = *n;
  
  for(j = 1; j <= i_0; ++j) {
    xi[j] = xmin * xi[j];
    p[j] += xi[j];
    
    /* L12: */
  }
  
  return(0);
} /* linmin */

/* Subroutine used by POWELL */

double fdim(float *x)
{
  /* System generated locals */
  
  int i_0;
  float ret_val;
  
  /* Local variables */
  
  static int j;
  static float xt[N];
  
  /* Subroutine */
  
  extern double func(float *);
  
  i_0 = fcom.ncom;
  
  for(j = 1; j <= i_0; ++j) {
    xt[j - 1] = fcom.pcom[j - 1] + *x * fcom.xicom[j - 1];

    /* L11: */
  }
  
  ret_val = func(xt);
  
  return(ret_val);
} /* fdim */

/* Subroutine used by POWELL */

int mnbrak(float *ax, float *bx, float *cx, float *fa, float *fb,
	   float *fc, double (*func)())
{
  /* System generated locals */
    
  float r_1, r_2, r_3, r_4;
  
  /* Builtin functions */

  double r_sign(float *, float *);
  
  /* Local variables */
  
  static float q, r, u, fu, dum, ulim;
  
  *fa = (*func)(ax);
  *fb = (*func)(bx);
  
  if(*fb > *fa) {
    dum = *ax;
    *ax = *bx;
    *bx = dum;
    dum = *fb;
    *fb = *fa;
    *fa = dum;
  }
  
  *cx = *bx + (*bx - *ax) * 1.618034f;
  *fc = (*func)(cx);
 
 L1:

  if(*fb >= *fc) {
    r = (*bx - *ax) * (*fb - *fc);
    q = (*bx - *cx) * (*fb - *fa);

    /* Computing MAX */
    
    r_3 = (r_1 = q - r, dabs(r_1));
    r_2 = dmax(r_3, 1e-20f);
    r_4 = q - r;
    u = *bx 
      - ((*bx - *cx) * q - (*bx - *ax) * r) / (r_sign(&r_2, &r_4) * 2.0f);
    
    if((r_1 = *cx - *bx, dabs(r_1)) > 5e7f) {
      r_1 = *cx - *bx;
      ulim = r_sign(&c_b19, &r_1) * 1e10f;
    } else if (*bx > 5e9f) {
      ulim = 1e10f;
    } else {
      ulim = *bx + (*cx - *bx) * 100.0f;
    }
    
    r_1 = *bx - u;
    r_2 = u - *cx;
    
    if(r_sign(&c_b19, &r_1) * r_sign(&c_b19, &r_2) > 0.0f) {
      fu = (*func)(&u);
      
      if(fu < *fc) {
	*ax = *bx;
	*fa = *fb;
	*bx = u;
	*fb = fu;
	goto L1;
      } else if(fu > *fb) {
	*cx = u;
	*fc = fu;
	goto L1;
      }
      
      u = *cx + (*cx - *bx) * 1.618034f;
      fu = (*func)(&u);
    } else {
      /* if(complicated condition) */ 

      r_1 = *cx - u;
      r_2 = u - ulim;
      
      if(r_sign(&c_b19, &r_1) * r_sign(&c_b19, &r_2) > 0.0f) {
	fu = (*func)(&u);
	
	if(fu < *fc) {
	  *bx = *cx;
	  *cx = u;
	  u = *cx + (*cx - *bx) * 1.618034f;
	  *fb = *fc;
	  *fc = fu;
	  fu = (*func)(&u);
	}
      } else { 
	/* if(complicated condition) */ 
	
	r_1 = u - ulim;
	r_2 = ulim - *cx;
	
	if(r_sign(&c_b19, &r_1) * r_sign(&c_b19, &r_2) >= 0.0f) {
	  u = ulim;
	  fu = (*func)(&u);
	} else {
	  u = *cx + (*cx - *bx) * 1.618034f;
	  fu = (*func)(&u);
	}
      }
    }
    
    *ax = *bx;
    *bx = *cx;
    *cx = u;
    *fa = *fb;
    *fb = *fc;
    *fc = fu;

    goto L1;
  }
  
  return(0);
} /* mnbrak */

/* Subroutine used by POWELL */

double brent(float *ax, float *bx, float *cx, double (*f)(), float *tol, 
	     float *xmin)
{
  /* System generated locals */
  
  float ret_val, r_0;
  
  /* Builtin functions */
  
  double r_sign(float *, float *);

  /* Local variables */
  
  static float a, b, d, e, p, q, r, u, v, w, x;
  static float fv, fw, fx, fu, xm, tol1, tol2;
  static int iter;
  static float etemp;
  
  /* Fortran I/O blocks */
  
  a = dmin(*ax, *cx);
  b = dmax(*ax, *cx);
  v = *bx;
  w = v;
  x = v;
  e = 0.0f;
  fx = (*f)(&x);
  fv = fx;
  fw = fx;
  
  for(iter = 1; iter <= 100; ++iter) {
    xm = (a + b) * 0.5f;
    tol1 = *tol * dabs(x) + 1e-10f;
    tol2 = tol1 * 2.0f;
    
    if((r_0 = x - xm, dabs(r_0)) <= tol2 - (b - a) * 0.5f) {
      goto L3;
    }
	
    if(dabs(e) > tol1) {
      r = (x - w) * (fx - fv);
      q = (x - v) * (fx - fw);
      p = (x - v) * q - (x - w) * r;
      q = (q - r) * 2.0f;
      
      if(q > 0.0f) {
	p = - p;
      }
      
      q = dabs(q);
      etemp = e;
      e = d;
	
      if(dabs(p) >= (r_0 = q * 0.5f * etemp, dabs(r_0)) || 
	 p <= q * (a - x) || p >= q * (b - x)) {
	goto L1;
      }
      
      d = p / q;
      u = x + d;
      
      if(u - a < tol2 || b - u < tol2) {
	r_0 = xm - x;
	d = r_sign(&tol1, &r_0);
      }

      goto L2;
    }

  L1:
    
    if(x >= xm) {
      e = a - x;
    } else {
      e = b - x;
    }
    
    d = e * 0.381966f;
    
  L2:
    
    if(dabs(d) >= tol1) {
      u = x + d;
    } else {
      u = x + r_sign(&tol1, &d);
    }
    
    fu = (*f)(&u);
    
    if(fu <= fx) {
      if(u >= x) {
	a = x;
      } else {
	b = x;
      }
      
      v = w;
      fv = fw;
      w = x;
      fw = fx;
      x = u;
      fx = fu;
    } else {
      if(u < x) {
	a = u;
      } else {
	b = u;
      }
      
      if(fu <= fw || w == x) {
	v = w;
	fv = fw;
	w = u;
	fw = fu;
      } else if(fu <= fv || v == x || v == w) {
	v = u;
	fv = fu;
      }
    }
    
    /* L11: */
  }
  
  printf("BRENT exceed maximum iterations. \n");
  
 L3:
  
  *xmin = x;
  ret_val = fx;
  
  return(ret_val);
} /* brent */

/* dummy function for POWELL to be replaced by user-supplied modul */

double funct(float *x)
{
  /* System generated locals */
  
  float ret_val;
  
  /* Parameter adjustments */
  
  --x;
  
  /* Function Body */
  
  ret_val = x[1];
  
  return(ret_val);
} /* funct */

double r_sign(float *a, float *b)
{
  double x;
  
  x = (*a >= 0 ? *a : - *a);
  
  return(*b >= 0 ? x : - x);
}

