/* Praxis.c */

#include<stdio.h>
#include<math.h>

#define N    400
#define N1    20
#define N2    21
#define N3    50
#define N4   127

#define a(i, j, l, n)  (int)((i) + (l) * (j) - n)

/* Common Block Declarations */

/* v was v[20][20] */

struct {
  double v[N], x[N1], d[N1], q0[N1], q1[N1], dmin, epsmch, fx, h;
  double qd0, qd1, qf1, small, t, xldt, xm2, xm4, dseed, scbd;
  int n, nl, nf, lp, jprint, nmax, illcin, ktm, nfmax, jranch;
} cpraxis;

#define cprax cpraxis

struct {
    double pi;
    int jfunc, maxjf;
} prtests;

#define prtest prtests

struct {
  double ran3[N4], ran1;
  int jran2;
} comrans;

#define comran comrans

/* Table of constant values */

static int c0        = 0;
static int c1        = 1;
static int c2        = 2;
static int c4        = 4;

static double c_b22  =          2.0;
static double c_b83  =          4.0;
static double c_b310 =       8190.0;
static double c_b313 =       8191.0;
static double c_b314 =         32.0;
static double c_b318 = 2147483647.0;

/* Subroutines */

extern double ftest();
extern int praset(), testin(), praxis();
extern int ranini();
extern int prandom();
extern int minfit();
extern int min();
extern int flin();
extern int pquad();
extern int sort();
extern double dmod();
extern double powd();
extern double drandm();
extern double model();

/*
  
extern void lens_model_praxis(double *x, int i);
extern void choice(double *x);
extern void choice_initial();
extern void choice_restrict();

**/

// int main_praxis();

void main_praxis();

#include "lens_model_praxis.c"
#include "choice.c"

/*  PRAXIS.CNTL          PRAXIS 1.2     SEPTEMBER 1995                       */

/*  RECENT CHANGES...                                                        */
/*     ERROR CORRECTED IN FUNCTION FTEST (SEE BELOW) 25 SEPT. 1995           */
/*     IMPLICIT STATEMENTS REMOVED AND                                       */
/*        DOCUMENTATION ADDED TO SUBROUTINE TESTIN 26 SEPT. 1995             */

/*  MAIN PROGRAM TO TEST SUBROUTINE PRAXIS BY BRENT                          */

/*  J. P. CHANDLER, COMPUTER SCIENCE DEPARTMENT,                             */
/*     OKLAHOMA STATE UNIVERSITY, STILLWATER, OK 74078                       */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES",                       */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGE 138                           */

/* Main program */

// int main_praxis()

void main_praxis()
{
  printf("Solve optimization problem using the Brent Method. \n");

  praset();
  
  cprax.jprint = 1;

  cprax.nfmax = 3000;

  cprax.h = 1.0;      
  cprax.t = 1.0e-15;

  cprax.scbd = 1.0;

  cprax.illcin = 0;

  cprax.ktm = 1;

  choice_initial();

  praxis(model);

  choice_final();
  
  /*  END MAIN (PRAXIS TEST DRIVER) */

  // return 0;
}

/* Model */

double model(double *x, int *n)
{
  int i;    
  double function = 0.0;
  
  for(i = 0; i < Number_of_images; i++) {
    lens_model_praxis(x, i);
  }

  // restrictions(Xprmt);

  function = restrict_func(x);
  
  return function;
}

/* Subroutine */

/*  INITIALIZE FOR PROBLEM NUMBER JFUNC FROM BRENT'S TEST SUITE.             */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES",                       */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGE 138                           */

/*  CALL SUBROUTINE PRASET BEFORE CALLING TESTIN.                            */
/*  IN PARTICULAR,                                                           */
/*  H MUST HAVE BEEN SET TO 1.0 BEFORE CALLING TESTIN,                       */
/*  ILLCIN MUST HAVE BEEN SET TO 0,                                          */
/*  KTM MUST HAVE BEEN SET TO 1,                                             */
/*  AND A CALL RANINI(4.0D0) MUST HAVE BEEN MADE.                            */

/*  NOTE THAT FOR JFUNC = 6, 7, 10, OR 11,                                   */
/*  THE VALUE OF N MUST BE SET BEFORE CALLING TESTIN.                        */

int testin()
{
  /* Local variables */

  static int j;
  static double enplu;

  prtest.maxjf = 12;

  prtest.pi = 4.0 * atan(1.0);

  if(prtest.jfunc < 1 || prtest.jfunc > prtest.maxjf) {
    return 0;
  }

  switch(((int) prtest.jfunc)) {
  case 1:
    /*  JFUNC = 1 */
    /*  ROSENBROCK'S TEST FUNCTION */

    /*     THE MINIMUM IS F(1.0, 1.0) = 0.0 . */

    cprax.n = 2;

    cprax.x[0] = - 1.2;
    cprax.x[1] =   1.0;

    printf("ROSENBROCK TEST FUNCTION \n");

    break;
  case 2:
    /*  JFUNC = 2 */
    /*  POWELL'S SINGULAR TEST FUNCTION */

    /*     THE MINIMUM IS F(0.0, 0.0, 0.0, 0.0) = 0.0 . */

    cprax.illcin = 1;

    ranini(&c_b22);

    cprax.n = 4;
    
    cprax.x[0] =   3.0;
    cprax.x[1] = - 1.0;
    cprax.x[2] =   0.0;
    cprax.x[3] =   1.0;

    printf("SINGULAR TEST FUNCTION OF POWELL \n");

    break;
  case 3:
    /*  JFUNC = 3 */
    /*  HELICAL VALLEY TEST FUNCTION OF FLETCHER AND POWELL */

    /*     THE MINIMUM IS F(1.0, 0.0, 0.0) = 0.0 . */

    cprax.n = 3;
    
    cprax.x[0] = - 1.0;
    cprax.x[1] =   0.0;
    cprax.x[2] =   0.0;

    printf("HELICAL VALLEY TEST FUNCTION OF FLETCHER AND POWELL \n ");
  
    break;
  case 4:
    /*  JFUNC = 4 */
    /*  LEON'S CUBIC TEST FUNCTION */

    /*     THE MINIMUM IS F(1.0, 1.0) = 0.0 . */

    cprax.n = 2;
    
    cprax.x[0] = - 1.2;
    cprax.x[1] = - 1.0;

    printf("CUBIC TEST FUNCTION OF LEON \n");

    break;
  case 5:
    /*  JFUNC = 5 */
    /*  BEALE'S TEST FUNCTION */

    /*     THE MINIMUM IS F(3.0, 0.5) = 0.0 . */

    cprax.n = 2;
    
    cprax.x[0] = 0.1;
    cprax.x[1] = 0.1;

    printf("BEALE TEST FUNCTION \n ");
   
    break;
  case 6:
    /*  JFUNC = 6 */
    /*  WATSON'S TEST FUNCTION (SEE KOWALIK AND OSBORNE) */

    /*     FOR N = 6, THE MINIMUM IS NEAR */
    /* F(- 0.015725, 1.012435, - 0.232992, 1.260430, - 1.513729, 0.992996) = */
    /*           2.28767005355D-3 . */

    /*     FOR N = 9, THE MINIMUM IS NEAR */
    /* F(- 0.000015, 0.999790, 0.014764, 0.146342, 1.00081, - 2.617731, */
    /*           4.104403, -3.143612, 1.052627) = 1.399760138D-6 . */

    cprax.illcin = 1;

    ranini(&c_b22);

    for(j = 1; j <= cprax.n; j++) {
      cprax.x[j - 1] = 0.0;
    }

    printf("WATSON TEST FUNCTION WITH N = %d \n", cprax.n);

    break;
  case 7:
    /*  JFUNC = 7 */
    /*  FLETCHER'S CHEBYQUAD TEST FUNCTION */

    /*     FOR N = 2, THE MINIMUM IS NEAR F(0.2113249, 0.7886751) = 0.0 . */

    /*     FOR N = 4, THE MINIMUM IS NEAR */
    /*        F(0.1026728, 4062037, 0.5937963, 0.8973272) = 0.0 . */

    /*     FOR N = 6, THE MINIMUM IS NEAR */
    /*  F(0.066877, 0.288741, 0.366682, 0.63318, 0.711259, 0.933123) = 0.0 . */

    /*     FOR N = 8, THE MINIMUM IS NEAR */
    /*        F(0.043153, 0.193091, 0.266329, 0.500000, 0.500000, 0.733671, */
    /*          0.806910, 0.956847) = 0.00351687372568 . */

    cprax.h = 0.1;
    enplu   = ((double)(cprax.n + 1));

    for(j = 1; j <= cprax.n; j++) {
	cprax.x[j - 1] = ((double) j) / enplu;
    }

    printf("CHEBYQUAD TEST FUNCTION WITH N = %d \n", cprax.n);

    break;
  case 8:
    /*  JFUNC = 8 */
    /*  POWELL'S 1964 TEST FUNCTION */

    /*     THE MINIMUM IS F(1.0, 1.0, 1.0) = 3.0 . */

    cprax.n = 3;

    cprax.x[0] = 0.0;
    cprax.x[1] = 1.0;
    cprax.x[2] = 2.0;

    printf("POWELL (1964) TEST FUNCTION \n");
    
    break;
  case 9:
    /*  JFUNC = 9 */
    /*  WOOD'S TEST FUNCTION */

    /*     THE MINIMUM IS F(1.0, 1.0, 1.0, 1.0) = 0.0 . */

    cprax.illcin = 1;

    ranini(&c_b22);

    cprax.n = 4;
    
    cprax.h = 10.0;
    
    cprax.x[0] = - 3.0;
    cprax.x[1] = - 1.0;
    cprax.x[2] = - 3.0;
    cprax.x[3] = - 1.0;

    printf("TEST FUNCTION OF WOOD \n");
    
    break;    
  case 10:
    /*  JFUNC = 10 */
    /*  BRENT'S HILBERT MATRIX TEST FUNCTION */

    /*     THE MINIMUM IS F(0.0, 0.0, 0.0, ..., 0.0) = 0.0 . */

    /*     FOR N > 10, PRAXIS MAY RUN FOR A VERY, VERY LONG TIME */
    /*        AND TERMINATE WITH SOME COMPONENTS OF X(*) FAR FROM ZERO, */
    /*        BECAUSE OF THE EXTREME ILL-CONDITIONING OF THIS PROBLEM. */

    /*  SHOULDN'T ILLCIN BE SET TO 1 FOR THESE FUNCTIONS, */
    /*  AT LEAST FOR LARGE VALUES OF N? */

    if(cprax.n >= 10) {
	cprax.ktm = 4;
    }

    cprax.h = 10.0;
    
    for(j = 1; j <= cprax.n; j++) {
      cprax.x[j - 1] = 1.0;
    }
    
    printf("HILBERT MATRIX TEST FUNCTION WITH N = %d \n", cprax.n);

    break;
  case 11:
    /*  JFUNC = 11 */
    /*  TRIDIAGONAL MATRIX TEST FUNCTION */

    /*     THE MINIMUM IS F(N, N - 1, N - 2, ..., 2, 1) = - N . */

    cprax.h = 2.0 * cprax.n;

    for(j = 1; j <= cprax.n; j++) {
      cprax.x[j - 1] = 0.0;
    }

    printf("TRIDIAGONAL MATRIX TEST FUNCTION WITH N = %d \n", cprax.n);
    
    break;
  case 12:
    /*  JFUNC = 12 */
    /*  BOX'S TEST FUNCTION */

    /*     THE MINIMUM IS F(1.0, 10.0, 1.0) = 0.0 . */

    cprax.n = 3;
    
    cprax.h = 20.0;
    
    cprax.x[0] =  0.0;
    cprax.x[1] = 10.0;
    cprax.x[2] = 20.0;

    printf("TEST FUNCTION OF BOX \n");
    
    break;
  default:
    break;
  }

  /*  END TESTIN */

  return 0;
}

/* Subroutine */

/*  FTEST 1.1          SEPTEMBER 1995                                        */

/*     AN ERROR (SEE BELOW) WAS FIXED ON 25 SEPTEMBER 1995.                  */

/*  COMPUTE THE VALUE OF FUNCTION NUMBER JFUNC                               */
/*  FROM BRENT'S SUITE OF TEST PROBLEMS.                                     */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES",                       */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 137-154, 164-166             */

double ftest(double *x, int *n)
{
  /* System generated locals */

  double temp, temp1, temp2, temp3, temp4, temp5, ret_val = 0.0;

  /* Local variables */

  static int imax;
  static double term, f;
  static int i, j, new;
  static double p, r, s, t, u, y[N3], delta;
  static int jeven, jjmax;
  static double tplus;
  static int jj;
  static double ti[N3], yy, tminus[N3];

  new = *n;
  
  /* Function Body */

  if(prtest.jfunc < 1 || prtest.jfunc > prtest.maxjf) {
    return 0;
  }

  switch(((int) prtest.jfunc)) {
  case 1:
    /*  JFUNC = 1 */
    /*  ROSENBROCK'S TEST FUNCTION */

    temp = 10.0 * (x[1] - x[0] * x[0]);

    ret_val = temp * temp + (1.0 - x[0]) * (1.0 - x[0]);

    break;
  case 2: 
    /*  JFUNC = 2 */
    /*  POWELL'S SINGULAR TEST FUNCTION */

    temp  = (x[0] + 10.0 * x[1]) * (x[0] + 10.0 * x[1]);
    temp1 = 5.0 * (x[2] - x[3]) * (x[2] - x[3]);
    temp2 = pow((x[1] - 2.0 * x[2]), 4) + 10.0 * pow((x[0] - x[3]), 4);
      
    ret_val = temp + temp1 + temp2;
    
    break;
  case 3:
    /*  JFUNC = 3 */
    /*  HELICAL VALLEY TEST FUNCTION OF FLETCHER AND POWELL */

    r = sqrt(x[0] * x[0] + x[1] * x[1]);

    if(x[0] == 0.0) {
      t = 0.25;
    } else {
      t = atan(x[1] / x[0]) / (2.0 * prtest.pi);
    }

    if(x[0] < 0.0) {
	t += 0.5;
    }

    temp = (x[2] - 10.0 * t) * (x[2] - 10.0 * t);
      
    ret_val = 100.0 * (temp + (r - 1.0) * (r - 1.0)) + x[2] * x[2];

    break;
  case 4:
    /*  JFUNC = 4 */
    /*  LEON'S CUBIC TEST FUNCTION */

    temp = 100.0 * (x[1] - pow(x[0], 3)) * (x[1] - pow(x[0], 3));
    
    ret_val = temp + (1.0 - x[0]) * (1.0 - x[0]);
    
    break;
  case 5:
    /*  JFUNC = 5 */
    /*  BEALE'S TEST FUNCTION */

    temp  = (1.5 - x[0] * (1.0 - x[1])) * (1.5 - x[0] * (1.0 - x[1]));
    temp1 = (2.25 - x[0] * (1.0 - x[1] * x[1]));
    temp2 = (2.625 - x[0] * (1.0 - x[1] * (x[1] * x[1])));
    
    ret_val = temp + temp1 * temp1 + temp2 * temp2;
    
    break;
  case 6:
    /*  JFUNC = 6 */
    /*  WATSON'S TEST FUNCTION (SEE KOWALIK AND OSBORNE) */

    s = x[0] * x[0] + (x[1] - x[0] * x[0] - 1.0) * (x[1] - x[0] * x[0] - 1.0);

    for(i = 2; i <= 30; i++) {
      yy = ((double) (i - 1)) / 29.0;
      t  = x[new - 1];

      jjmax = new - 1;

      for(jj = 1; jj <= jjmax; jj++) {
	    j = jjmax - jj + 1;
            
	    t = x[j - 1] + yy * t;
      }

      u = ((double) (new - 1)) * x[new - 1];

      /*  VERSIONS OF THIS PACKAGE BEFORE 1.1 HAD AN ERROR JUST BELOW... */
      /*     THE 2 IN EACH OF THE NEXT TWO STATEMENTS WAS A 1 . */

      for(jj = 2; jj <= jjmax; jj++) {
        j = jjmax - jj + 2;

        u = ((double) (j - 1)) * x[j - 1] + yy * u;
      }

      s += (u - t * t - 1.0) * (u - t * t - 1.0);
    }
    
    ret_val = s;

    break;
  case 7:
    /*  JFUNC = 7 */
    /*  FLETCHER'S CHEBYQUAD TEST FUNCTION */

    delta = 0.0;

    for(j = 1; j <= new; j++) {
      y[j - 1] = 2.0 * x[j - 1] - 1.0;

      delta += y[j - 1];

      ti[j - 1]     = y[j - 1];
      tminus[j - 1] = 1.0;
    }

    f = delta * delta;
    
    jeven = 0;
    
    for(i = 2; i <= new; i++) {
      jeven = 1 - jeven;

      delta = 0.0;

      for(j = 1; j <= new; j++) {
        tplus  = 2.0 * y[j - 1] * ti[j - 1] - tminus[j - 1];
        delta += tplus;

        tminus[j - 1] = ti[j - 1];
        ti[j - 1]     = tplus;
      }

      if(jeven == 1) {
        term = 1.0 / (1.0 - ((double) (i  * i)));
      } else {
        term = 0.0;
      }

      delta = delta / ((double) new) - term;

      f += delta * delta;
    }

    ret_val = f;

    break;
  case 8:
    /*  JFUNC = 8 */
    /*  POWELL'S 1964 TEST FUNCTION */

    if(x[1] == 0.0) {
	term = 0.0;
    } else {
      temp = (x[0] + x[2]) / x[1] - 2.0;
      term = exp(- (temp * temp));
    }
    
    temp1 = x[0] - x[1];
    temp2 = sin(0.5 * prtest.pi * x[1] * x[2]);
      
    ret_val = 3.0 - 1.0 / (temp1 * temp1 + 1.0) - temp2 - term;

    break;
  case 9:
    /*  JFUNC = 9 */
    /*  WOOD'S TEST FUNCTION */

    temp  = 100.0 * (x[1] - x[0] * x[0]) * (x[1] - x[0] * x[0]);
    temp1 = (1.0 - x[0]) * (1.0 - x[0]);
    temp2 = 90.0 * (x[3] - x[2] * x[2]) * (x[3] - x[2] * x[2]);
    temp3 = (1.0 - x[2]) * (1.0 - x[2]);
    temp4 = 10.1 * ((x[1] - 1.0) * (x[1] - 1.0) + (x[3] - 1.0) * (x[3] - 1.0));
    temp5 = 19.8 * (x[1] - 1.0) * (x[3] - 1.0);
      
    ret_val = temp + temp1 + temp2 + temp3 + temp4 + temp5;

    break;
  case 10: 
    /*  JFUNC = 10 */
    /*  BRENT'S HILBERT MATRIX TEST FUNCTION */

    s = 0.0;
    
    for(i = 1; i <= new; i++) {
      t = 0.0;

      for(j = 1; j <= new; j++) {
        t += x[j - 1] / (((double) (i + j)) - 1.0);
      }

      s += t * x[i - 1];
    }

    ret_val = s;

    break;
  case 11:
    /*  JFUNC = 11 */
    /*  TRIDIAGONAL MATRIX TEST FUNCTION */

    s = x[0] * (x[0] - x[1]);

    imax = new - 1;

    for(i = 2; i <= imax; i++) {
      s += x[i - 1] * ((x[i - 1] - x[i - 2]) + (x[i - 1] - x[i]));
    }

    ret_val = s + x[new - 1] * (2.0 * x[new - 1] - x[new - 2]) - 2.0 * x[0];

    break;
  case 12:
    /*  JFUNC = 12 */
    /*  BOX'S TEST FUNCTION */

    s = 0.0;
    
    for(i = 1; i <= 10; i++) {
      p = - ((double) i) / 10.0;

      if(p * x[1] < - 40.0) {
        term = 0.0;
      } else {
        term = exp(p * x[1]);
      }

      temp = exp(p * x[0]) - term - x[2] * (exp(p) - exp(10.0 * p));

      s += temp * temp;
    }

    ret_val = s;

    break;
  default:
    break;
  }

  /*  END FTEST */

  return ret_val;
}

/* Subroutine */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*  THE CODE ABOVE IS FOR TESTING PRAXIS. */
/*  FOR PRODUCTION WORK, */
/*  DELETE THESE COMMENTS AND ALL CODE ABOVE THIS POINT. */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*  PRASET 1.0           JUNE 1995 */

/*  SET INITIAL VALUES FOR SOME QUANTITIES USED IN SUBROUTINE PRAXIS. */
/*  THE USER CAN RESET THESE, IF DESIRED, */
/*  AFTER CALLING PRASET AND BEFORE CALLING PRAXIS. */

/*  J. P. CHANDLER, COMPUTER SCIENCE DEPARTMENT, */
/*     OKLAHOMA STATE UNIVERSITY */

/*  ON MANY MACHINES, SUBROUTINE PRAXIS WILL CAUSE UNDERFLOW AND/OR */
/*  DIVIDE CHECK WHEN COMPUTING EPSMCH**4 AND EPSMCH**(-4). */
/*  IN THAT CASE, SET EPSMCH=1.0D-9 (OR POSSIBLY EPSMCH=1.0D-8) */
/*  AFTER CALLING SUBROUTINE PRASET. */

int praset()
{
  /* Local variables */

  static double xmid, rtwo, a, b;
  static int j;
  static double rzero, unitr, xplus;

  rzero = 0.0;
  unitr = 1.0;
  rtwo  = 2.0;

  /*  NMAX IS THE DIMENSION OF THE ARRAYS V(*, *), X(*), D(*), */
  /*  Q0(*), AND Q1(*). */

  cprax.nmax = 20;

  /*  NFMAX IS THE MAXIMUM NUMBER OF FUNCTION EVALUATIONS PERMITTED. */

  cprax.nfmax = 100000;

  /*  LP IS THE LOGICAL UNIT NUMBER FOR PRINTED OUTPUT. */

  cprax.lp = 6;

  /*  T IS A CONVERGENCE TOLERANCE USED IN SUBROUTINE PRAXIS. */

  cprax.t = 1.0e-5;

  /*  JPRINT CONTROLS PRINTED OUTPUT IN PRAXIS. */

  cprax.jprint = 4;

  /*  H IS AN ESTIMATE OF THE DISTANCE FROM THE INITIAL POINT */
  /*  TO THE SOLUTION. */

  cprax.h = 1.0;

  /*  USE BISECTION TO COMPUTE THE VALUE OF EPSMCH, "MACHINE EPSILON". */
  /*  EPSMCH IS THE SMALLEST FLOATING POINT (REAL OR DOUBLE PRECISION) */
  /*  NUMBER WHICH, WHEN ADDED TO ONE, GIVES A RESULT GREATER THAN ONE. */

  a = rzero;
  b = unitr;

 L10:

  xmid = a + (b - a) / rtwo;

  if(xmid <= a || xmid >= b) {
    goto L20;
  }

  xplus = unitr + xmid;

  if(xplus > unitr) {
    b = xmid;
  } else {
    a = xmid;
  }

  goto L10;

 L20:

  cprax.epsmch = b;

  for(j = 1; j <= cprax.nmax; j++) {
	cprax.x[j - 1] = rzero;
  }

  /*  JRANCH = 1 TO USE BRENT'S RANDOM, */
  /*  JRANCH = 2 TO USE FUNCTION DRANDM. */

  cprax.jranch = 1;
  
  ranini(&c_b83);    

  /*  DSEED IS AN INITIAL SEED FOR DRANDM, */
  /*  A SUBROUTINE THAT GENERATES PSEUDORANDOM NUMBERS */
  /*  UNIFORMLY DISTRIBUTED ON (0, 1). */

  cprax.dseed = 1234567.0;

  /*  SCBD IS AN UPPER BOUND ON THE SCALE FACTORS IN PRAXIS. */
  /*  IF THE AXES MAY BE BADLY SCALED (WHICH IS TO BE AVOIDED IF */
  /*  POSSIBLE) THEN SET SCBD = 10, OTHERWISE 1. */

  cprax.scbd = 1.0;

  /*  ILLCIN IS THE INITIAL VALUE OF ILLC, */
  /*  THE FLAG THAT SIGNALS AN ILL-CONDITIONED PROBLEM. */
  /*  IF THE PROBLEM IS KNOWN TO BE ILL-CONDITIONED SET ILLCIN=1, */
  /*  OTHERWISE 0. */

  cprax.illcin = 0;

  /*  KTM IS A CONVERGENCE SWITCH USED IN PRAXIS. */
  /*  KTM+1 IS THE NUMBER OF ITERATIONS WITHOUT IMPROVEMENT */
  /*  BEFORE THE ALGORITHM TERMINATES. */
  /*  KTM=4 IS VERY CAUTIOUS. */
  /*  USUALLY KTM=1 IS SATISFACTORY. */

  cprax.ktm = 1;
  
  /*  END PRASET */
  
  return 0;   
}

/* Subroutine */

/*  PRAXIS 2.0        JUNE 1995 */

/*  THE PRAXIS PACKAGE MINIMIZES THE FUNCTION F(X,N) OF N                    */
/*  VARIABLES X(1),...,X(N), USING THE PRINCIPAL AXIS METHOD.                */
/*  F MUST BE A SMOOTH (CONTINUOUSLY DIFFERENTIABLE) FUNCTION.               */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES",                       */
/*     RICHARD P. BRENT, PRENTICE-HALL 1973 (ISBN 0-13-022335-2),            */
/*     PAGES 156-167                                                         */

/*  TRANSLATED FROM ALGOL W TO A.N.S.I. 1966 STANDARD BASIC FORTRAN          */
/*     BY ROSALEE TAYLOR AND SUE PINSKI, COMPUTER SCIENCE DEPARTMENT,        */
/*     OKLAHOMA STATE UNIVERSITY (DECEMBER 1973).                            */

/*  UPDATED TO A.N.S.I. STANDARD FORTRAN 77 BY J. P. CHANDLER                */
/*     COMPUTER SCIENCE DEPARTMENT, OKLAHOMA STATE UNIVERSITY                */

/*  SUBROUTINE PRAXIS CALLS SUBPROGRAMS                                      */
/*     F, MIN, RANDOM (OR DRANDM), QUAD, MINFIT, SORT.                       */

/*  SUBROUTINE QUAD CALLS MIN.                                               */

/*  SUBROUTINE MIN CALLS FLIN.                                               */

/*  SUBROUTINE FLIN CALLS F.                                                 */

/*  INPUT QUANTITIES (SET IN THE CALLING PROGRAM)...                         */

/*     F        FUNCTION F(X, N) TO BE MINIMIZED                             */

/*     X(*)     INITIAL GUESS OF MINIMUM                                     */

/*     N        DIMENSION OF X  (NOTE...  N MUST BE > 2)                     */

/*     H        MAXIMUM STEP SIZE                                            */

/*     T        TOLERANCE                                                    */

/*     EPSMCH   MACHINE PRECISION                                            */

/*     JPRINT   PRINT SWITCH                                                 */


/*  OUTPUT QUANTITIES...                                                     */

/*     X(*)     ESTIMATED POINT OF MINIMUM                                   */

/*     FX       VALUE OF F AT X                                              */

/*     NL       NUMBER OF LINEAR SEARCHES                                    */

/*     NF       NUMBER OF FUNCTION EVALUATIONS                               */

/*     V(*, *)   EIGENVECTORS OF A NEW DIRECTIONS                            */

/*     D(*)     EIGENVALUES OF A NEW D                                       */

/*     Z(*)     SCALE FACTORS                                                */

/*  ON ENTRY X(*) HOLDS A GUESS.  ON RETURN IT HOLDS THE ESTIMATED           */
/*  POINT OF MINIMUM, WITH (HOPEFULLY)                                       */
/*  ABS(ERROR) LESS THAN SQRT(EPSMCH) * ABS(X) + T, WHERE                    */
/*  EPSMCH IS THE MACHINE PRECISION, THE SMALLEST NUMBER SUCH THAT           */
/*  (1 + EPSMCH) IS GREATER THAN 1.                                          */

/*  T IS A TOLERANCE.                                                        */

/*  H IS THE MAXIMUM STEP SIZE, SET TO ABOUT THE MAXIMUM EXPECTED            */
/*  DISTANCE FROM THE GUESS TO THE MINIMUM.  IF H IS SET TOO                 */
/*  SMALL OR TOO LARGE THEN THE INITIAL RATE OF CONVERGENCE WILL             */
/*  BE SLOW.                                                                 */

/*  THE USER SHOULD OBSERVE THE COMMENT ON HEURISTIC NUMBERS                 */
/*  AT THE BEGINNING OF THE SUBROUTINE.                                      */

/*  JPRINT CONTROLS THE PRINTING OF INTERMEDIATE RESULTS.                    */
/*  IT USES SUBROUTINES FLIN, MIN, QUAD, SORT, AND MINFIT.                   */
/*  IF JPRINT = 1, F IS PRINTED AFTER EVERY N + 1 OR N + 2 LINEAR            */
/*  MINIMIZATIONS, AND FINAL X IS PRINTED, BUT INTERMEDIATE                  */
/*  X ONLY IF N IS LESS THAN OR EQUAL TO 4.                                  */
/*  IF JPRINT = 2, EIGENVALUES OF A AND SCALE FACTORS ARE ALSO PRINTED.      */
/*  IF JPRINT = 3, F AND X ARE PRINTED AFTER EVERY FEW LINEAR                */
/*  MINIMIZATIONS.                                                           */
/*  IF JPRINT = 4, EIGENVECTORS ARE ALSO PRINTED.                            */
/*  IF JPRINT = 5, ADDITIONAL DEBUGGING INFORMATION IS ALSO PRINTED.         */

/*  RANDOM RETURNS A RANDOM NUMBER UNIFORMLY DISTRIBUTED IN (0, 1).          */

/*  THIS SUBROUTINE IS MACHINE-INDEPENDENT, APART FROM THE                   */
/*  SPECIFICATION OF EPSMCH.  WE ASSUME THAT EPSMCH**(-4) DOES NOT           */
/*  OVERFLOW (IF IT DOES THEN EPSMCH MUST BE INCREASED), AND THAT ON         */
/*  FLOATING-POINT UNDERFLOW THE RESULT IS SET TO ZERO.                      */

int praxis(double (*f)())
{
  /* Local variables */

  static int illc;
  static double xlds, zero;
  static double e[N1];
  static int i, j, k;
  static double s, y[N1], z[N1], rhalf, tenth, f1;
  static int k2;
  static double t2, f1valu, df, dldfac, dn;
  static int ik, im, kl;
  static double sf;
  static int kt;
  static double sl;
  static double vlarge;
  static double xlarge, ranval, hundrd;
  static double hundth, vsmall;
  static int km1;
  static double fxvalu, pt9, one;
  static double ten;
  static int imu;
  static float two;
  
  /*  INITIALIZATION... */

  zero   =   0.0;
  hundth =   0.01;
  tenth  =   0.1;
  rhalf  =   0.5;
  pt9    =   0.9;
  one    =   1.0;
  ten    =  10.0;
  two    =   2.0;
  hundrd = 100.0;

  /*  MACHINE DEPENDENT NUMBERS... */

  /*  ON MANY COMPUTERS, VSMALL WILL UNDERFLOW, */
  /*  AND COMPUTING XLARGE MAY CAUSE A DIVISION BY ZERO. */
  /*  IN THAT CASE, EPSMCH SHOULD BE SET EQUAL TO 1.0D-9 */
  /*  (OR POSSIBLY 1.0D-8) BEFORE CALLING PRAXIS. */

  cprax.small = cprax.epsmch * cprax.epsmch;

  vsmall = cprax.small * cprax.small;
  xlarge = one / cprax.small;
  vlarge = one / vsmall;

  cprax.xm2 = sqrt(cprax.epsmch);
  cprax.xm4 = sqrt(cprax.xm2);

  /*  HEURISTIC NUMBERS... */

  /*  IF THE AXES MAY BE BADLY SCALED (WHICH IS TO BE AVOIDED IF */
  /*  POSSIBLE) THEN SET SCBD = 10, OTHERWISE 1. */

  /*  IF THE PROBLEM IS KNOWN TO BE ILL-CONDITIONED SET ILLC = 1, */
  /*  OTHERWISE 0. */

  /*  KTM + 1 IS THE NUMBER OF ITERATIONS WITHOUT IMPROVEMENT */
  /*  BEFORE THE ALGORITHM TERMINATES. */
  /*  KTM = 4 IS VERY CAUTIOUS. */
  /*  USUALLY KTM = 1 IS SATISFACTORY. */

  /*  BRENT RECOMMENDED THE FOLLOWING VALUES FOR MOST PROBLEMS... */

  /*                    SCBD = 1.0 */
  /*                    ILLC = 0 */
  /*                    KTM  = 1 */

  /*  SCBD, ILLCIN, AND KTM ARE NOW IN COMMON. */
  /*  THEY ARE INITIALIZED IN SUBROUTINE PRASET, */
  /*  AND CAN BE RESET BY THE USER AFTER CALLING PRASET. */
  
  illc = cprax.illcin;

  if(illc == 1) {
    dldfac = tenth;
  } else {
    dldfac = hundth;
  }

  kt = 0;

  cprax.nl = 0;
  cprax.nf = 1;

  cprax.fx  = (*f)(cprax.x, &cprax.n);    
  cprax.qf1 = cprax.fx;
  cprax.t   = cprax.small + fabs(cprax.t);

  t2 = cprax.t;
  
  cprax.dmin = cprax.small;

  if(cprax.h < hundrd * cprax.t) {
    cprax.h = hundrd * cprax.t;
  }

  cprax.xldt = cprax.h;

  for(i = 1; i <= cprax.n; i++) {
    for(j = 1; j <= cprax.n; j++) {
      cprax.v[a(i, j, N1, N2)] = zero;
    }       

    cprax.v[a(i, i, N1, N2)] = one;
  }

  cprax.qd0  = zero;
  cprax.d[0] = zero;

  /*  Q0(*) AND Q1(*) ARE PREVIOUS X(*) POINTS, */
  /*  INITIALIZED IN PRAXIS, USED IN FLIN, */
  /*  AND CHANGED IN QUAD. */

  for(i = 1; i <= cprax.n; i++) {
    cprax.q1[i - 1] = cprax.x[i - 1];

    /*  Q0(*) WAS NOT INITIALIZED IN BRENT'S ALGOL PROCEDURE. */

    cprax.q0[i - 1] = cprax.x[i - 1];
  }

  if(cprax.jprint > 0) {
    printf("NL = %d, NF = %d, FX = %g \n", cprax.nl, cprax.nf, cprax.fx);
    
    if(cprax.n <= 4 || cprax.jprint > 2) {     
      for(i = 1; i <= cprax.n; i++) {
        printf("X[%d] = %g \n", i - 1, cprax.x[i - 1]);
      }
    }
  }

  /*  MAIN LOOP... */
  /*  LABEL L0... */

 L60:

  sf = cprax.d[0];
  s  = zero;

  cprax.d[0] = zero;

  /*  MINIMIZE ALONG THE FIRST DIRECTION. */

  if(cprax.jprint >= 5) {
    printf("CALL NO. 1 TO MIN. D(0) = %g \n", cprax.d[0]);
    printf("S = %g \n", s);
    printf("FX = %g \n", cprax.fx);
  }

  fxvalu = cprax.fx;

  min(&c1, &c2, cprax.d, &s, &fxvalu, &c0, f);

  if(s <= zero) {
    for(i = 1; i <= cprax.n; i++) {
      cprax.v[i - 1] = - cprax.v[i - 1];
    }
  }

  if(sf <= pt9 * cprax.d[0] || pt9 * sf >= cprax.d[0]) {
    if(cprax.n >= 2) {
      for(i = 2; i <= cprax.n; i++) {
        cprax.d[i - 1] = zero;
      }
    }
  }
       
  if(cprax.n < 2) {
    goto L320;
  }

  for(k = 2; k <= cprax.n; k++) {
    for(i = 1; i <= cprax.n; i++) {
      y[i - 1] = cprax.x[i - 1];
    }

    sf = cprax.fx;

    if(kt > 0) {
      illc = 1;
    }

    /*  LABEL L1... */

  L110:

    kl = k;

    df = zero;

    /*  TAKE A RANDOM STEP TO GET OUT OF A RESOLUTION VALLEY. */

    /*  PRAXIS ASSUMES THAT RANDOM (OR DRANDM) RETURNS */
    /*  A PSEUDORANDOM NUMBER UNIFORMLY DISTRIBUTED IN (0, 1), */
    /*  AND THAT ANY INITIALIZATION OF THE RANDOM NUMBER GENERATOR */
    /*  HAS ALREADY BEEN DONE. */

    if(illc == 1) {
      for(i = 1; i <= cprax.n; i++) {
        if(cprax.jranch == 1) {
          prandom(&ranval);
        } else {
          ranval = drandm(&cprax.dseed);
        }
        
        s = (tenth * cprax.xldt + t2 * powd(&ten, &kt)) * (ranval - rhalf);

        // s = (tenth * cprax.xldt + t2 * pow(ten, kt)) * (ranval - rhalf);

        z[i - 1] = s;

        for(j = 1; j <= cprax.n; j++) {
          cprax.x[j - 1] += s * cprax.v[a(j, i, N1, N2)];
        }
      }

      cprax.fx = (*f)(cprax.x, &cprax.n);

      ++cprax.nf;

      if(cprax.jprint >= 1) {
        printf("*****  RANDOM STEP IN PRAXIS. \n");
        printf("NF = %d \n", cprax.nf);
        printf("SF = %g \n", sf);
        printf("FX = %g \n", cprax.fx);
      }
    }

    if(k > cprax.n) {
      goto L170;
    }

    for(k2 = k; k2 <= cprax.n; k2++) {
      sl = cprax.fx;
      s  = zero;

      /*  MINIMIZE ALONG NON-CONJUGATE DIRECTIONS. */

      if(cprax.jprint >= 5) {
        printf("CALL NO. 2 TO MIN. K2 = %d \n", k2);
        printf("D(K2) = %g \n", cprax.d[k2 - 1]);
        printf("S = %g \n", s);
        printf("FX = %g \n", cprax.fx);
      }

      fxvalu = cprax.fx;

      min(&k2, &c2, &cprax.d[k2 - 1], &s, &fxvalu, &c0, f);

      if(illc == 1) {
        s = cprax.d[k2 - 1] * ((s + z[k2 - 1]) * (s + z[k2 - 1]));
      } else {
        s = sl - cprax.fx;
      }

      if(df < s) {
        df = s;

        kl = k2;
      }
    }

  L170:

    if(illc == 0 && df < fabs(hundrd * cprax.epsmch * cprax.fx)) {
      /*  NO SUCCESS WITH ILLC = 0, SO TRY ONCE WITH ILLC = 1 . */

      illc = 1;

      /*  GO TO L1. */

      goto L110;
    }

    if(k == 2 && cprax.jprint > 1) {
      for(i = 1; i <= cprax.n; i++) {
        printf("NEW D = %g \n", cprax.d[i - 1]);
      }
    }
    
    km1 = k - 1;

    if(km1 < 1) {
      goto L210;
    }

    for(k2 = 1; k2 <= km1; k2++) {
      /*  MINIMIZE ALONG CONJUGATE DIRECTIONS. */

      if(cprax.jprint >= 5) {
        printf("CALL NO. 3 TO MIN. K2 = %d \n", k2);
        printf("D(K2) = %g \n", cprax.d[k2 - 1]);
        printf("S = %g \n", s);
        printf("FX = %g \n", cprax.fx);
      }

      s = zero;
      
      fxvalu = cprax.fx;              

      min(&k2, &c2, &cprax.d[k2 - 1], &s, &fxvalu, &c0, f);
    }

  L210:

    f1 = cprax.fx;
    
    cprax.fx = sf;

    xlds = zero;
    
    for(i = 1; i <= cprax.n; i++) {    
      sl = cprax.x[i - 1];
      
      cprax.x[i - 1] = y[i - 1];      
      
      sl -= y[i - 1];

      y[i - 1]  = sl;

      xlds += sl * sl;
    }

    xlds = sqrt(xlds);
      
    if(xlds > cprax.small) {
      /*  THROW AWAY THE DIRECTION KL AND MINIMIZE ALONG */
      /*  THE NEW CONJUGATE DIRECTION. */

      ik = kl - 1;
      
      if(k > ik) {    
        goto L250;
      }

      for(im = k; im <= ik; im++) {
        i = ik - im + k;
        
        for(j = 1; j <= cprax.n; j++) {
          cprax.v[a(j, (i + 1), N1, N2)] = cprax.v[a(j, i, N1, N2)];
        }

        cprax.d[i] = cprax.d[i - 1];
      }

    L250:

      cprax.d[k - 1] = zero;

      for(i = 1; i <= cprax.n; i++) {
        cprax.v[a(i, k, N1, N2)] = y[i - 1] / xlds;
      }

      if(cprax.jprint >= 5) {
        printf("CALL NO. 4 TO MIN. K = %d \n", k);
        printf("D(K) = %g \n", cprax.d[k - 1]);
        printf("XLDS = %g \n", xlds);
        printf("F1 = %g \n", f1);
      }

      f1valu = f1;

      min(&k, &c4, &cprax.d[k - 1], &xlds, &f1valu, &c1, f);

      if(xlds <= zero) {
        xlds = - xlds;

        for(i = 1; i <= cprax.n; i++) {     
          cprax.v[a(i, k, N1, N2)] = - cprax.v[a(i, k, N1, N2)];
        }
      }
    }

    cprax.xldt = dldfac * cprax.xldt;
    
    if(cprax.xldt < xlds) {
      cprax.xldt = xlds;
    }
    
    if(cprax.jprint > 0) {
      printf("NL = %d, NF = %d, FX = %g \n", cprax.nl, cprax.nf, cprax.fx);

      if(cprax.n <= 4 || cprax.jprint > 2) {
        for(i = 1; i <= cprax.n; i++) {
          printf("X[%d] = %g \n", i - 1, cprax.x[i - 1]);
        }
      }
    }

    t2 = zero;

    for(i = 1; i <= cprax.n; i++) {
      t2 += cprax.x[i - 1] * cprax.x[i - 1];
    }

    t2 = cprax.xm2 * sqrt(t2) + cprax.t;

    /*  SEE IF THE STEP LENGTH EXCEEDS HALF THE TOLERANCE. */
    
    if(cprax.xldt > rhalf * t2) {
      kt = 0;
    } else {
      ++kt;
    }

    /*  IF(...) GO TO L2 */

    if(kt > cprax.ktm) {
      goto L550;
    }
    
    if(cprax.nf >= cprax.nfmax) {
      printf("IN PRAXIS, NF REACHED THE LIMIT NFMAX = %d \n", cprax.nfmax);
      printf("THIS IS AN ABNORMAL TERMINATION. \n");
      printf("THE FUNCTION HAS NOT BEEN MINIMIZED. \n");
      printf("THE RESULTING X(*) VECTOR SHOULD NOT BE USED. \n");
      
      goto L550;
    }
  }

  /*  TRY QUADRATIC EXTRAPOLATION IN CASE WE ARE STUCK IN A CURVED VALLEY. */

 L320:

  pquad(f);
  
  dn = zero;

  for(i = 1; i <= cprax.n; i++) {
    cprax.d[i - 1] = one / sqrt(cprax.d[i - 1]);

    if(dn < cprax.d[i - 1]) {
        dn = cprax.d[i - 1];
    }
  }

  printf("NEW DIRECTIONS \n");
    
  if(cprax.jprint > 3) {
    for(i = 1; i <= cprax.n; i++) {
      printf("I = %d \n", i);
      
      for(j = 1; j <= cprax.n; j++) {
        printf("V[%d] = %g \n", a(i, j, N1, N2), cprax.v[a(i, j, N1, N2)]);
      }
    }
  }   

  for(j = 1; j <= cprax.n; j++) {
    s = cprax.d[j - 1] / dn;
      
    for(i = 1; i <= cprax.n; i++) {
      cprax.v[a(i, j, N1, N2)] = s * cprax.v[a(i, j, N1, N2)];
    }
  }

  if(cprax.scbd > one) {
    /*  SCALE THE AXES TO TRY TO REDUCE THE CONDITION NUMBER. */

    s = vlarge;
    
    for(i = 1; i <= cprax.n; i++) {
      sl = zero;
      
      for(j = 1; j <= cprax.n; j++) {
        sl += cprax.v[a(i, j, N1, N2)] * cprax.v[a(i, j, N1, N2)];
      }

      z[i - 1] = sqrt(sl);

      if(z[i - 1] < cprax.xm4) {
        z[i - 1] = cprax.xm4;
      }

      if(s > z[i - 1]) {
        s = z[i - 1];
      }
    }

    for(i = 1; i <= cprax.n; i++) {
      sl = s / z[i - 1];

      z[i - 1] = one / sl;
      
      if(z[i - 1] > cprax.scbd) {
        sl = one / cprax.scbd;

        z[i - 1] = cprax.scbd;
      }

      /*  IT APPEARS THAT THERE ARE TWO MISSING END; STATEMENTS */
      /*  AT THIS POINT IN BRENT'S LISTING. */
    }
  }

  /*  TRANSPOSE V FOR MINFIT. */

  if(cprax.n < 2) {
    goto L440;
  }   

  for(i = 2; i <= cprax.n; i++) {
    imu = i - 1;

    for(j = 1; j <= imu; j++) {
      s = cprax.v[a(i, j, N1, N2)];
      
      cprax.v[a(i, j, N1, N2)] = cprax.v[a(j, i, N1, N2)];    
      cprax.v[a(j, i, N1, N2)] = s;
    }
  }

  /*  FIND THE SINGULAR VALUE DECOMPOSITION OF V. */  
  /*  THIS GIVES THE EIGENVALUES AND PRINCIPAL AXES */
  /*  OF THE APPROXIMATING QUADRATIC FORM */
  /*  WITHOUT SQUARING THE CONDITION NUMBER. */

 L440:

  minfit(&cprax.n, &cprax.epsmch, &vsmall, cprax.v, cprax.d, e,
         &cprax.nmax, &cprax.lp);

  if(cprax.scbd > one) {
    /*  UNSCALING... */

    for(i = 1; i <= cprax.n; i++) {
      s = z[i - 1];
      
      for(j = 1; j <= cprax.n; j++) {
        cprax.v[a(i, j, N1, N2)] = s * cprax.v[a(i, j, N1, N2)];
      }
    }

    for(i = 1; i <= cprax.n; i++) {
      s = zero;
      
      for(j = 1; j <= cprax.n; j++) {  
        s += cprax.v[a(j, i, N1, N2)] * cprax.v[a(j, i, N1, N2)];
      }

      s = sqrt(s);

      cprax.d[i - 1] = s * cprax.d[i - 1];
      
      s = one / s;

      for(j = 1; j <= cprax.n; j++) {
        cprax.v[a(j, i, N1, N2)] = s * cprax.v[a(j, i, N1, N2)];
      }
    }
  }

  for(i = 1; i <= cprax.n; i++) {
    if(dn * cprax.d[i - 1] > xlarge) {
      cprax.d[i - 1] = vsmall;
    } else if(dn * cprax.d[i - 1] < cprax.small) {
      cprax.d[i - 1] = vlarge;
    } else {
      cprax.d[i - 1] = one / (dn * dn * cprax.d[i - 1] * cprax.d[i - 1]);
    }
  }
  
  /*  SORT THE NEW EIGENVALUES AND EIGENVECTORS. */

  sort();

  cprax.dmin = cprax.d[cprax.n - 1];
  
  if(cprax.dmin < cprax.small) {
    cprax.dmin = cprax.small;
  }
  
  if(cprax.xm2 * cprax.d[0] > cprax.dmin) {
    illc = 1;
  } else {
    illc = 0;
  }
    
  if(cprax.jprint > 1 && cprax.scbd > one) {
    printf("SCALE FACTORS \n");
    
    for(i = 1; i <= cprax.n; i++) {    
      printf("Z[%d] = %g \n", i - 1, z[i - 1]);       
    }
  }

  if(cprax.jprint > 1) {
    printf("EIGENVALUES OF A \n");
    
    for(i = 1; i <= cprax.n; i++) {    
      printf("D[%d] = %g \n", i - 1, cprax.d[i - 1]);
    }
  }

  if(cprax.jprint > 3) {
    printf("EIGENVECTORS OF A \n");
    
    for(i = 1; i <= cprax.n; i++) {
      printf("I = %d \n", i);

      for(j = 1; j <= cprax.n; j++) {
        printf("V[%d] = %g \n", a(i, j, N1, N2), cprax.v[a(i, j, N1, N2)]);
      }
    }
  }

  /*  GO BACK TO THE MAIN LOOP. */
  /*  GO TO L0 */
  
  /*  HANDLE THE CASE N .EQ. 1 IN AN AD HOC WAY. */   
  /*  (BRENT DID NOT PROVIDE FOR THIS CASE.) */
  
  if(cprax.n >= 2) {
    goto L60;
  }
  
  /*  LABEL L2... */

 L550:

  if(cprax.jprint > 0) {
    for(i = 1; i <= cprax.n; i++) {
      printf("X[%d] = %g \n", i - 1, cprax.x[i - 1]);
    } 
  }
  
  cprax.fx = (*f)(cprax.x, &cprax.n);
  
  if(cprax.jprint >= 0) {
    printf("EXIT PRAXIS. \n");
    printf("FX = %g, NL = %d, NF = %d \n", cprax.fx, cprax.nl, cprax.nf);
  }

  /*  END PRAXIS */

  return 0;
}
  
/* Subroutine */      

/*  THIS SUBROUTINE LOOKS FOR THE MINIMUM ALONG */
/*  A CURVE DEFINED BY Q0, Q1, AND X. */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGE 161 */

/*  SUBROUTINE QUAD IS CALLED BY SUBROUTINE PRAXIS. */
  
int pquad(double (*f)())
{
  /* Local variables */

  static double zero;
  static int i;
  static double s, qf1val, qa, qb, qc, xlg, one;

  zero = 0.0;
  one  = 1.0;

  s = cprax.fx;

  cprax.fx  = cprax.qf1;
  cprax.qf1 = s;
  cprax.qd1 = zero;

  for(i = 1; i <= cprax.n; i++) {
    s   = cprax.x[i - 1];
    xlg = cprax.q1[i - 1];

    cprax.x[i - 1]  = xlg;
    cprax.q1[i - 1] = s;

    cprax.qd1 += (s - xlg) * (s - xlg);
  }

  cprax.qd1 = sqrt(cprax.qd1);

  xlg = cprax.qd1;
  s   = zero;

  if((cprax.qd0 > zero && cprax.qd1 > zero) &&
     (cprax.nl >= 3 * cprax.n * cprax.n)) {
    if(cprax.jprint >= 1) {
      printf("*****  CALL MIN FROM QUAD. NF = %d \n", cprax.nf);
      printf("QD0 = %g \n", cprax.qd0);
      printf("QD1 = %g \n", cprax.qd1);
      printf("FX = %g \n", cprax.fx);
      printf("QD0 = %g \n", cprax.qf1);
    }

    qf1val = cprax.qf1;

    min(&c0, &c2, &s, &xlg, &qf1val, &c1, f);

    qa = xlg * (xlg - cprax.qd1) / (cprax.qd0 * (cprax.qd0 + cprax.qd1));
    qb = (xlg + cprax.qd0) * (cprax.qd1 - xlg) / (cprax.qd0 * cprax.qd1);
    qc = xlg * (xlg + cprax.qd0) / (cprax.qd1 * (cprax.qd0 + cprax.qd1));
  } else {
    cprax.fx = cprax.qf1;

    qa = zero;
    qb = zero;
    qc = one;
  }

  cprax.qd0 = cprax.qd1;

  for(i = 1; i <= cprax.n; i++) {
    s = cprax.q0[i - 1];

    cprax.q0[i - 1] = cprax.x[i - 1];
    cprax.x[i - 1] = qa * s + qb * cprax.x[i - 1] + qc * cprax.q1[i - 1];
  }
  
  /*  END QUAD */

  return 0;
} 

/* Subroutine */

/*  SUBROUTINE MIN MINIMIZES F FROM X IN THE DIRECTION V(*, J) */
/*  UNLESS J IS LESS THAN 1, WHEN A QUADRATIC SEARCH IS DONE IN */
/*  THE PLANE DEFINED BY Q0, Q1, AND X. */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 159-160 */

/*  SUBROUTINE MIN IS CALLED BY SUBROUTINES PRAXIS AND QUAD. */

/*  D2 AND X1 RETURN RESULTS. */
/*  J, NITS, F1 AND IFK ARE VALUE PARAMETERS THAT RETURN NOTHING. */
/*  DO NOT SEND A COMMON VARIABLE TO MIN FOR PARAMETER F1. */

/*  D2 IS AN APPROXIMATION TO HALF OF */
/*  THE SECOND DERIVATIVE OF F (OR ZERO). */

/*  X1 IS AN ESTIMATE OF DISTANCE TO MINIMUM, */
/*  RETURNED AS THE DISTANCE FOUND. */

/*  IF IFK = 1 THEN F1 IS FLIN(X1), OTHERWISE X1 AND F1 ARE */
/*  IGNORED ON ENTRY UNLESS FINAL FX IS GREATER THAN F1. */

/*  NITS CONTROLS THE NUMBER OF TIMES AN ATTEMPT IS MADE TO */
/*  HALVE THE INTERVAL. */

int min(int *j, int *nits, double *d2, double *x1, double *f1, int *ifk,
        double (*f)())
{
  /* System generated locals */

  double temp;

  /* Local variables */

  static double zero;
  static int i, k;
  static double s, rhalf, denom, d1, f0, f2, t2, x2, fm, xm, hundth, sf1, sx1;
  static int idz;
  static float two;

  hundth = 0.01;
  zero   = 0.0;
  two    = 2.0;
  rhalf  = 0.5;
  
  sf1 = *f1;
  sx1 = *x1;
  
  k = 0;

  xm = zero;
  fm = cprax.fx;
  f0 = cprax.fx;

  if(*d2 < cprax.epsmch) {
    idz = 1;
  } else {
    idz = 0;
  }

  /*  FIND THE STEP SIZE. */

  s = zero;

  for(i = 1; i <= cprax.n; i++) {
    s += cprax.x[i - 1] * cprax.x[i - 1];
  }

  s = sqrt(s);

  if(idz == 1) {
    denom = cprax.dmin;
  } else {
    denom = *d2;
  }
  
  temp = fabs(cprax.fx) / denom + s * cprax.xldt;
  t2   = cprax.xm4 * sqrt(temp) + cprax.xm2 * cprax.xldt;
  s    = cprax.xm4 * s + cprax.t;
    
  if(idz == 1 && t2 > s) {
    t2 = s;
  }
    
  if(t2 < cprax.small) {
    t2 = cprax.small;
  }

  if(t2 > hundth * cprax.h) {
    t2 = hundth * cprax.h;
  }

  if(*ifk == 1 && *f1 <= fm) {
    xm = *x1;
    fm = *f1;
  }

  if(*ifk == 0 || fabs(*x1) < t2) {
    if(*x1 >= zero) { 
      *x1 = t2;
    } else {
      *x1 = - t2;
    }

    flin(x1, j, f, f1);
  }

  if(*f1 < fm) {
    xm = *x1;
    fm = *f1;
  }

  /*  LABEL L0... */

 L20:

  if(idz == 1) {
    /*  EVALUATE FLIN AT ANOTHER POINT, */
    /*  AND ESTIMATE THE SECOND DERIVATIVE. */

    if(f0 < *f1) {
      x2 = - (*x1);
    } else {
      x2 = two * *x1;
    }

    flin(&x2, j, f, &f2);

    if(f2 <= fm) {
      xm = x2;
      fm = f2;
    }

    *d2 = (x2 * (*f1 - f0) - *x1 * (f2 - f0)) / (*x1 * x2 * (*x1 - x2));

    if(cprax.jprint >= 5) {
      printf("COMPUTE D2 IN SUBROUTINE MIN. \n");
      printf("X1 = %g \n", (*x1));
      printf("X2 = %g \n", x2);
      printf("F0 = %g \n", f0);
      printf("F1 = %g \n", (*f1));
      printf("F2 = %g \n", f2);
      printf("D2 = %g \n", (*d2));
    }
  }

  /*  ESTIMATE THE FIRST DERIVATIVE AT 0. */

  d1 = (*f1 - f0) / *x1 - *x1 * *d2;

  idz = 1;

  /*  PREDICT THE MINIMUM. */

  if(*d2 <= cprax.small) {
    if(d1 < zero) {
      x2 = cprax.h;
    } else {
      x2 = - cprax.h;
    }
  } else {
    x2 = - rhalf * d1 / *d2;
  }

  if(fabs(x2) > cprax.h) {
    if (x2 > zero) {
      x2 = cprax.h;
    } else {
      x2 = - cprax.h;
    }
  }

  /*  EVALUATE F AT THE PREDICTED MINIMUM. */
  /*  LABEL L1... */

 L40:

  flin(&x2, j, f, &f2);

  if(k < *nits && f2 > f0) {
    /*  NO SUCCESS, SO TRY AGAIN. */

    ++k;

    /*  IF(...) GO TO L0 */

    if(f0 < *f1 && *x1 * x2 > zero) {
      goto L20;
    }
    
    x2 /= two;

    /*  GO TO L1 */

    goto L40;
  }

  /*  INCREMENT THE ONE-DIMENSIONAL SEARCH COUNTER. */

  ++cprax.nl;

  if(f2 > fm) {
    x2 = xm;
  } else {
    fm = f2;
  }

  /*  GET A NEW ESTIMATE OF THE SECOND DERIVATIVE. */

  if(fabs(x2 * (x2 - *x1)) > cprax.small) {
    *d2 = (x2 * (*f1 - f0) - *x1 * (fm - f0)) / (*x1 * x2 * (*x1 - x2));
    
    if(cprax.jprint >= 5) { 
      printf("RECOMPUTE D2 IN SUBROUTINE MIN. \n");
      printf("X1 = %g \n", (*x1));
      printf("X2 = %g \n", x2);
      printf("F0 = %g \n", f0);
      printf("FM = %g \n", fm);
      printf("X2 = %g \n", x2);
      printf("F1 = %g \n", (*f1));
      printf("D2 = %g \n", (*d2));
    }
  } else if (k > 0) {
    *d2 = zero;

    if(cprax.jprint >= 5) {
      printf("SET D2=0 IN SUBROUTINE MIN. \n");
    }
  } else {
    *d2 = *d2;
  }

  if(*d2 <= cprax.small) {
    *d2 = cprax.small;

    if(cprax.jprint >= 5) {
      printf("SET D2 = SMALL = %g IN SUBROUTINE MIN. \n", (*d2));
    }
  }

  if(cprax.jprint >= 5) {
    printf("SUBROUTINE MIN. \n");
    printf("X1 = %g", (*x1));
    printf("X2 = %g", x2);
    printf("FX = %g", cprax.fx);
    printf("FM = %g", fm);
    printf("SF1 = %g", sf1);
  }

  *x1 = x2;

  cprax.fx = fm;

  if(sf1 < cprax.fx) {
    cprax.fx = sf1;

    *x1 = sx1;
  }

  /*  UPDATE X FOR A LINEAR SEARCH BUT NOT FOR A PARABOLIC SEARCH. */

  if(*j > 0) {
    for(i = 1; i <= cprax.n; i++) {
      // cprax.x[i - 1] += *x1 * cprax.v[i + 20 * (*j) - 21];

      cprax.x[i - 1] += *x1 * cprax.v[a(i, *j, 20, 21)];
    }
  }

  if(cprax.jprint >= 5) {
    printf("LEAVE SUBROUTINE MIN. \n");
    printf("D2 = %g \n", (*d2));
    printf("X1 = %g \n", (*x1));
    printf("F1 = %g \n", (*f1));
    printf("FX = %g \n", cprax.fx);
  }


  /*  END MIN */

  return 0;
}

/* Subroutine */

/*  FLIN IS A FUNCTION OF ONE VARIABLE XLG WHICH IS MINIMIZED BY */
/*  SUBROUTINE MIN. */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 159-160 */

/*  SUBROUTINE FLIN IS CALLED BY SUBROUTINE MIN. */

int flin(double *xlg, int *j, double (*f)(), double *fln)
{
  /* Local variables */

  static int i;
  static double qa, qb, qc, tt[N1];
  
  if(*j > 0) {   
    /*  LINEAR SEARCH... */

    for(i = 1; i <= cprax.n; i++) {
      // tt[i - 1] = cprax.x[i - 1] + *xlg * cprax.v[i + 20 * *j - 21];

      tt[i - 1] = cprax.x[i - 1] + *xlg * cprax.v[a(i, *j, 20, 21)];
    }    
  } else {
    /*  SEARCH ALONG A PARABOLIC SPACE CURVE. */

    qa = *xlg * (*xlg - cprax.qd1) / (cprax.qd0 * (cprax.qd0 + cprax.qd1));
    qb = (*xlg + cprax.qd0) * (cprax.qd1 - *xlg) / (cprax.qd0 * cprax.qd1);
    qc = *xlg * (*xlg + cprax.qd0) / (cprax.qd1 * (cprax.qd0 + cprax.qd1));

    for(i = 1; i <= cprax.n; i++) {
      tt[i - 1] = qa * cprax.q0[i - 1] + qb * cprax.x[i - 1]
        + qc * cprax.q1[i - 1];
    }
  }

  /*  INCREMENT FUNCTION EVALUATION COUNTER. */

  ++cprax.nf;

  *fln = (*f)(tt, &cprax.n);

  /*  END FLIN */

  return 0;
} 

/* Subroutine */

/*  AN IMPROVED VERSION OF MINFIT, RESTRICTED TO M = N, P = 0. */
/*  SEE GOLUB AND REINSCH (1970). */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 156-158 */

/*  G. H. GOLUB AND C. REINSCH, */
/*  "SINGULAR VALUE DECOMPOSITION AND LEAST SQUARES SOLUTIONS', */
/*  NUMERISCHE MATHEMATIK 14 (1970) PAGES 403-420 */

/*  THE SINGULAR VALUES OF THE ARRAY AB ARE RETURNED IN Q, */
/*  AND AB IS OVERWRITTEN WITH THE ORTHOGONAL MATRIX V SUCH THAT */
/*  U.DIAG(Q)=AB.V, WHERE U IS ANOTHER ORTHOGONAL MATRIX. */

/*  SUBROUTINE MINFIT IS CALLED BY SUBROUTINE PRAXIS. */

int minfit(int *n, double *eps, double *tol, double *ab, double *q, double *e,
           int *nmax, int *lp)
{
  /* System generated locals */

  int ab_dim1, ab_offset;

  /* Local variables */

  static double zero, c, f, g, h;
  static int i, j, k, l;
  static double s, x, y, z, denom;
  static int l2, ii, kk, kt, jthirt, ll2;
  static double one;
  static int lpi;
  static double two;
  static int new;
  
  /* Parameter adjustments */

  --e;
  --q;  

  ab_dim1 = *nmax;

  ab_offset = 1 + ab_dim1 * 1;

  ab -= ab_offset;
  
  /* Function Body */
  
  jthirt = 30;

  new = *n;
  
  zero = 0.0;
  one  = 1.0;
  two  = 2.0;

  /*  HOUSEHOLDER'S REDUCTION TO BIDIAGONAL FORM... */

  x = zero;
  g = zero;

  for(i = 1; i <= new; i++) {
    e[i] = g;

    s = zero;

    l = i + 1;
    
    for(j = i; j <= new; j++) {
      s += ab[j + i * ab_dim1] * ab[j + i * ab_dim1];      
    }

    if(s < *tol) {
      g = zero;
    } else {
      f = ab[i + i * ab_dim1];

      if(f < zero) {
        g = sqrt(s);
      } else {
        g = - sqrt(s);
      }

      h = f * g - s;

      ab[i + i * ab_dim1] = f - g;

      if(l > *n) {
        goto L60;
      }

      for(j = l; j <= new; j++) {
        f = zero;

	if(i > *n) {
          goto L30;
        }

        for(k = i; k <= new; k++) {
          f += ab[k + i * ab_dim1] * ab[k + j * ab_dim1];
        }

      L30:

        f /= h;

        if(i > *n) {
          goto L50;
        }

        for(k = i; k <= new; k++) {
          ab[k + j * ab_dim1] += f * ab[k + i * ab_dim1];
        }       
        
      L50:

	;

        /*    ;   */
      }
    }

  L60:

    q[i] = g;
    s    = zero;

    if(i <= *n) {
      if(l > *n) {
        goto L80;
      }

      for(j = l; j <= new; j++) {
        s += ab[i + j * ab_dim1] * ab[i + j * ab_dim1];
      }
    }

  L80:

    if(s < *tol) {
      g = zero;
    } else {
      f = ab[i + (i + 1) * ab_dim1];

      if(f < zero) {
        g = sqrt(s);
      } else {
        g = - sqrt(s);
      }

      h = f * g - s;

      ab[i + (i + 1) * ab_dim1] = f - g;

      if(l > *n) {
        goto L130;
      }

      for(j = l; j <= new; j++) {
        e[j] = ab[i + j * ab_dim1] / h;
      }

      for(j = l; j <= new; j++) {       
        s = zero;

        for(k = l; k <= new; k++) {
          s += ab[j + k * ab_dim1] * ab[i + k * ab_dim1];
        }

	for(k = l; k <= new; k++) {
          ab[j + k * ab_dim1] += s * e[k];
        }
      }
    }

  L130:

    y = fabs(q[i]) + fabs(e[i]);
      
    if(y > x) {
      x = y;
    }
  }

  /*  ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS... */

  for(ii = 1; ii <= new; ii++) {
    i = *n - ii + 1;

    if(g != zero) {
      h = ab[i + (i + 1) * ab_dim1] * g;

      if(l > *n) {
        goto L200;
      }

      for(j = l; j <= new; j++) {
        ab[j + i * ab_dim1] = ab[i + j * ab_dim1] / h;
      }

      for(j = l; j <= new; j++) {
        s = zero;

        for(k = l; k <= new; k++) {
          s += ab[i + k * ab_dim1] * ab[k + j * ab_dim1];
        }

	for(k = l; k <= new; k++) {
          ab[k + j * ab_dim1] += s * ab[k + i * ab_dim1];
        }
      }
    }

    if(l > *n) {
      goto L200;
    }
    
    for(j = l; j <= new; j++) {
      ab[j + i * ab_dim1] = zero;
      ab[i + j * ab_dim1] = zero;
    }

  L200:

    ab[i + i * ab_dim1] = one;

    g = e[i];

    l = i;
  }

  /*  DIAGONALIZATION OF THE BIDIAGONAL FORM... */

  *eps *= x;

  for(kk = 1; kk <= new; kk++) {
    k  = *n - kk + 1;
    kt = 0;

    /*  LABEL TESTFSPLITTING... */

  L220:

    ++kt;
        
    if(kt > jthirt) {
      e[k] = zero;
      
      printf("QR FAILED. \n");
    }

    for(ll2 = 1; ll2 <= k; ll2++) {
      l2 = k - ll2 + 1;
      l  = l2;

      /*  IF(...) GO TO TESTFCONVERGENCE */

      if(fabs(e[l]) <= *eps) {
        goto L270;
      }

      /*  IF(...) GO TO CANCELLATION */

      if(fabs(q[l - 1]) <= *eps) {
        goto L250;
      }
    }

    /*  CANCELLATION OF E(L) IF L IS GREATER THAN 1... */
    /*  LABEL CANCELLATION... */

  L250:

    c = zero;
    s = one;

    if(l > k) {
      goto L270;
    }

    for(i = l; i <= k; i++) {
      f = s * e[i];

      e[i] = c * e[i];

      /*  IF(...) GO TO TESTFCONVERGENCE */

      if(fabs(f) <= *eps) {
        goto L270;
      }

      g = q[i];

      if(fabs(f) < fabs(g)) {
        h = fabs(g) * sqrt(one + (f / g) * (f / g));
      } else if(f != zero) {
        h = fabs(f) * sqrt(one + (g / f) * (g / f));
      } else {
        h = zero;
      }

      q[i] = h;

      if(h == zero) {
        h = one;
        g = one;
      }

      /*  THE ABOVE REPLACES Q(I) AND H BY SQUARE ROOT OF (G * G + F * F) */
      /*  WHICH MAY GIVE INCORRECT RESULTS IF THE SQUARES UNDERFLOW OR IF */
      /*  F = G = 0 . */

      c = g / h;
      s = - f / h;
    }

    /*  LABEL TESTFCONVERGENCE... */

  L270:

    z = q[k];

    /*  IF(...) GO TO CONVERGENCE */

    if(l == k) {
      goto L310;
    }

    /*  SHIFT FROM BOTTOM 2 * 2 MINOR. */

    x = q[l];
    y = q[k - 1];
    g = e[k - 1];
    h = e[k];
    f = ((y - z) * (y + z) + (g - h) * (g + h)) / (two * h * y);
    g = sqrt(f * f + one);

    if(f < zero) {
      denom = f - g;
    } else {
      denom = f + g;
    }

    f = ((x - z) * (x + z) + h * (y / denom - h)) / x;

    /*  NEXT QR TRANSFORMATION... */

    s = one;
    c = one;

    lpi = l + 1;

    if(lpi > k) {
      goto L300;
    }

    for(i = lpi; i <= k; i++) {
      g  = e[i];
      y  = q[i];
      h  = s * g;
      g *= c;

      if(fabs(f) < fabs(h)) {
        z = fabs(h) * sqrt(one + (f / h) * (f / h));
      } else if (f != zero) {
        z = fabs(f) * sqrt(one + (h / f) * (h / f));
      } else {
        z = zero;
      }

      e[i - 1] = z;

      if(z == zero) {
        f = one;
        z = one;
      }

      c = f / z;
      s = h / z;
      f = x * c + g * s;
      g = - x * s + g * c;
      h = y * s;
      y *= c;

      for(j = 1; j <= new; j++) {
        x = ab[j + (i - 1) * ab_dim1];
        z = ab[j + i * ab_dim1];

        ab[j + (i - 1) * ab_dim1] = x * c + z * s;
        ab[j + i * ab_dim1]       = - x * s + z * c;
      }

      if(fabs(f) < fabs(h)) {
        z = fabs(h) * sqrt(one + (f / h) * (f / h));
      } else if(f != zero) {
        z = fabs(f) * sqrt(one + (h / f) * (h / f));
      } else {
        z = zero;
      }

      q[i - 1] = z;

      if(z == zero) {
        f = one;
        z = one;
      }

      c = f / z;
      s = h / z;
      f = c * g + s * y;
      x = - s * g + c * y;
    }

  L300:

    e[l] = zero;
    e[k] = f;
    q[k] = x;

    /*  GO TO TESTFSPLITTING */

    goto L220;

    /*  LABEL CONVERGENCE... */

  L310:

    if(z < zero) {
      /*  Q(K) IS MADE NON-NEGATIVE. */

      q[k] = - z;

      for(j = 1; j <= new; j++) {
        ab[j + k * ab_dim1] = - ab[j + k * ab_dim1];
      }
    }
  }

  /*  END MINFIT */

  return 0;
}

/* Subroutine */

/*  THIS SUBROUTINE SORTS THE ELEMENTS OF D */
/*  AND THE CORRESPONDING COLUMNS OF V INTO DESCENDING ORDER. */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 158-159 */

int sort()
{
  /* Local variables */

  static int i, j, k;
  static double s;
  static int ipi, nmi;

  nmi = cprax.n - 1;

  if(nmi < 1) {
    goto L50;
  }

  for(i = 1; i <= nmi; i++) {
    k = i;

    s = cprax.d[i - 1];

    ipi = i + 1;

    if(ipi > cprax.n) {
      goto L20;
    }

    for(j = ipi; j <= cprax.n; j++) {
      if(cprax.d[j - 1] > s) {
        k = j;
        
        s = cprax.d[j - 1];     
      }
    }

  L20:

    if(k > i) {
      cprax.d[k - 1] = cprax.d[i - 1];
      cprax.d[i - 1] = s;

      for(j = 1; j <= cprax.n; j++) {
        s = cprax.v[a(j, i, N1, N2)];

	cprax.v[a(j, i, N1, N2)] = cprax.v[a(j, k, N1, N2)];
        cprax.v[a(j, k, N1, N2)] = s;
      }
    }
  }

 L50:

  /*  END SORT */

  return 0;
}

/* Subroutine */

/*  SUBROUTINE RANINI PERFORMS INITIALIZATION FOR SUBROUTINE RANDOM. */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 163-164 */

int ranini(double *rvalue)
{
  /* System generated locals */

  double temp;

  /* Local variables */

  static int i;
  static double r;

  temp = fabs(*rvalue);
  r    = dmod(&temp, &c_b310) + 1.0;

  comran.jran2 = 127;

 L10:

  if(comran.jran2 > 0) {
    --comran.jran2;
    comran.ran1 = -36028797018963968.0;

    for(i = 1; i <= 7; i++) {
      temp = 1756.0 * r;
      r    = dmod(&temp, &c_b313);

      comran.ran1 = (comran.ran1 + (r - dmod(&r, &c_b314)) / 32.0) / 256.0;
    }

    comran.ran3[comran.jran2] = comran.ran1;

    goto L10;
  }
  
  /*  END RANINI */

  return 0;    
}

/* Subroutine */

/*  SUBROUTINE RANDOM RETURNS A DOUBLE PRECISION PSEUDORANDOM NUMBER */
/*  UNIFORMLY DISTRIBUTED IN (0, 1) (INCLUDING 0 BUT NOT 1). */

/*  "ALGORITHMS FOR MINIMIZATION WITHOUT DERIVATIVES", */
/*  RICHARD P. BRENT, PRENTICE-HALL 1973, PAGES 163-164 */

/*  BEFORE THE FIRST CALL TO RANDOM, THE USER MUST */
/*  CALL RANINI(R) ONCE (ONLY) WITH R A DOUBLE PRECISION NUMBER */
/*  EQUAL TO ANY INTEGER VALUE. */
/*  BRENT (PAGE 166) USED THE EQUIVALENT OF */
/*     CALL RANINI(4.0D0) . */

/*  THE ALGORITHM USED IN SUBROUTINE RANDOM RETURNS X(N)/2**56, */
/*  WHERE   X(N) = X(N - 1) + X(N - 127)  (MOD 2**56) . */
/*  SINCE (1 + X + X**127) IS PRIMITIVE (MOD 2), */
/*  THE PERIOD IS AT LEAST (2**127 - 1), WHICH EXCEEDS 10**38. */

/*  SEE "SEMINUMERICAL ALGORITHMS", VOLUME 2 OF */
/*  "THE ART OF COMPUTER PROGRAMMING" BY DONALD E. KNUTH, */
/*  ADDISON-WESLEY 1969, PAGES 26, 34, AND 464. */

/*  X(N) IS STORED IN DOUBLE PRECISION AS  RAN3 = X(N)/2**56 - 1/2, */
/*  AND ALL DOUBLE PRECISION ARITHMETIC IS EXACT. */

int prandom(double *ranval)
{
  if(comran.jran2 == 0) {
    comran.jran2 = 126;
  } else {
    --comran.jran2;
  }

  comran.ran1 += comran.ran3[comran.jran2];

  if(comran.ran1 < 0.) {
    comran.ran1 += 0.5;
  } else {
    comran.ran1 += - 0.5;
  }

  comran.ran3[comran.jran2] = comran.ran1;
  
  *ranval = comran.ran1 + 0.5;
    
  /*  END RANDOM */

  return 0;     
}

/* Subroutine */

/*  SIMPLE PORTABLE PSEUDORANDOM NUMBER GENERATOR. */

/*  DRANDM RETURNS FUNCTION VALUES THAT ARE PSEUDORANDOM */
/*  NUMBERS UNIFORMLY DISTRIBUTED ON THE INTERVAL (0,1). */

/*  'NUMERICAL MATHEMATICS AND COMPUTING' BY WARD CHENEY AND */
/*  DAVID KINCAID, BROOKS/COLE PUBLISHING COMPANY */
/*  (FIRST EDITION, 1980), PAGE 203 */

/*  AT THE BEGINNING OF EXECUTION, OR WHENEVER A NEW SEQUENCE IS */
/*  TO BE INITIATED, SET DL EQUAL TO AN INTEGER VALUE BETWEEN */
/*  1.0D0 AND 2147483646.0D0, INCLUSIVE.  DO THIS ONLY ONCE. */
/*  THEREAFTER, DO NOT SET OR ALTER DL IN ANY WAY. */
/*  FUNCTION DRANDM WILL MODIFY DL FOR ITS OWN PURPOSES. */

/*  DRANDM USES A MULTIPLICATIVE CONGRUENTIAL METHOD. */
/*  THE NUMBERS GENERATED BY DRANDM SUFFER FROM THE PARALLEL */
/*  PLANES DEFECT DISCOVERED BY G. MARSAGLIA, AND SHOULD NOT BE */
/*  USED WHEN HIGH-QUALITY RANDOMNESS IS REQUIRED.  IN THAT */
/*  CASE, USE A "SHUFFLING" METHOD. */

double drandm(double *dl)
{
  /* System generated locals */

  double ret_val, temp;

 L10:

  temp = *dl * 16807.0;
  *dl  = dmod(&temp, &c_b318);

  ret_val = *dl / 2147483647.0;

  if(ret_val <= 0.0 || ret_val >= 1.0) {
    goto L10;
  }

  return ret_val;
}

double dmod(double *x, double *y)
{
  double quotient;

  if((quotient = *x / *y) >= 0) {
    quotient = floor(quotient);
  } else {
    quotient = - floor(- quotient);
  }

  return(*x - (*y) * quotient);
}

double powd(double *ap, int *bp)
{
  double pow, x;
  int n;
  unsigned long u;

  pow = 1;
  x   = *ap;
  n   = *bp;

  if(n != 0) {
    if(n < 0) {
      n = - n;
      
      x = 1 / x;
    }

    for(u = n; ; ) {
      if(u & 01)
        pow *= x;

      if(u >>= 1) {
        x *= x;
      } else {
        break;
      } 
    }
  }

 return(pow);
}
