/* praxis.f -- translated by f2c (version 20020621) */

/*                             LAST MODIFIED 3/1/73                       */

/*     PRAXIS RETURNS THE MINIMUM OF THE FUNCTION F(X,N) OF N VARIABLES   */
/*     USING THE PRINCIPAL AXIS METHOD.  THE GRADIENT OF THE FUNCTION IS  */
/*     NOT REQUIRED.                                                      */

/*     FOR A DESCRIPTION OF THE ALGORITHM, SEE CHAPTER SEVEN OF           */
/*     "ALGORITHMS FOR FINDING ZEROS AND EXTREMA OF FUNCTIONS WITHOUT     */
/*     CALCULATING DERIVATIVES" BY RICHARD P BRENT.                       */

/*     THE PARAMETERS ARE:                                                */

/*     T0       IS A TOLERANCE.  PRAXIS ATTEMPTS TO RETURN PRAXIS=F(X)    */
/*              SUCH THAT IF X0 IS THE TRUE LOCAL MINIMUM NEAR X, THEN    */
/*              NORM(X-X0) < T0 + SQUAREROOT(MACHEP)*NORM(X).             */
/*     MACHEP   IS THE MACHINE PRECISION, THE SMALLEST NUMBER SUCH THAT   */
/*              1 + MACHEP > 1.  MACHEP SHOULD BE 16.**-13 (ABOUT         */
/*              2.22D-16) FOR REAL*8 ARITHMETIC ON THE IBM 360.           */
/*     H0       IS THE MAXIMUM STEP SIZE.  H0 SHOULD BE SET TO ABOUT THE  */
/*              MAXIMUM DISTANCE FROM THE INITIAL GUESS TO THE MINIMUM.   */
/*              (IF H0 IS SET TOO LARGE OR TOO SMALL, THE INITIAL RATE OF */
/*              CONVERGENCE MAY BE SLOW.)                                 */
/*     N        (AT LEAST TWO) IS THE NUMBER OF VARIABLES UPON WHICH      */
/*              THE FUNCTION DEPENDS.                                     */
/*     PRIN     CONTROLS THE PRINTING OF INTERMEDIATE RESULTS.            */
/*              IF PRIN=0, NOTHING IS PRINTED.                            */
/*              IF PRIN=1, F IS PRINTED AFTER EVERY N+1 OR N+2 LINEAR     */
/*              MINIMIZATIONS.  FINAL X IS PRINTED, BUT INTERMEDIATE X IS */
/*              PRINTED ONLY IF N IS AT MOST 4.                           */
/*              IF PRIN=2, THE SCALE FACTORS AND THE PRINCIPAL VALUES OF  */
/*              THE APPROXIMATING QUADRATIC FORM ARE ALSO PRINTED.        */
/*              IF PRIN=3, X IS ALSO PRINTED AFTER EVERY FEW LINEAR       */
/*              MINIMIZATIONS.                                            */
/*              IF PRIN=4, THE PRINCIPAL VECTORS OF THE APPROXIMATING     */
/*              QUADRATIC FORM ARE ALSO PRINTED.                          */
/*     X        IS AN ARRAY CONTAINING ON ENTRY A GUESS OF THE POINT OF   */
/*              MINIMUM, ON RETURN THE ESTIMATED POINT OF MINIMUM.        */
/*     F(X,N)   IS THE FUNCTION TO BE MINIMIZED.  F SHOULD BE A REAL*8    */
/*              FUNCTION DECLARED EXTERNAL IN THE CALLING PROGRAM.        */
/*     FMIN     IS AN ESTIMATE OF THE MINIMUM, USED ONLY IN PRINTING      */
/*              INTERMEDIATE RESULTS.                                     */
/*     THE APPROXIMATING QUADRATIC FORM IS                                */
/*              Q(X') = F(X,N) + (1/2) * (X'-X)-TRANSPOSE * A * (X'-X)    */
/*     WHERE X IS THE BEST ESTIMATE OF THE MINIMUM AND A IS               */
/*              INVERSE(V-TRANSPOSE) * D * INVERSE(V)                     */
/*     (V(*,*) IS THE MATRIX OF SEARCH DIRECTIONS; D(*) IS THE ARRAY      */
/*     OF SECOND DIFFERENCES).  IF F HAS CONTINUOUS SECOND DERIVATIVES    */
/*     NEAR X0, A WILL TEND TO THE HESSIAN OF F AT X0 AS X APPROACHES X0. */

/*     IT IS ASSUMED THAT ON FLOATING-POINT UNDERFLOW THE RESULT IS SET   */
/*     TO ZERO. */
/*     THE USER SHOULD OBSERVE THE COMMENT ON HEURISTIC NUMBERS AFTER     */
/*     THE INITIALIZATION OF MACHINE DEPENDENT NUMBERS.                   */

#include <stdio.h>
#include <math.h>

#define FALSE_ 0
#define TRUE_  1
#define min_(a, b)  ((a) <= (b) ? (a) : (b))
#define max_(a, b)  ((a) >= (b) ? (a) : (b))

/* Common Block Declarations */

struct {
  double fx, ldt, dmin_;
  int nf, nl;
} global;

/* v[400] was [20][20] */

struct {
  double v[400], q0[20], q1[20], qa, qb, qc, qd0, qd1, qf1;
} q;

/* Table of constant values */

static int c_0 = 0;
static int c_1 = 1;
static int c_2 = 2;
static int c_3 = 3;
static int c_4 = 4;
static int c_10 = 10;

static int c_false = FALSE_;
static int c_true = TRUE_;

double praxis(double *t0, double *machep, double *h0, 
	      int *n, int *prin, double *x, double (*f)(), double *fmin)
{
  /* System generated locals */
  
  int i_1, i_2, i_3;
  double ret_val, d_1;

  /* Builtin functions */

  double fabs(double);  
  double sqrt(double);
  int pow_ii(int *, int *);
  
  /* Local variables */

  static int i_, j, k;
  static int k2;
  static int kl, ii;
  static int kt;
  static int im1, km1;
  static int ktm;
  static int idim;
  static int illc;
  static int klmk;  
  static double d_[20], h_;
  static double s, t, y[20], z_[20], f1;
  static double m2, m4, t2, df, dn;
  static double sf;
  static double sl;
  static double dni, lds;
  static double scbd;
  static double ldfac, large, small, value;
  static double vlarge;
  static double vsmall;
  
  /* Subroutines */ 

  extern int min(int *, int *, int *, double *, double *, double *, int *, 
		 double (*f)(), double *, double *, double *, double *);
  extern int quad(int *, double (*f)(), double *, double *, double *, 
		  double *);
  extern int sort(int *, int *, double *, double *);
  extern int print(int *, double *, int *, double *);
  extern double random(int *);
  extern int minfit(int *, int *, double *, double *, double *, double *);
  extern int maprnt(int *, double *, int *, int *); 
  extern int vcprnt(int *, double *, int *);
  
  /* .....IF N>20 OR IF N<20 AND YOU NEED MORE SPACE, CHANGE '20' TO THE  */
  /*     LARGEST VALUE OF N IN THE NEXT CARD, IN THE CARD 'IDIM=20', AND  */
  /*     IN THE DIMENSION STATEMENTS IN SUBROUTINES MINFIT,MIN,FLIN,QUAD. */
  
  /* .....INITIALIZATION..... */
  /*     MACHINE DEPENDENT NUMBERS: */
  
  /* Parameter adjustments */
  
  --x;
  
  /* Function Body */
  
  small = *machep * *machep;
  vsmall = small * small;
  large = 1.0 / small;
  vlarge = 1.0 / vsmall;
  m2 = sqrt(*machep);
  m4 = sqrt(m2);
  
  /*     HEURISTIC NUMBERS:                                             */
  /*     IF THE AXES MAY BE BADLY SCALED (WHICH IS TO BE AVOIDED IF     */
  /*     POSSIBLE), THEN SET SCBD=10.  OTHERWISE SET SCBD=1.            */
  /*     IF THE PROBLEM IS KNOWN TO BE ILL-CONDITIONED, SET ILLC=TRUE.  */
  /*     OTHERWISE SET ILLC=FALSE.                                      */
  /*     KTM IS THE NUMBER OF ITERATIONS WITHOUT IMPROVEMENT BEFORE THE */
  /*     ALGORITHM TERMINATES.  KTM=4 IS VERY CAUTIOUS; USUALLY KTM=1   */
  /*     IS SATISFACTORY.                                               */

  scbd = 1.;
  illc = FALSE_;
  ktm = 1;
  
  ldfac = 0.01;
  
  if(illc) {
    ldfac = 0.1;
  }
   
  kt = 0;
  global.nl = 0;
  global.nf = 1;
  global.fx = (*f)(&x[1], n);
  q.qf1 = global.fx;
  t = small + fabs(*t0);
  t2 = t;
  global.dmin_ = small;
  h_ = *h0;
  
  if(h_ < t * 100) {
    h_ = t * 100;
  }
  
  global.ldt = h_;

  /* .....THE FIRST SET OF SEARCH DIRECTIONS V IS THE IDENTITY MATRIX..... */
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    i_2 = *n;
    for(j = 1; j <= i_2; ++j) {
      q.v[i_ + j * 20 - 21] = 0.0;
    }
    
    q.v[i_ + i_ * 20 - 21] = 1.0;
  }
  
  d_[0] = 0.0;
  q.qd0 = 0.0;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    q.q0[i_ - 1] = x[i_];
    q.q1[i_ - 1] = x[i_];
  }
  
  if(*prin > 0) {
    print(n, &x[1], prin, fmin);
  }
  
  /* .....THE MAIN LOOP STARTS HERE..... */

 L40:
  
  sf = d_[0];
  d_[0] = 0.;
  s = 0.0;

  /* .....MINIMIZE ALONG THE FIRST DIRECTION V(*,1). */
  /*     FX MUST BE PASSED TO MIN BY VALUE. */

  value = global.fx;
  
  min(n, &c_1, &c_2, d_, &s, &value, &c_false, f, &x[1], &t, machep, &h_);
  
  if(s > 0.0) {
    goto L50;
  }
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    q.v[i_ - 1] = -q.v[i_ - 1];
  }

 L50:
  
  if(sf > d_[0] * 0.9 && sf * 0.9 < d_[0]) {
    goto L70;
  }
  
  i_1 = *n;
  
  for(i_ = 2; i_ <= i_1; ++i_) {
    d_[i_ - 1] = 0.0;
  }
  
  /* .....THE INNER LOOP STARTS HERE..... */

 L70:

  i_1 = *n;
  
  for(k = 2; k <= i_1; ++k) {
    i_2 = *n;
    for (i_ = 1; i_ <= i_2; ++i_) {
      y[i_ - 1] = x[i_];
    }
	
    sf = global.fx;
    
    if(kt > 0) {
      illc = TRUE_;
    }

  L80:
	
    kl = k;
    df = 0.0;
    
    /* .....A RANDOM STEP FOLLOWS (TO AVOID RESOLUTION VALLEYS).        */
    /*     PRAXIS ASSUMES THAT RANDOM RETURNS A RANDOM NUMBER UNIFORMLY */
    /*     DISTRIBUTED IN (0, 1).                                       */

    if(! illc) {
      goto L95;
    }
    
    i_2 = *n;
    
    for(i_ = 1; i_ <= i_2; ++i_) {
      s = (global.ldt * 0.1 + t2 * pow_ii(&c_10, &kt)) * (random(n) - 0.5);
      z_[i_ - 1] = s;
      i_3 = *n;
	
      for(j = 1; j <= i_3; ++j) {
	x[j] += s * q.v[j + i_ * 20 - 21];
      }
    }
    
    global.fx = (*f)(&x[1], n);
    ++global.nf;

    /* .....MINIMIZE ALONG THE "NON-CONJUGATE" DIRECTIONS V(*,K),...,V(*,N) */

  L95:
	
    i_2 = *n;
	
    for(k2 = k; k2 <= i_2; ++k2) {
      sl = global.fx;
      s = 0.0;
      value = global.fx;
      
      min(n, &k2, &c_2, &d_[k2 - 1], &s, &value, &c_false, f, &x[1], &t, 
	  machep, &h_);
      
      if(illc) {
	goto L97;
      }
      
      s = sl - global.fx;
      
      goto L99;

    L97:
      
      /* Computing 2nd power */
      
      d_1 = s + z_[k2 - 1];
      s = d_[k2 - 1] * (d_1 * d_1);
    
    L99:
	
      if(df > s) {
	goto L105;
      }
      
      df = s;
      kl = k2;

    L105:

      ;
    }
	
    if(illc || df >= (d_1 = *machep * 100 * global.fx, fabs(d_1))) {
      goto L110;
    }
    

    /* .....IF THERE WAS NOT MUCH IMPROVEMENT ON THE FIRST TRY, SET */
    /*     ILLC=TRUE AND START THE INNER LOOP AGAIN.....            */

    illc = TRUE_;
    
    goto L80;

  L110:
    
    if(k == 2 && *prin > 1) {
      vcprnt(&c_1, d_, n);
    }
    
    /* .....MINIMIZE ALONG THE "CONJUGATE" DIRECTIONS V(*,1),...,V(*,K-1) */

    km1 = k - 1;
    i_2 = km1;
    
    for(k2 = 1; k2 <= i_2; ++k2) {
      s = 0.0;
      value = global.fx;
      
      min(n, &k2, &c_2, &d_[k2 - 1], &s, &value, &c_false, f, &x[1], &t, 
	  machep, &h_);
    }
	
    f1 = global.fx;
    global.fx = sf;
    lds = 0.0;
    i_2 = *n;
	
    for(i_ = 1; i_ <= i_2; ++i_) {
      sl = x[i_];
      x[i_] = y[i_ - 1];
      sl -= y[i_ - 1];
      y[i_ - 1] = sl;
      lds += sl * sl;
    
    }
    
    lds = sqrt(lds);
    
    if(lds <= small) {
      goto L160;
    }

    /* .....DISCARD DIRECTION V(*,KL).                                  */
    /*     IF NO RANDOM STEP WAS TAKEN, V(*,KL) IS THE "NON-CONJUGATE"  */
    /*     DIRECTION ALONG WHICH THE GREATEST IMPROVEMENT WAS MADE..... */

    klmk = kl - k;
    
    if(klmk < 1) {
      goto L141;
    }
	
    i_2 = klmk;
	
    for(ii = 1; ii <= i_2; ++ii) {
      i_ = kl - ii;
      i_3 = *n;
      
      for(j = 1; j <= i_3; ++j) {
	q.v[j + (i_ + 1) * 20 - 21] = q.v[j + i_ * 20 - 21];
      }
      
      d_[i_] = d_[i_ - 1];
    }

  L141:
    
    d_[k - 1] = 0.0;
    i_2 = *n;
	
    for(i_ = 1; i_ <= i_2; ++i_) {
      q.v[i_ + k * 20 - 21] = y[i_ - 1] / lds;
    }
    
    /* .....MINIMIZE ALONG THE NEW "CONJUGATE" DIRECTION V(*,K), WHICH IS */
    /*     THE NORMALIZED VECTOR:  (NEW X) - (0LD X).....                 */
    
    value = f1;
    
    min(n, &k, &c_4, &d_[k - 1], &lds, &value, &c_true, f, &x[1], &t, 
	machep, &h_);
	
    if(lds > 0.0) {
      goto L160;
    }
    
    lds = -lds;
    i_2 = *n;
    
    for(i_ = 1; i_ <= i_2; ++i_) {
      q.v[i_ + k * 20 - 21] = -q.v[i_ + k * 20 - 21];
    }

  L160:
    
    global.ldt = ldfac * global.ldt;
	
    if(global.ldt < lds) {
      global.ldt = lds;
    }
    
    if(*prin > 0) {
      print(n, &x[1], prin, fmin);
    }
    
    t2 = 0.0;
    i_2 = *n;
	
    for(i_ = 1; i_ <= i_2; ++i_) {
      /* Computing 2nd power */
      
      d_1 = x[i_];
      t2 += d_1 * d_1;
    }
    
    t2 = m2 * sqrt(t2) + t;

    /* .....SEE WHETHER THE LENGTH OF THE STEP TAKEN SINCE STARTING THE */
    /*     INNER LOOP EXCEEDS HALF THE TOLERANCE.....                   */
    
    if(global.ldt > t2 * 0.5f) {
      kt = -1;
    }
    
    ++kt;
    
    if(kt > ktm) {
      goto L400;
    }
  }

  /* .....THE INNER LOOP ENDS HERE. */
  
  /*     TRY QUADRATIC EXTRAPOLATION IN CASE WE ARE IN A CURVED VALLEY. */

  quad(n, f, &x[1], &t, machep, &h_);
  
  dn = 0.0;
  i_1 = *n;
    
  for(i_ = 1; i_ <= i_1; ++i_) {
    d_[i_ - 1] = 1.0 / sqrt(d_[i_ - 1]);
    
    if(dn < d_[i_ - 1]) {
      dn = d_[i_ - 1];
    }
  }
  
  if(*prin > 3) {
    maprnt(&c_1, q.v, &idim, n);
  }
  
  i_1 = *n;
  
  for(j = 1; j <= i_1; ++j) {
    s = d_[j - 1] / dn;
    i_2 = *n;
    
    for(i_ = 1; i_ <= i_2; ++i_) {
      q.v[i_ + j * 20 - 21] = s * q.v[i_ + j * 20 - 21];
    }
  }
  
  /* .....SCALE THE AXES TO TRY TO REDUCE THE CONDITION NUMBER..... */
  
  if(scbd <= 1.) {
    goto L200;
  }
  
  s = vlarge;
  i_2 = *n;
  
  for(i_ = 1; i_ <= i_2; ++i_) {
    sl = 0.0;
    i_1 = *n;
	
    for(j = 1; j <= i_1; ++j) {
      sl += q.v[i_ + j * 20 - 21] * q.v[i_ + j * 20 - 21];
    }
	
    z_[i_ - 1] = sqrt(sl);
	
    if(z_[i_ - 1] < m4) {
      z_[i_ - 1] = m4;
    }
    
    if(s > z_[i_ - 1]) {
      s = z_[i_ - 1];
    }
  }
  
  i_2 = *n;
  
  for(i_ = 1; i_ <= i_2; ++i_) {
    sl = s / z_[i_ - 1];
    z_[i_ - 1] = 1. / sl;
    
    if(z_[i_ - 1] <= scbd) {
      goto L189;
    }
	
    sl = 1.0 / scbd;
    z_[i_ - 1] = scbd;

L189:
    
    i_1 = *n;
    
    for (j = 1; j <= i_1; ++j) {
      q.v[i_ + j * 20 - 21] = sl * q.v[i_ + j * 20 - 21];
    }
  }
  
  /* .....CALCULATE A NEW SET OF ORTHOGONAL DIRECTIONS BEFORE REPEATING */
  /*     THE MAIN LOOP.                                                 */
  /*     FIRST TRANSPOSE V FOR MINFIT:                                  */

 L200:
  
  i_2 = *n;
  
  for(i_ = 2; i_ <= i_2; ++i_) {
    im1 = i_ - 1;
    i_1 = im1;
    
    for(j = 1; j <= i_1; ++j) {
      s = q.v[i_ + j * 20 - 21];
      q.v[i_ + j * 20 - 21] = q.v[j + i_ * 20 - 21];
      q.v[j + i_ * 20 - 21] = s;
	}
  }
  
  /* .....CALL MINFIT TO FIND THE SINGULAR VALUE DECOMPOSITION OF V. */
  /*     THIS GIVES THE PRINCIPAL VALUES AND PRINCIPAL DIRECTIONS OF THE */
  /*     APPROXIMATING QUADRATIC FORM WITHOUT SQUARING THE CONDITION */
  /*     NUMBER..... */
  
  minfit(&idim, n, machep, &vsmall, q.v, d_);
  
  /* .....UNSCALE THE AXES..... */
  
  if(scbd <= 1.0) {
    goto L250;
  }
  
  i_2 = *n;
  
  for(i_ = 1; i_ <= i_2; ++i_) {
    s = z_[i_ - 1];
    i_1 = *n;
    
    for(j = 1; j <= i_1; ++j) {
      q.v[i_ + j * 20 - 21] = s * q.v[i_ + j * 20 - 21];
    }
  }
  
  i_2 = *n;
  
  for(i_ = 1; i_ <= i_2; ++i_) {
    s = 0.0;
    i_1 = *n;
    
    for(j = 1; j <= i_1; ++j) {
      /* Computing 2nd power */
      
      d_1 = q.v[j + i_ * 20 - 21];
      s += d_1 * d_1;
    }
	
    s = sqrt(s);
    d_[i_ - 1] = s * d_[i_ - 1];
    s = 1 / s;
    i_1 = *n;
	
    for(j = 1; j <= i_1; ++j) {
      q.v[j + i_ * 20 - 21] = s * q.v[j + i_ * 20 - 21];
    }
  }
  
 L250:
 
  i_2 = *n;
  
  for(i_ = 1; i_ <= i_2; ++i_) {
    dni = dn * d_[i_ - 1];
    
    if(dni > large) {
      goto L265;
    }
    
    if(dni < small) {
      goto L260;
    }
    
    d_[i_ - 1] = 1 / (dni * dni);
    goto L270;
  
  L260:
    
    d_[i_ - 1] = vlarge;
    goto L270;

  L265:
    
    d_[i_ - 1] = vsmall;

  L270:
    
    ;
  }

  /* .....SORT THE EIGENVALUES AND EIGENVECTORS..... */

  sort(&idim, n, d_, q.v);
  
  global.dmin_ = d_[*n - 1];
  
  if(global.dmin_ < small) {
    global.dmin_ = small;
  }
  
  illc = FALSE_;
  
  if(m2 * d_[0] > global.dmin_) {
    illc = TRUE_;
  }
  
  if(*prin > 1 && scbd > 1.) {
    vcprnt(&c_2, z_, n);
  }
  
  if(*prin > 1) {
    vcprnt(&c_3, d_, n);
  }
  
  if(*prin > 3) {
    maprnt(&c_2, q.v, &idim, n);
  }

  /* .....THE MAIN LOOP ENDS HERE..... */
  
  goto L40;
  
  /* .....RETURN..... */
  
 L400:
   
  if(*prin > 0) {
    vcprnt(&c_4, &x[1], n);
  }
   
  ret_val = global.fx;
  
  return(ret_val);
} /* praxis */

/* Subroutine */ 

/* ...AN IMPROVED VERSION OF MINFIT (SEE GOLUB AND REINSCH, 1969)         */
/*   RESTRICTED TO M=N,P=0.                                               */
/*   THE SINGULAR VALUES OF THE ARRAY AB ARE RETURNED IN Q AND AB IS      */
/*   OVERWRITTEN WITH THE ORTHOGONAL MATRIX V SUCH THAT U.DIAG(Q) = AB.V, */
/*   WHERE U IS ANOTHER ORTHOGONAL MATRIX.                                */
/* ...HOUSEHOLDER'S REDUCTION TO BIDIAGONAL FORM...                       */

int minfit(int *m, int *n, double *machep, double *tol, double *ab, double *q)
{
  /* System generated locals */
  
  int ab_dim1, ab_offset, i_1, i_2, i_3;
  double d_1, d_2;
  
  /* Builtin functions */
  
  double fabs(double);
  double sqrt(double);
  
  /* Local variables */

  static int i_, j, k, l;  
  static int l2, ii, kk, kt, ll2, lp1;
  static double c_, e[20], f, g, h_;
  static double s, x, y, z_;
  static double eps, temp;
  
  /* Parameter adjustments */

  --q;
  ab_dim1 = *m;
  ab_offset = 1 + ab_dim1 * 1;
  ab -= ab_offset;
  
  /* Function Body */
  
  if(*n == 1) {
    goto L200;
  }
  
  eps = *machep;
  g = 0.0;
  x = 0.0;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    e[i_ - 1] = g;
    s = 0.;
    l = i_ + 1;
    i_2 = *n;
    
    for(j = i_; j <= i_2; ++j) {
      /* Computing 2nd power */
      
      d_1 = ab[j + i_ * ab_dim1];
      s += d_1 * d_1;
    }
    
    g = 0.0;
    
    if(s < *tol) {
      goto L4;
    }
    
    f = ab[i_ + i_ * ab_dim1];
    g = sqrt(s);
    
    if(f >= 0.0) {
      g = -g;
    }
    
    h_ = f * g - s;
    ab[i_ + i_ * ab_dim1] = f - g;
	
    if(l > *n) {
      goto L4;
    }
	
    i_2 = *n;
	
    for(j = l; j <= i_2; ++j) {
      f = 0.0;
      i_3 = *n;
	    
      for (k = i_; k <= i_3; ++k) {
	f += ab[k + i_ * ab_dim1] * ab[k + j * ab_dim1];
      }
      
      f /= h_;
      i_3 = *n;
      
      for(k = i_; k <= i_3; ++k) {
	ab[k + j * ab_dim1] += f * ab[k + i_ * ab_dim1];
      }
    }
  
  L4:
    
    q[i_] = g;
    s = 0.0;
	
    if(i_ == *n) {
      goto L6;
    }
    
    i_3 = *n;
    
    for(j = l; j <= i_3; ++j) {
      s += ab[i_ + j * ab_dim1] * ab[i_ + j * ab_dim1];
    }

  L6:
    
    g = 0.0;
    
    if(s < *tol) {
      goto L10;
    }
    
    if(i_ == *n) {
      goto L16;
    }
    
    f = ab[i_ + (i_ + 1) * ab_dim1];
  
  L16:
    
    g = sqrt(s);
	
    if(f >= 0.0) {
      g = -g;
    }
    
    h_ = f * g - s;
    
    if(i_ == *n) {
      goto L10;
    }
    
    ab[i_ + (i_ + 1) * ab_dim1] = f - g;
    i_3 = *n;
	
    for(j = l; j <= i_3; ++j) {
      e[j - 1] = ab[i_ + j * ab_dim1] / h_;
    }
    
    i_3 = *n;
    
    for(j = l; j <= i_3; ++j) {
      s = 0.0;
      i_2 = *n;
      
      for(k = l; k <= i_2; ++k) {
	s += ab[j + k * ab_dim1] * ab[i_ + k * ab_dim1];
      }
      
      i_2 = *n;
      
      for(k = l; k <= i_2; ++k) {
	ab[j + k * ab_dim1] += s * e[k - 1];
      }
    }
    
  L10:
    
    y = (d_1 = q[i_], fabs(d_1)) + (d_2 = e[i_ - 1], fabs(d_2));

    if(y > x) {
      x = y;
    }
  }

  /* ...ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS... */
  
  ab[*n + *n * ab_dim1] = 1.0;
  g = e[*n - 1];
  l = *n;
  i_1 = *n;
    
  for(ii = 2; ii <= i_1; ++ii) {
    i_ = *n - ii + 1;
    
    if(g == 0.0) {
      goto L23;
    }
	
    h_ = ab[i_ + (i_ + 1) * ab_dim1] * g;
    i_2 = *n;
	
    for(j = l; j <= i_2; ++j) {
      ab[j + i_ * ab_dim1] = ab[i_ + j * ab_dim1] / h_;
    }
    
    i_2 = *n;
	
    for(j = l; j <= i_2; ++j) {
      s = 0.0;
      i_3 = *n;
      
      for(k = l; k <= i_3; ++k) {
	s += ab[i_ + k * ab_dim1] * ab[k + j * ab_dim1];
      }
      
      i_3 = *n;
      
      for(k = l; k <= i_3; ++k) {
	ab[k + j * ab_dim1] += s * ab[k + i_ * ab_dim1];
      }
    }

  L23:
    
    i_3 = *n;
	
    for(j = l; j <= i_3; ++j) {
      ab[i_ + j * ab_dim1] = 0.0;
      ab[j + i_ * ab_dim1] = 0.0;
    }
    
    ab[i_ + i_ * ab_dim1] = 1.0;
    g = e[i_ - 1];
    l = i_;
  }

  /* ...DIAGONALIZATION OF THE BIDIAGONAL FORM... */

  eps *= x;
  i_1 = *n;
  
  for(kk = 1; kk <= i_1; ++kk) {
    k = *n - kk + 1;
    kt = 0;
  
  L101:
    
    ++kt;
    
    if(kt <= 30) {
      goto L102;
    }
    
    e[k - 1] = 0.0;

    printf("QR FAILED \n");
    
  L102:
   
    i_3 = k;
    
    for(ll2 = 1; ll2 <= i_3; ++ll2) {
      l2 = k - ll2 + 1;
      l = l2;
	
      if((d_1 = e[l - 1], fabs(d_1)) <= eps) {
	goto L120;
      }
	
      if(l == 1) {
	goto L103;
      }
      
      if((d_1 = q[l - 1], fabs(d_1)) <= eps) {
	goto L110;
      }

    L103:
      
      ;
    }
    
    /* ...CANCELLATION OF E(L) IF L>1... */

  L110:
    
    c_ = 0.0;
    s = 1.0;
    i_3 = k;
    
    for(i_ = l; i_ <= i_3; ++i_) {
      f = s * e[i_ - 1];
      e[i_ - 1] = c_ * e[i_ - 1];
      
      if(fabs(f) <= eps) {
	goto L120;
      }
      
      g = q[i_];

      /* ...Q(I) = H = DSQRT(G*G + F*F)... */
      
      if(fabs(f) < fabs(g)) {
	goto L113;
      }
      
      if(f != 0.0) {
	goto L112;
      } else {
	goto L111;
      }

    L111:
      
      h_ = 0.0;
	    
      goto L114;

    L112:

      /* Computing 2nd power */
      
      d_1 = g / f;
      h_ = fabs(f) * sqrt(d_1 * d_1 + 1);
      
      goto L114;

    L113:

      /* Computing 2nd power */
      
      d_1 = f / g;
      h_ = fabs(g) * sqrt(d_1 * d_1 + 1);
    
    L114:
      
      q[i_] = h_;

      if(h_ != 0.0) {
	goto L115;
      }
      
      g = 1.0;
      h_ = 1.0;

    L115:

      c_ = g / h_;
      s = -f / h_;
    }

    /* ...TEST FOR CONVERGENCE... */

  L120:
    
    z_ = q[k];
	
    if(l == k) {
      goto L140;
    }

    /* ...SHIFT FROM BOTTOM 2*2 MINOR... */
	
    x = q[l];
    y = q[k - 1];
    g = e[k - 2];
    h_ = e[k - 1];
    f = ((y - z_) * (y + z_) + (g - h_) * (g + h_)) / (h_ * 2 * y);
    g = sqrt(f * f + 1.);
    temp = f - g;
	
    if(f >= 0.) {
      temp = f + g;
    }
	
    f = ((x - z_) * (x + z_) + h_ * (y / temp - h_)) / x;

    /* ...NEXT QR TRANSFORMATION... */
	
    c_ = 1.0;
    s = 1.0;
    lp1 = l + 1;
	
    if(lp1 > k) {
      goto L133;
    }
	
    i_3 = k;
	
    for(i_ = lp1; i_ <= i_3; ++i_) {
      g = e[i_ - 1];
      y = q[i_];
      h_ = s * g;
      g *= c_;
	
      if(fabs(f) < fabs(h_)) {
	goto L123;
      }
      
      if(f != 0.0) {
	goto L122;
      } else {
	goto L121;
      }

    L121:
      
      z_ = 0.0;
      
      goto L124;
    
    L122:
      
      /* Computing 2nd power */

      d_1 = h_ / f;
      z_ = fabs(f) * sqrt(d_1 * d_1 + 1);
      
      goto L124;
    
    L123:

      /* Computing 2nd power */
	
      d_1 = f / h_;
      z_ = fabs(h_) * sqrt(d_1 * d_1 + 1);
    
    L124:
      
      e[i_ - 2] = z_;
      
      if(z_ != 0.0) {
	goto L125;
      }
      
      f = 1.0;
      z_ = 1.0;

    L125:
	
      c_ = f / z_;
      s = h_ / z_;
      f = x * c_ + g * s;
      g = -x * s + g * c_;
      h_ = y * s;
      y *= c_;
      i_2 = *n;
	
      for(j = 1; j <= i_2; ++j) {
	x = ab[j + (i_ - 1) * ab_dim1];
	z_ = ab[j + i_ * ab_dim1];
	ab[j + (i_ - 1) * ab_dim1] = x * c_ + z_ * s;
	ab[j + i_ * ab_dim1] = -x * s + z_ * c_;
      }
	
      if(fabs(f) < fabs(h_)) {
	goto L129;
      }
      
      if(f != 0.0) {
	goto L128;
      } else {
	goto L127;
      }

    L127:
      
      z_ = 0.0;
	    
      goto L130;
    
    L128:
      
      /* Computing 2nd power */
      
      d_1 = h_ / f;
      z_ = fabs(f) * sqrt(d_1 * d_1 + 1);
      
      goto L130;
    
    L129:
      
      /* Computing 2nd power */
      
      d_1 = f / h_;
      z_ = fabs(h_) * sqrt(d_1 * d_1 + 1);
    
    L130:
      
      q[i_ - 1] = z_;

      if(z_ != 0.0) {
	goto L131;
      }
      
      f = 1.0;
      z_ = 1.0;

    L131:
      
      c_ = f / z_;
      s = h_ / z_;
      f = c_ * g + s * y;
      x = -s * g + c_ * y;
    }

  L133:
    
    e[l - 1] = 0.0;
    e[k - 1] = f;
    q[k] = x;
    
    goto L101;

    /* ...CONVERGENCE:  Q(K) IS MADE NON-NEGATIVE... */
  
  L140:
	
    if(z_ >= 0.0) {
      goto L150;
    }
    
    q[k] = -z_;
    i_3 = *n;
    
    for (j = 1; j <= i_3; ++j) {
      ab[j + k * ab_dim1] = -ab[j + k * ab_dim1];
    }
  
  L150:
    ;
  }
  
  return(0);

 L200:
  
  q[1] = ab[ab_dim1 + 1];
  ab[ab_dim1 + 1] = 1.0;
  
  return(0);
} /* minfit */

/* Subroutine */ 

/* ...THE SUBROUTINE MIN MINIMIZES F FROM X IN THE DIRECTION V(*,J) UNLESS */
/*   J IS LESS THAN 1, WHEN A QUADRATIC SEARCH IS MADE IN THE PLANE        */
/*   DEFINED BY Q0,Q1,X.                                                   */
/*   D2 IS EITHER ZERO OR AN APPROXIMATION TO HALF F".                     */
/*   ON ENTRY, X1 IS AN ESTIMATE OF THE DISTANCE FROM X TO THE MINIMUM     */
/*   ALONG V(*,J) (OR, IF J=0, A CURVE).  ON RETURN, X1 IS THE DISTANCE    */
/*   FOUND.                                                                */
/*   IF FK=.TRUE., THEN F1 IS FLIN(X1).  OTHERWISE X1 AND F1 ARE IGNORED   */
/*   ON ENTRY UNLESS FINAL FX IS GREATER THAN F1.                          */
/*   NITS CONTROLS THE NUMBER OF TIMES AN ATTEMPT WILL BE MADE TO HALVE    */
/*   THE INTERVAL.                                                         */

int min(int *n, int *j, int *nits, double *d2, double *x1, double *f1, 
	int *fk, double (*f)(), double *x, double *t, double *machep, 
	double *h_)
{
  /* System generated locals */

  int i_1;
  double d_1, d_2;
  
  /* Builtin functions */
  
  double fabs(double);
  double sqrt(double);
  
  /* Local variables */
  
  static int i_, k;
  static int dz;
  static double s, d1, f0, f2, m2, m4, t2, x2, fm;
  static double xm, sf1, sx1;
  static double temp, small;

  /* Subroutine */

  extern double flin(int *, int *, double *, double (*f)(), double *, int *);

  /* Parameter adjustments */
  
  --x;

  /* Function Body */
  
  /* Computing 2nd power */
  
  d_1 = *machep;
  small = d_1 * d_1;
  m2 = sqrt(*machep);
  m4 = sqrt(m2);
  sf1 = *f1;
  sx1 = *x1;
  k = 0;
  xm = 0.0;
  fm = global.fx;
  f0 = global.fx;
  dz = *d2 < *machep;

  /* ...FIND THE STEP SIZE... */
  
  s = 0.0;
  i_1 = *n;
    
  for(i_ = 1; i_ <= i_1; ++i_) {
    /* Computing 2nd power */
    
    d_1 = x[i_];
    s += d_1 * d_1;
  }
  
  s = sqrt(s);
  temp = *d2;
  
  if(dz) {
    temp = global.dmin_;
  }

  t2 = m4 * sqrt(fabs(global.fx) / temp + s * global.ldt) + m2 * global.ldt;
  s = m4 * s + *t;
  
  if(dz && t2 > s) {
    t2 = s;
  }
  
  t2 = max_(t2, small);
  
  /* Computing MIN */
  
  d_1 = t2, d_2 = *h_ * 0.01;
  t2 = min_(d_1, d_2);
  
  if(!(*fk) || *f1 > fm) {
    goto L2;
  }
  
  xm = *x1;
  fm = *f1;
 
 L2:
  
  if(*fk && fabs(*x1) >= t2) {
    goto L3;
  }
  
  temp = 1.0;
  
  if(*x1 < 0.0) {
    temp = -1.0;
  }
  
  *x1 = temp * t2;
  *f1 = flin(n, j, x1, f, &x[1], &global.nf);

 L3:

  if(*f1 > fm) {
    goto L4;
  }
  
  xm = *x1;
  fm = *f1;

 L4:

  if(! dz) {
    goto L6;
  }

  /* ...EVALUATE FLIN AT ANOTHER POINT AND ESTIMATE THE SECOND DERIVATIVE... */
  
  x2 = -(*x1);
  
  if(f0 >= *f1) {
    x2 = *x1 * 2.;
  }
  
  f2 = flin(n, j, &x2, f, &x[1], &global.nf);
  
  if(f2 > fm) {
    goto L5;
  }
  
  xm = x2;
  fm = f2;
 
 L5:

  *d2 = (x2 * (*f1 - f0) - *x1 * (f2 - f0)) / (*x1 * x2 * (*x1 - x2));
  
  /* ...ESTIMATE THE FIRST DERIVATIVE AT 0... */

 L6:
  
  d1 = (*f1 - f0) / *x1 - *x1 * *d2;
  dz = TRUE_;
  
  /* ...PREDICT THE MINIMUM... */
  
  if(*d2 > small) {
    goto L7;
  }
  
  x2 = *h_;
  
  if(d1 >= 0.0) {
    x2 = -x2;
  }
  
  goto L8;

 L7:
  
  x2 = d1 * - 0.5 / *d2;

 L8:

  if(fabs(x2) <= *h_) {
    goto L11;
  }
  
  if(x2 <= 0.0) {
    goto L9;
  } else {
    goto L10;
  }

 L9:

  x2 = -(*h_);
  goto L11;

 L10:
  
  x2 = *h_;

  /* ...EVALUATE F AT THE PREDICTED MINIMUM... */

 L11:

  f2 = flin(n, j, &x2, f, &x[1], &global.nf);
  
  if(k >= *nits || f2 <= f0) {
    goto L12;
  }

  /* ...NO SUCCESS, SO TRY AGAIN... */
  
  ++k;
  
  if(f0 < *f1 && *x1 * x2 > 0.0) {
    goto L4;
  }
   
  x2 *= 0.5;
  
  goto L11;

  /* ...INCREMENT THE ONE-DIMENSIONAL SEARCH COUNTER... */

 L12:

  ++global.nl;
  
  if(f2 <= fm) {
    goto L13;
  }
  
  x2 = xm;
  
  goto L14;

 L13:

  fm = f2;
  
  /* ...GET A NEW ESTIMATE OF THE SECOND DERIVATIVE... */

 L14:

  if((d_1 = x2 * (x2 - *x1), fabs(d_1)) <= small) {
    goto L15;
  }
  
  *d2 = (x2 * (*f1 - f0) - *x1 * (fm - f0)) / (*x1 * x2 * (*x1 - x2));
  
  goto L16;

 L15:

  if(k > 0) {
    *d2 = 0.0;
  }

 L16:
    
  if(*d2 <= small) {
    *d2 = small;
  }
  
  *x1 = x2;
  global.fx = fm;
  
  if(sf1 >= global.fx) {
    goto L17;
  }
  
  global.fx = sf1;
  *x1 = sx1;
  
  /* ...UPDATE X FOR LINEAR BUT NOT PARABOLIC SEARCH... */

 L17:
  
  if(*j == 0) {
    return(0);
  }
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    x[i_] += *x1 * q.v[i_ + *j * 20 - 21];
  }
  
  return(0);
} /* min */

/* ...FLIN IS THE FUNCTION OF ONE REAL VARIABLE L THAT IS MINIMIZED */
/*   BY THE SUBROUTINE MIN...                                       */

double flin(int *n, int *j, double *l, double (*f)(), double *x, int *nf)
{
  /* System generated locals */
  
  int i_1;
  double ret_val;
  
  /* Local variables */
  
  static int i_;
  static double t[20];
    
  /* Parameter adjustments */
  
  --x;
  
  /* Function Body */
  
  if(*j == 0) {
    goto L2;
  }

  /* ...THE SEARCH IS LINEAR... */
  
  i_1 = *n;
  
  for (i_ = 1; i_ <= i_1; ++i_) {
    t[i_ - 1] = x[i_] + *l * q.v[i_ + *j * 20 - 21];
  }
  
  goto L4;

  /* ...THE SEARCH IS ALONG A PARABOLIC SPACE CURVE... */

 L2:
  
  q.qa = *l * (*l - q.qd1) / (q.qd0 * (q.qd0 + q.qd1));
  q.qb = (*l + q.qd0) * (q.qd1 - *l) / (q.qd0 * q.qd1);
  q.qc = *l * (*l + q.qd0) / (q.qd1 * (q.qd0 + q.qd1));
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    t[i_ - 1] = q.qa * q.q0[i_ - 1] + q.qb * x[i_] + q.qc * q.q1[i_ - 1];
  }
  
  /* ...THE FUNCTION EVALUATION COUNTER NF IS INCREMENTED... */

 L4:

  ++(*nf);
  ret_val = (*f)(t, n);
  
  return(ret_val);
} /* flin */

/* Subroutine */ 

/* ...SORTS THE ELEMENTS OF D(N) INTO DESCENDING ORDER AND MOVES THE */
/*   CORRESPONDING COLUMNS OF V(N,N).                                */
/*   M IS THE ROW DIMENSION OF V AS DECLARED IN THE CALLING PROGRAM. */

int sort(int *m, int *n, double *d_, double *v)
{
  /* System generated locals */

  int v_dim1, v_offset, i_1, i_2;
  
  /* Local variables */
  
  static int i_, j, k;
  static double s;
  static int ip1, nm1;
  
  /* Parameter adjustments */
  
  v_dim1 = *m;
  v_offset = 1 + v_dim1 * 1;
  v -= v_offset;
  --d_;
  
  /* Function Body */
  
  if(*n == 1) {
    return(0);
  }
  
  nm1 = *n - 1;
  i_1 = nm1;
    
  for(i_ = 1; i_ <= i_1; ++i_) {
    k = i_;
    s = d_[i_];
    ip1 = i_ + 1;
    i_2 = *n;
	
    for(j = ip1; j <= i_2; ++j) {
      if(d_[j] <= s) {
	goto L1;
      }
      
      k = j;
      s = d_[j];

    L1:
      
      ;
    }
	
    if(k <= i_) {
      goto L3;
    }
    
    d_[k] = d_[i_];
    d_[i_] = s;
    i_2 = *n;
    
    for(j = 1; j <= i_2; ++j) {
      s = v[j + i_ * v_dim1];
      v[j + i_ * v_dim1] = v[j + k * v_dim1];
      v[j + k * v_dim1] = s;
    }

  L3:
    
    ;
  }
  
  return(0);
} /* sort */

/* Subroutine */ 

/* ...QUAD LOOKS FOR THE MINIMUM OF F ALONG A CURVE DEFINED BY Q0,Q1,X... */

int quad(int *n, double (*f)(), double *x, double *t, double *machep, 
	 double *h_)
{
  /* System generated locals */
  
  int i_1;
  double d_1;
  
  /* Builtin functions */
  
  double sqrt(double);
  
  /* Local variables */
  
  static int i_;
  static double l, s;
  static double value;

  /* Subroutine */ 
  
  extern int min(int *, int *, int *, double *, double *, double *, int *, 
		 double (*f)(), double *, double *, double *, double *);

  /* Parameter adjustments */
  
  --x;
  
  /* Function Body */
  
  s = global.fx;
  global.fx = q.qf1;
  q.qf1 = s;
  q.qd1 = 0.0;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    s = x[i_];
    l = q.q1[i_ - 1];
    x[i_] = l;
    q.q1[i_ - 1] = s;

    /* Computing 2nd power */
	
    d_1 = s - l;
    q.qd1 += d_1 * d_1;
  }
  
  q.qd1 = sqrt(q.qd1);
  l = q.qd1;
  s = 0.0;
  
  if(q.qd0 <= 0.0 || q.qd1 <= 0.0 || global.nl < *n * 3 * *n) {
    goto L2;
  }
  
  value = q.qf1;
  
  min(n, &c_0, &c_2, &s, &l, &value, &c_true, f, &x[1], t, machep, h_);
  
  q.qa = l * (l - q.qd1) / (q.qd0 * (q.qd0 + q.qd1));
  q.qb = (l + q.qd0) * (q.qd1 - l) / (q.qd0 * q.qd1);
  q.qc = l * (l + q.qd0) / (q.qd1 * (q.qd0 + q.qd1));
  goto L3;

 L2:

  global.fx = q.qf1;
  q.qa = 0.0;
  q.qb = q.qa;
  q.qc = 1.0;
 
 L3:

  q.qd0 = q.qd1;
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    s = q.q0[i_ - 1];
    q.q0[i_ - 1] = x[i_];
    x[i_] = q.qa * s + q.qb * x[i_] + q.qc * q.q1[i_ - 1];
  }
  
  return(0);
} /* quad */

/* Subroutine */ 

int vcprnt(int *option, double *v, int *n)
{
  /* System generated locals */
  
  int i_1;
  
  /* Local variables */
  
  static int i_;
  
  /* Parameter adjustments */
  
  --v;
  
  /* Function Body */
  
  switch(*option) {
  case 1:  goto L1;
  case 2:  goto L2;
  case 3:  goto L3;
  case 4:  goto L4;
  }
  
 L1:
  
  printf("THE SECOND DIFFERENCE ARRAY D(*) IS: \n");

  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    printf("%lf \n", v[i_]);
  }
  
  return(0);

 L2:
    
  printf("THE SCALE FACTORS ARE:\n");
  
  i_1 = *n;
    
  for(i_ = 1; i_ <= i_1; ++i_) {
    printf("%lf \n", v[i_]);
  }
  
  return(0);

 L3:

  printf("THE APPROXIMATING QUADRATIC FORM HAS THE PRINCIPAL VALUES: \n");

  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    printf("%lf \n", v[i_]);
  }
  
  return(0);
 
 L4:

  printf("X IS: \n");  

  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    printf("%lf \n", v[i_]);
  }
  
  return(0);
} /* vcprnt */

/* Subroutine */ 

int print(int *n, double *x, int *prin, double *fmin)
{
  /* System generated locals */
  
  int i_1;
  double d_1;
  
  /* Builtin functions */
  
  double d_lg10(double *);
  
  /* Local variables */
  
  static int i_;
  static double ln;

  /* Parameter adjustments */

  --x;
  
  /* Function Body */
  
  printf("AFTER %d LINEAR SEARCHES, THE FUNCTION HAS BEEN EVALUATED"
	 " %d TIMES. THE SMALLEST VALUE FOUND IS F(X) = %lf \n", 
	 global.nl, global.nf, global.fx);
  
  if(global.fx <= *fmin) {
    goto L1;
  }
  
  d_1 = global.fx - *fmin;
  ln = d_lg10(&d_1);

  printf("LOG(F(X)- %lf) = %lf \n", *fmin, ln);
  
  goto L2;

 L1:

  printf("LOG(F(X)- %lf) IS UNDEFINED", *fmin);  

 L2:
  
  if(*n > 4 && *prin <= 2) {
    return(0);
  }
  
  printf("X IS: \n");  

  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    printf("%lf \n", x[i_]);
  }
  
  return(0);
} /* print */

/* Subroutine */ 

/* ...THE SUBROUTINE MAPRNT PRINTS THE COLUMNS OF THE NXN MATRIX V     */
/*   WITH A HEADING AS SPECIFIED BY OPTION.                            */
/*   M IS THE ROW DIMENSION OF V AS DECLARED IN THE CALLING PROGRAM... */

int maprnt(int *option, double *v, int *m, int *n)
{
  /* System generated locals */
  
  int v_dim1, v_offset, i_1, i_2;
  
  /* Local variables */
  
  static int i_, j, low, upp;
  
  /* Parameter adjustments */

  v_dim1 = *m;
  v_offset = 1 + v_dim1 * 1;
  v -= v_offset;
  
  /* Function Body */
  
  low = 1;
  upp = 5;
  
  switch(*option) {
  case 1:  goto L1;
  case 2:  goto L2;
  }
 
 L1:
  
  printf("THE NEW DIRECTIONS ARE: \n");
  
  goto L3;

 L2:

  printf("AND THE PRINCIPAL AXES: \n");  

 L3:
  
  if(*n < upp) {
    upp = *n;
  }
  
  i_1 = *n;
  
  for(i_ = 1; i_ <= i_1; ++i_) {
    i_2 = upp;
	
    for(j = low; j <= i_2; ++j) {
      printf("%lf \n", v[i_ + j * v_dim1]);
    }
  }
   
  low += 5;
  
  if(*n < low) {
    return(0);
  }
  
  upp += 5;

  printf("\n");

  goto L3;
} /* maprnt */

double random(int *naught)
{
  /* Initialized data */
  
  static int init = FALSE_;

  /* System generated locals */
  
  double ret_val;
  
  /* Local variables */
  
  static int i_, j, q, r_;
  static double ran1;
  static int ran2;
  static double ran3[127], half;
  
  if(init) {
    goto L3;
  }
    
  r_ = *naught % 8190 + 1;
  ran2 = 128;
  
  for(i_ = 1; i_ <= 127; ++i_) {
    --ran2;
    ran1 = -36028797018963968.0;
	
    for(j = 1; j <= 7; ++j) {
      r_ = r_ * 1756 % 8191;
      q = r_ / 32;
      ran1 = (ran1 + q) * 0.00390625;
    }

    ran3[ran2 - 1] = ran1;
  }
  
  init = TRUE_;

 L3:
  
  if(ran2 == 1) {
    ran2 = 128;
  }
  
  --ran2;
  ran1 += ran3[ran2 - 1];
  half = 0.5;
  
  if(ran1 >= 0.0) {
    half = -half;
  }
  
  ran1 += half;
  ran3[ran2 - 1] = ran1;
  ret_val = ran1 + 0.5;
  
  return(ret_val);
} /* random */

double d_log(double *x)
{
  extern double log();

  return(log(*x));
}

#define log10e 0.43429448190325182765

double d_lg10(double *x)
{
  extern double log();

  return(log10e * log(*x));
}

int pow_ii(int *ap, int *bp)
{
  int pow, x, n;
  unsigned long u;
  
  x = *ap;
  n = *bp;
  
  if(n <= 0) {
    if (n == 0 || x == 1)
      return(1);

    if(x != -1)
      return(x == 0 ? 1/x : 0);

    n = -n;
  }
  
  u = n;
  
  for(pow = 1; ; ) {
    if(u & 01)
      pow *= x;
    
    if(u >>= 1)
      x *= x;
    else
      break;
  }

  return(pow);
}
