/* Simplex.f -- translated by f2c (version 20020621). */

#include <stdio.h>
#include <stdlib.h>

/* Table of constant values */

#define N1 20
#define N2 21
#define N3 22
#define N4 (N1 * N2)

#define abs(x)     ((x) >= 0 ? (x) : - (x))
#define dabs(x)    ((double) abs(x))
#define min(a, b)  ((a) <= (b) ? (a) : (b))
#define max(a, b)  ((a) >= (b) ? (a) : (b))
#define dmin(a, b) ((double) min(a, b))
#define dmax(a, b) ((double) max(a, b))

/* Subroutine                                                              */ 
/*                                                                         */
/* DOWNHILL SIMPLEX METHOD                                                 */
/* BASED ON NUMERICAL RECIPES P. 292                                       */
/* RAINER KAYSER 1989-07-17                                                */
/*                                                                         */
/* INPUT                                                                   */
/*                                                                         */
/* REAL FUNCTION      FCT(X)     X=N-DIM VECTOR                            */
/* REAL               X0(NDIM)   START POINT                               */
/* REAL               XD0(NDIM)  START STEPS                               */
/*                                                                         */
/* X0 AND XD0 BUILD UP INITIAL SIMPLEX                                     */
/*                                                                         */
/* INTEGER            NDIM                                                 */
/* REAL               EXACT      MAXIMAL RELATIV ERROR OF FUNCTION MINIMUM */
/* INTEGER            MAXST      MAX NUMBER OF STEPS                       */
/*                                                                         */
/* OUTPUT                                                                  */
/*                                                                         */
/* REAL               X(NDIM)    MINIMUM                                   */
/* REAL               FCTVAL     VALUE OF FCT AT X                         */
/*                                                                         */

int simplex(double (*fct)(), float *x0, float *xd0, int *ndim, 
	   float *exact, int *maxst, float *x, float *fctval)
{
  /* System generated locals */
  
  int i_1, i_2;
  float r_0;
  
  /* Builtin functions   */  
  /* Local variables     */
  /* p[420] was [21][20] */
  
  static int i, j;
  static float p[N4], y[N1], pr[N1];
  static int ihi;
  static float val;
  static int ilo;
  static float prr[N1], ypr, pbar[N1];
  static int inhi, iter;
  static float yprr;
  static int iflag;
  
  /* Fortran I/O blocks    */  
  /* INITIALIZE SIMPLEX    */
  /* Parameter adjustments */

  --x;
  --xd0;
  --x0;
  
  /* Function Body */
  
  i_1 = *ndim;
   
  for(i = 1; i <= i_1; ++i) {
    i_2 = *ndim;
	
    for(j = 1; j <= i_2; ++j) {
      p[i + j * N2 - N3] = x0[j];
      x[j] = x0[j];
      
      if(i == j) {
	p[i + j * N2 - N3] += xd0[j];
	x[j] += xd0[j];
      }
      
      /* L101: */
    }
    
    y[i - 1] = (*fct)(&x[1]);
   
    /* L100: */
  }
  
  i_1 = *ndim;
  
  for(i = 1; i <= i_1; ++i) {
    p[*ndim + 1 + i * N2 - N3] = x0[i];
    x[i] = x0[i];
    
    /* L102: */
  }
  
  y[*ndim] = (*fct)(&x[1]);
  
  /* INIT ITERATION */
  
  iter = 0;
  iflag = 0;
 
 L200:
  
  ilo = 1;

  /* SEARCH HIGHEST AND LOWEST POINTS */
  
  if(y[0] > y[1]) {
    ihi = 1;
    inhi = 2;
  } else {
    ihi = 2;
    inhi = 1;
  }
  
  i_1 = *ndim + 1;
  
  for(i = 1; i <= i_1; ++i) {
    if(y[i - 1] < y[ilo - 1]) {
      ilo = i;
    }
    
    if(y[i - 1] > y[ihi - 1]) {
      inhi = ihi;
      ihi = i;
    } else if(y[i - 1] > y[inhi - 1]) {
      if(i != ihi) {
	inhi = i;
      }
    }
    
    /* L211: */
  }

  /* CHECK DIFFERENCE AND RETURN IF DONE */
  
  i_1 = *ndim;
  
  for(i = 1; i <= i_1; ++i) {
    x[i] = p[ilo + i * N2 - N3];

    /* L300: */
  }
  
  val = (*fct)(&x[1]);
  
  if(iter != 0) {
    /* PRINT EVERY TENTH STEP */
    
    /*

    if(iter / 10 * 10 == iter) {
      printf("ITER   = %d \n", iter);
      printf("VAL    = %f \n", val);
      printf("FCTVAL = %f \n", *fctval);
    }

    */

    /* STOP IF TOO MANY STEPS */
    
    if(iter >= *maxst) {
      iflag = 1;
    }

    if(val < 1e-19f) {
      *fctval = val;
      
      return(0);
    }
    
    if((r_0 = val - y[ihi - 1], dabs(r_0)) / val < *exact || iflag == 1) {
      *fctval = val;
      
      return(0);
    }
  }

  /* SAVE LOWEST VALUE */
  
  *fctval = val;
  
  /* ITERATION */

  ++iter;
  i_1 = *ndim;
  
  for(j = 1; j <= i_1; ++j) {
    pbar[j - 1] = 0.0f;
   
    /* L212: */
  }
  
  i_1 = *ndim + 1;
  
  for(i = 1; i <= i_1; ++i) {
    if(i != ihi) {
      i_2 = *ndim;
      
      for(j = 1; j <= i_2; ++j) {
	pbar[j - 1] += p[i + j * N2 - N3];
	
	/* L213: */
      }
    }
    
    /* L214: */
  }
  
  i_1 = *ndim;
  
  for(j = 1; j <= i_1; ++j) {
    pbar[j - 1] /= *ndim;
    pr[j - 1] = pbar[j - 1] * 2.0f - p[ihi + j * N2 - N3] * 1.0f;
    
    /* L215: */
  }
  
  ypr = (*fct)(pr);
  
  if(ypr <= y[ilo - 1]) {
    i_1 = *ndim;
    
    for(j = 1; j <= i_1; ++j) {
      prr[j - 1] = pr[j - 1] * 2.0f + pbar[j - 1] * (- 1.0f);
     
      /* L216: */
    }
    
    yprr = (*fct)(prr);
    
    if(yprr < y[ilo - 1]) {
      i_1 = *ndim;
      
      for(j = 1; j <= i_1; ++j) {
	p[ihi + j * N2 - N3] = prr[j - 1];

	/* L217: */
      }
      
      y[ihi - 1] = yprr;
    } else {
      i_1 = *ndim;
      
      for(j = 1; j <= i_1; ++j) {
	p[ihi + j * N2 - N3] = pr[j - 1];
	
	/* L218: */
      }
  
      y[ihi - 1] = ypr;
    }
  } else if(ypr >= y[inhi - 1]) {
    if(ypr < y[ihi - 1]) {
      i_1 = *ndim;
      
      for(j = 1; j <= i_1; ++j) {
	p[ihi + j * N2 - N3] = pr[j - 1];
	
	/* L219: */
      }
      
      y[ihi - 1] = ypr;
    }
    
    i_1 = *ndim;
    
    for(j = 1; j <= i_1; ++j) {
      prr[j - 1] = p[ihi + j * N2 - N3] * 0.5f + pbar[j - 1] * 0.5f;
     
      /* L221: */
    }
    
    yprr = (*fct)(prr);
    
    if(yprr < y[ihi - 1]) {
      i_1 = *ndim;
      
      for(j = 1; j <= i_1; ++j) {
	p[ihi + j * N2 - N3] = prr[j - 1];
	
	/* L222: */
      }

      y[ihi - 1] = yprr;
    } else {
      i_1 = *ndim + 1;
      
      for(i = 1; i <= i_1; ++i) {
	if(i != ilo) {
	  i_2 = *ndim;
	  
	  for(j = 1; j <= i_2; ++j) {
	    pr[j - 1] = (p[i + j * N2 - N3] + p[ilo + j * N2 - N3]) * 0.5f;
	   
	    /* L223: */
	  }
	  
	  y[i - 1] = (*fct)(pr);
	}
	
	/* L224: */
      }
    }
  } else {
    i_1 = *ndim;
    
    for(j = 1; j <= i_1; ++j) {
      p[ihi + j * N2 - N3] = pr[j - 1];
      
      /* L225: */
    }
   
    y[ihi - 1] = ypr;
  }
  
  goto L200;
} /* simplex */

