#include <stdio.h>
#include <math.h>

double jacobian(double x, double y);
void critical(double dxy_critic, double x_tol);
void caustic();
void critic_caustic();
void caustic_new(int c_critic);

double jacobian(double x, double y) 
{
  double fnct1, fnct2, Funct2, funck, funchb, funchg;
  double func1, func2, func3, func4, funcnis, funcrl, funcs, funcg, functs;
  double poly1, poly2, poly3, poly4, poly5, poly6, poly7;
  double dipolm1, dipolm2, quadrupolm1, quadrupolm2; 
  double rbeta, s2, rs12;  
  double B1, B2, D2, yp1, yp2, rd2, xd1, xd2, Mkaprp, Mkapto;  
  double hubble_factor =  2.0;    
  double ring_factor   = 10.0;
  double Epsilon1 = 1.0 - parms[P_EPSILON];
  double Epsilon2 = 1.0 + parms[P_EPSILON];
  double delta    = 1.0 - parms[P_ALPHA];
  double delta2   = 2.0 - parms[P_ALPHA]; 
  double det_J, j11 = A1, j12 = D, j21 = D, j22 = A2;
  double xbar0 = 0.0, xbar1 = 0.0;  
  double sxbr0 = 0.0, sxbr1 = 0.0, sxbr2 = 0.0; 
  double sxbre2 = 0.0, sxbre4 = 0.0;
  double radius_E, radius_E2, radius_E3, radius_E4;
  double alpha, dalph, cor_factor = parms[P_E] * parms[P_E];
  double q1 = parms[P_Q3] * cos((M_PI * parms[P_THETA_INC]) / 180.0); 
  double q2 = sin((M_PI * parms[P_THETA_INC]) / 180.0); 
  double ff, g1, g2, psi, q;
  double jbh11, jbh12, jbh21, jbh22;
  
  /*

  xl = source_position.x;
  yl = source_position.y;

  **/

  xl = 0.0;
  yl = 0.0;

  xbar0 =   (x - xl) * cs + (y - yl) * ss;
  xbar1 = - (x - xl) * ss + (y - yl) * cs;  

  radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / R_E2;
  radius_E  = sqrt(radius_E2);
  radius_E3 = radius_E  * radius_E2;
  radius_E4 = radius_E2 * radius_E2;

  sxbr0  = xbar0 - dist_1;
  sxbr1  = xbar1 - dist_2;
  sxbr2  = sxbr0 * sxbr0 + sxbr1 * sxbr1;
  sxbre2 = sxbr2 / R_E2;
  sxbre4 = sxbre2 * sxbre2;

  switch(current_model) {
  case M_MODEL_MENU:
    break;
  case M_CHANG_REFSDAL:
    j11 -= (xbar1 * xbar1 - xbar0 * xbar0) / (cor_factor * radius_E4);
    j12 += (2.0 * xbar0 * xbar1) / (cor_factor * radius_E4);
    j21  = j12;
    j22 -= (xbar0 * xbar0 - xbar1 * xbar1) / (cor_factor * radius_E4);
  
    break;
  case M_SECOND:
    
    break;      
  case M_SING_ISOT_SPHERE:
    rbeta = pow(radius_E, (parms[P_EXP] - 4.0));
    
    j11 -= rbeta * (xbar1 * xbar1 + (parms[P_EXP] - 1.0) * xbar0 * xbar0);
    j12 -= rbeta * (parms[P_EXP] - 2.0) * xbar0 * xbar1;
    j21  = j12;
    j22 -= rbeta * (xbar0 * xbar0 + (parms[P_EXP] - 1.0) * xbar1 * xbar1);

    break;      
  case M_DE_VAUCOULEUR:
    poly1 = 720.0 / tau8 + (720.0 * pow(radius_E, (1.0 / 4.0))) / tau7;
    poly2 = poly1 + pow(radius_E, (3.0 / 2.0)) / tau2;
    poly3 = poly2 + (6.0 * pow(radius_E, (5.0 / 4.0))) / tau3;
    poly4 = poly3 + (30.0 * radius_E) / tau4;
    poly5 = poly4 + (120.0 * pow(radius_E, (3.0 / 4.0))) / tau5;
    poly6 = poly5 + (360.0 * sqrt(radius_E)) / tau6;
    poly7 = 7.0 * poly6 + pow(radius_E, (7.0 / 4.0)) / tau;

    func1 = poly7 * exp(- tau * (pow(radius_E, (1.0 / 4.0)) - 1.0));
    func2 = func1 - (5040.0 * Ftau) / tau8;

    alpha = - (8.0 * parms[P_KAPPA] * func2) / radius_E;
    
    poly1 = pow(radius_E, (3.0 / 4.0)) / tau;
    poly2 = poly1 + 720.0 * pow(radius_E, - (3.0 / 4.0)) / tau7;
    poly3 = poly2 + 6.0 * sqrt(radius_E) / tau2;
    poly4 = poly3 + 30.0 * pow(radius_E, (1.0 / 4.0)) / tau3;
    poly5 = poly4 + 360.0 * pow(radius_E, - (1.0 / 4.0)) / tau5; 
    poly6 = poly5 + 720.0 / (tau6 * sqrt(radius_E));  
    poly7 = poly6 + 120.0 / tau4;  

    func3 = 7.0 * poly7 * exp(- tau * (pow(radius_E, (1.0 / 4.0)) - 1.0));
    func4 = (func3 - tau * func1 * pow(radius_E, - (3.0 / 4.0))) / 4.0;

    dalph = 8.0 * parms[P_KAPPA] * (func2 / radius_E2 - func4 / radius_E);

    j11 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar0 * xbar0) / radius_E3;
    j12 -= ((radius_E * dalph - alpha) * xbar0 * xbar1) / radius_E3;
    j21  = j12;
    j22 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar1 * xbar1) / radius_E3;

    break;      
  case M_KING: 
    /* log1p(x) = log(1.0 + x) */

    funck = (radius_E2 / (1.0 + radius_E2) - log1p(radius_E2)) / radius_E4;
    
    j11 -= (parms[P_KAPPA] * log1p(radius_E2)) / radius_E2 
      + 2.0 * parms[P_KAPPA] * xbar0 * xbar0 * funck;
    j12 -= 2.0 * parms[P_KAPPA] * xbar0 * xbar1 * funck;
    j21 = j12;
    j22 -= (parms[P_KAPPA] * log1p(radius_E2)) / radius_E2 
      + 2.0 * parms[P_KAPPA] * xbar1 * xbar1 * funck;
    
    break;      
  case M_HUBBLE:
    funchb = log1p(radius_E) - radius_E / (1.0 + radius_E);
    funchg = 2.0 * funchb - radius_E2 / ((1.0 + radius_E) * (1.0 + radius_E));

    j11 -= hubble_factor * parms[P_KAPPA] 
      * (funchb - (funchg * xbar0 * xbar0) / radius_E2) / radius_E2;
    j12 += hubble_factor * parms[P_KAPPA] 
      * ((funchg * xbar0 * xbar1) / radius_E4);
    j21 = j12;
    j22 -= hubble_factor * parms[P_KAPPA] 
      * (funchb - (funchg * xbar1 * xbar1) / radius_E2) / radius_E2;

    break;      
  case M_SPIRAL:
    funcs = 2.0 * parms[P_KAPPA] * (1.0 - exp(- parms[P_ALPHA2] * radius_E) 
				    * (parms[P_ALPHA2] * radius_E + 1.0));
    funcg = parms[P_KAPPA] * exp(- parms[P_ALPHA2] * radius_E) 
      - funcs / (parms[P_ALPHA2] * parms[P_ALPHA2] * radius_E2);

    j11 -= (funcs / (parms[P_ALPHA2] * parms[P_ALPHA2]) 
	    + 2.0 * funcg * xbar0 * xbar0) / radius_E2;
    j12 -= 2.0 * funcg * xbar0 * xbar1 / radius_E2;
    j21 = j12;
    j22 -= (funcs / (parms[P_ALPHA2] * parms[P_ALPHA2]) 
	    + 2.0 * funcg * xbar1 * xbar1) / radius_E2;

    break;      
  case M_TRUNC_KING:
    /* hypot(x, y) = sqrt(x * x + y * y) */

    func1 = log1p(radius_E2) + radius_E2 / (1.0 + xc2); 
    func2 = (4.0 * (hypot(1.0, radius_E) - 1.0)) / hypot(1.0, xc);
    func3 = func1 - func2;
    func4 = 1.0 / hypot(1.0, radius_E) + 1.0 / hypot(1.0, xc) 
      - 2.0 / sqrt((1.0 + xc2) * (1.0 + radius_E2));
    
    alpha = (parms[P_KAPPA2] * func3) / radius_E2; 
    dalph = (parms[P_KAPPA2] * (2.0 * func4 - func3 / radius_E2));

    j11 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar0 * xbar0) / radius_E3;
    j12 -= ((radius_E * dalph - alpha) * xbar0 * xbar1) / radius_E3;
    j21  = j12;
    j22 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar1 * xbar1) / radius_E3;

    break;      
  case M_NSING_ISOT_SPHERE:
     s2 = xc2 + radius_E2;

     funcnis = sqrt(s2) - xc;     

     /*

     j11 -= parms[P_KAPPA] 
       * ((funcnis * (xbar1 * xbar1 - xbar0 * xbar0)) / radius_E4 
	  + xbar0 * xbar0 / sqrt(s2));
     j12 -= parms[P_KAPPA] 
       * (1.0 / sqrt(s2) - 2.0 * funcnis / radius_E4) * (xbar0 * xbar1); 
     j21  = j12;
     j22 -= parms[P_KAPPA] 
       * ((funcnis * (xbar0 * xbar0 - xbar1 * xbar1)) / radius_E4 
	  + xbar1 * xbar1 / sqrt(s2));

     **/
    
     j11 -= (parms[P_KAPPA] * (funcnis * (radius_E2 - 2.0 * xbar0 * xbar0)
			       + (radius_E2 * xbar0 * xbar0) / sqrt(s2)) 
	     + bh_beta * (xbar1 * xbar1 
			  - xbar0 * xbar0) / cor_factor) / radius_E4; 
     j12 += (2.0 * (parms[P_KAPPA] * (funcnis - radius_E2 / (2.0 * sqrt(s2))) 
		    + bh_beta / cor_factor) * xbar0 * xbar1) / radius_E4; 
     j21 = j12;
     j22 -= (parms[P_KAPPA] * (funcnis * (radius_E2 - 2.0 * xbar1 * xbar1)
			       + (radius_E2 * xbar1 * xbar1) / sqrt(s2)) 
	     + bh_beta * (xbar0 * xbar0 
			  - xbar1 * xbar1) / cor_factor) / radius_E4; 
     
     break;      
  case M_TRANSP_SPHERE:
    
    if(parms[P_C] > rd) {
      s2 = 1.0 - (xbar0 * xbar0 + xbar1 * xbar1)/ R_C2;
      
      funck = 1.0 - pow(s2, (3.0 / 2.0));
      
      alpha = funck / radius_E;

      dalph = - (2.0 * funck) / radius_E3 
	+ (3.0 * sqrt(s2)) / ( radius_E * xc2);

      j11 -= (alpha + dalph * xbar0 * xbar0) / (cor_factor * radius_E);
      j12 -= (dalph * xbar0 * xbar1) / (cor_factor * radius_E);
      j21  = j12;
      j22 -= (alpha + dalph * xbar1 * xbar1) / (cor_factor * radius_E);
    } else {
      j11 -= (xbar1 * xbar1 - xbar0 * xbar0) / (cor_factor * radius_E4);
      j12 += (2.0 * xbar0 * xbar1) / (cor_factor * radius_E4);
      j21  = j12;
      j22 -= (xbar0 * xbar0 - xbar1 * xbar1) / (cor_factor * radius_E4);
    }
    
    break;
  case M_ELLIPTICAL:
    s2 = xc2 + (Epsilon1 * xbar0 * xbar0 + Epsilon2 * xbar1 * xbar1) / R_E2;

    /*

    j11 -= parms[P_KAPPA] * (Epsilon1 * s2 - 2.0 * delta * Epsilon1 * Epsilon1 
			     * xbar0 * xbar0) / pow(s2, delta2);
    j12 += 2.0 * delta * parms[P_KAPPA] * Epsilon1 * Epsilon2 
      * xbar0 * xbar1 / pow(s2, delta2);
    j21 = j12; 
    j22 -= parms[P_KAPPA] * (Epsilon2 * s2 - 2.0 * delta * Epsilon2 * Epsilon2 
			     * xbar1 * xbar1) / pow(s2, delta2);

    **/

    j11 -=  parms[P_KAPPA] * (s2 - 2.0 * xbar0 * xbar0) / (s2 * s2)
      + (bh_beta * (sxbr1 * sxbr1 - sxbr0 * sxbr0)) / (cor_factor * sxbre4);
    j12 += 2.0 * ((parms[P_KAPPA] * xbar0 * xbar1) / (s2 * s2) 
		  + (bh_beta * sxbr0 * sxbr1) / (cor_factor * sxbre4)); 
    j21 = j12;
    j22 -= parms[P_KAPPA] * (s2 - 2.0 * xbar1 * xbar1) / (s2 * s2)
      + (bh_beta * (sxbr0 * sxbr0 - sxbr1 * sxbr1)) / (cor_factor * sxbre4);
    
    break;      
  case M_MULTIPOLE:

    break;
  case M_ROTATION_LENS:

    break;
  case M_DOUBLE_LENS:
    
    break;
  case M_UNIFORM_RING:

    break;
  case M_ELL_KK:
    q   = sqrt(q1 * q1 + q2 * q2);
    psi = sqrt(q * q * (xc * xc + (xbar0 * xbar0) / R_E2) 
	       + (xbar1 * xbar1) / R_E2); 
    ff  = (1.0 + q * q) * xc2 + 2.0 * xc * psi + radius_E2;
    g1  = q * q * xc2 + xc * psi + (xbar1 * xbar1) / R_E2; 
    g2  = xc2 + xc * psi + (xbar0 * xbar0) / R_E2;

    j11 -= (parms[P_KAPPA] * g1) / (ff * psi) 
      + (bh_beta * (xbar1 * xbar1 
		    - xbar0 * xbar0)) / (cor_factor * radius_E4);     
    j12 += parms[P_KAPPA] / (ff * psi) 
      + 2.0 * bh_beta * xbar0 * xbar1 / (cor_factor * radius_E4); 
    j21 = j12;
    j22 -= (parms[P_KAPPA] * g2) / (ff * psi) 
      + (bh_beta * (xbar0 * xbar0 
		    - xbar1 * xbar1)) / (cor_factor * radius_E4);
    
    break;
  default:
    break;
  }  

  if(black_hole == 1) {
    jbh11 -= (xbar1 * xbar1 - xbar0 * xbar0) / (cor_factor * radius_E4);
    jbh12 += (2.0 * xbar0 * xbar1) / (cor_factor * radius_E4);
    jbh21  = j12;
    jbh22 -= (xbar0 * xbar0 - xbar1 * xbar1) / (cor_factor * radius_E4);

    det_J = j11 * j22 - j12 * j21 + jbh11 * jbh22 - jbh12 * jbh21;
  } else {
    det_J = j11 * j22 - j12 * j21;
  }

  return(det_J);
}

int number_critical = 0;
double x1_caustic[Numx], x2_caustic[Numx];

void critical(double dxy_critic, double x_tol)
{
  int i = 0;
  
  // double x_critic, y_critic, dx_critic = 0.00025, dy_critic = 0.00025;
  // double x_critic, y_critic, dx_critic = 0.00075, dy_critic = 0.00075;
  // double x_critic, y_critic, dx_critic = 0.0005, dy_critic = 0.0005;
  // double x_critic, y_critic, dx_critic = 0.001, dy_critic = 0.001;
  // double x_critic, y_critic, dx_critic = 0.01, dy_critic = 0.01;

  double x_critic, y_critic;
  
  double dx_critic = dxy_critic, dy_critic = dxy_critic;

  double x_lim = BOUND;
  
  // double x_tol = TOL4;

  // double x_tol = TOL5;

  // double x_tol = TOL6;

  for(x_critic = - x_lim; x_critic <= x_lim; x_critic += dx_critic) { 
    for(y_critic = - x_lim; y_critic <= x_lim; y_critic += dy_critic) {
      if(fabs(jacobian(x_critic, y_critic)) <= x_tol) {
	// printf("|Jac| = %g \n", fabs(jacobian(x_critic, y_critic)));

	x1_caustic[i] = x_critic;
	x2_caustic[i] = y_critic;
	
	i = i + 1;

	// printf("x_critic = %g \n", x_critic);
	// printf("y_critic = %g \n", y_critic);
      }
    }
  }
  
  number_critical = i;
}

double y1_caustic[Numx], y2_caustic[Numx];

void caustic()
{
  int i;
  double d_0 = (M_PI * parms[P_PHI]) / 180.0;
  double y1cr, y2cr;
  
  R_E2 = parms[P_E] * parms[P_E];
  R_E3 = parms[P_E] * R_E2;  
  R_E4 = R_E2 * R_E2;
  R_E6 = R_E2 * R_E4;

  R_C2 = parms[P_C] * parms[P_C];
  
  xc  = parms[P_C] / parms[P_E];
  xc2 = xc * xc;
  
  xt  = parms[P_XT] / parms[P_E];
  xt2 = xt * xt;
  
  Ri_E = parms[P_Ri] / parms[P_E];
  Ro_E = parms[P_Ro] / parms[P_E];
  
  Ri_E2 = Ri_E * Ri_E;
  Ro_E2 = Ro_E * Ro_E;
  
  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(2.0 * d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(2.0 * d_0);
  D  = - parms[P_GAMMA] * sin(2.0 * d_0);

  for(i = 0; i < number_critical; i++) { 
    x12 = x1_caustic[i] * x1_caustic[i];
    x22 = x2_caustic[i] * x2_caustic[i];
    
    r2 = x12 + x22;
    r4 = r2 * r2;
    r6 = r2 * r4;
    rd = sqrt(r2);     

    x1_E = x1_caustic[i] / parms[P_E];
    x2_E = x2_caustic[i] / parms[P_E];
    
    x1_E2 = x1_E * x1_E;
    x2_E2 = x2_E * x2_E;
    
    r_E2 = x1_E2 + x2_E2;
    r_E  = sqrt(r_E2);
    r_E3 = r_E  * r_E2;
    r_E4 = r_E2 * r_E2;
    r_E6 = r_E2 * r_E4;

    y1cr = A1 * x1_caustic[i] + D * x2_caustic[i];
    y2cr = A2 * x2_caustic[i] + D * x1_caustic[i]; 
    
    models[current_model](x1_caustic[i], x2_caustic[i]);
    
    y1_caustic[i] = y1cr - alpha1;
    y2_caustic[i] = y2cr - alpha2;

    /* Magenta */

    draw_point(x1_caustic[i], x2_caustic[i], 1.0, 6);
    
    /* Black */
    
    draw_point(y1_caustic[i], y2_caustic[i], 1.0, 7);    
  }

  /*

  for(i = 0; i < number_critical - 3; i++) { 
    x12 = x1_caustic[i] * x1_caustic[i];
    x22 = x2_caustic[i] * x2_caustic[i];
    
    r2 = x12 + x22;
    r4 = r2 * r2;
    r6 = r2 * r4;
    rd = sqrt(r2);     
    
    x1_E = x1_caustic[i] / parms[P_E];
    x2_E = x2_caustic[i] / parms[P_E];
    
    x1_E2 = x1_E * x1_E;
    x2_E2 = x2_E * x2_E;
    
    r_E2 = x1_E2 + x2_E2;
    r_E  = sqrt(r_E2);
    r_E3 = r_E  * r_E2;
    r_E4 = r_E2 * r_E2;
    r_E6 = r_E2 * r_E4;

    y1cr = A1 * x1_caustic[i] + D * x2_caustic[i];
    y2cr = A2 * x2_caustic[i] + D * x1_caustic[i]; 
    
    models[current_model](x1_caustic[i], x2_caustic[i]);
    
    y1_caustic[i] = y1cr - alpha1;
    y2_caustic[i] = y2cr - alpha2;

    glColor3f(0.0, 0.0, 1.0); 
    
    glBegin(GL_LINES);
    glVertex2d(x1_caustic[i], x2_caustic[i]);   
    glVertex2d(x1_caustic[i + 2], x2_caustic[i + 2]);   
    glEnd();
      
    
    glColor3f(1.0, 0.0, 1.0);
    
    glBegin(GL_LINES);
    glVertex2d(y1_caustic[i + 1], y2_caustic[i + 1]);   
    glVertex2d(y1_caustic[i + 3], y2_caustic[i + 3]);   
    glEnd();
  }

  **/

  /*

  glBegin(GL_LINES);
  glVertex2d(x1_caustic[number_critical - 2], 
	     x2_caustic[number_critical - 2]);   
  glVertex2d(x1_caustic[0], x2_caustic[0]);   
  glEnd();
  
  glBegin(GL_LINES);
  glVertex2d(y1_caustic[number_critical - 1], 
	     y2_caustic[number_critical - 1]);   
  glVertex2d(y1_caustic[1], y2_caustic[1]);   
  glEnd();

  **/

  glFlush();  
}

void caustic_new(int c_critic)
{
  int i;
  double d_0 = (M_PI * parms[P_PHI]) / 180.0;
  double y1cr, y2cr;
  
  R_E2 = parms[P_E] * parms[P_E];
  R_E3 = parms[P_E] * R_E2;  
  R_E4 = R_E2 * R_E2;
  R_E6 = R_E2 * R_E4;

  R_C2 = parms[P_C] * parms[P_C];
  
  xc  = parms[P_C] / parms[P_E];
  xc2 = xc * xc;
  
  xt  = parms[P_XT] / parms[P_E];
  xt2 = xt * xt;
  
  Ri_E = parms[P_Ri] / parms[P_E];
  Ro_E = parms[P_Ro] / parms[P_E];
  
  Ri_E2 = Ri_E * Ri_E;
  Ro_E2 = Ro_E * Ro_E;
  
  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(2.0 * d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(2.0 * d_0);
  D  = - parms[P_GAMMA] * sin(2.0 * d_0);

  for(i = 0; i < number_critical; i++) { 
    x12 = x1_caustic[i] * x1_caustic[i];
    x22 = x2_caustic[i] * x2_caustic[i];
    
    r2 = x12 + x22;
    r4 = r2 * r2;
    r6 = r2 * r4;
    rd = sqrt(r2);     

    x1_E = x1_caustic[i] / parms[P_E];
    x2_E = x2_caustic[i] / parms[P_E];
    
    x1_E2 = x1_E * x1_E;
    x2_E2 = x2_E * x2_E;
    
    r_E2 = x1_E2 + x2_E2;
    r_E  = sqrt(r_E2);
    r_E3 = r_E  * r_E2;
    r_E4 = r_E2 * r_E2;
    r_E6 = r_E2 * r_E4;

    y1cr = A1 * x1_caustic[i] + D * x2_caustic[i];
    y2cr = A2 * x2_caustic[i] + D * x1_caustic[i]; 
    
    models[current_model](x1_caustic[i], x2_caustic[i]);
    
    y1_caustic[i] = y1cr - alpha1;
    y2_caustic[i] = y2cr - alpha2;

    /* Magenta */

    if(c_critic == 1)
      draw_point(x1_caustic[i], x2_caustic[i], 1.0, 6);
    
    /* Black */
    
    if(c_critic == 2)
      draw_point(y1_caustic[i], y2_caustic[i], 1.0, 7);    

    if(c_critic == 3) {
      draw_point(x1_caustic[i], x2_caustic[i], 1.0, 6);
      draw_point(y1_caustic[i], y2_caustic[i], 1.0, 7);    
    }
  }
  
  glFlush();  
}

void critic_caustic()
{
  int i;
  double d_0 = (M_PI * parms[P_PHI]) / 180.0;
  double y1cr, y2cr;
  
  R_E2 = parms[P_E] * parms[P_E];
  R_E3 = parms[P_E] * R_E2;  
  R_E4 = R_E2 * R_E2;
  R_E6 = R_E2 * R_E4;

  R_C2 = parms[P_C] * parms[P_C];
  
  xc  = parms[P_C] / parms[P_E];
  xc2 = xc * xc;
  
  xt  = parms[P_XT] / parms[P_E];
  xt2 = xt * xt;
  
  Ri_E = parms[P_Ri] / parms[P_E];
  Ro_E = parms[P_Ro] / parms[P_E];
  
  Ri_E2 = Ri_E * Ri_E;
  Ro_E2 = Ro_E * Ro_E;
  
  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(2.0 * d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(2.0 * d_0);
  D  = - parms[P_GAMMA] * sin(2.0 * d_0);

  for(i = 0; i < number_critical; i++) { 
    x12 = x1_caustic[i] * x1_caustic[i];
    x22 = x2_caustic[i] * x2_caustic[i];
    
    r2 = x12 + x22;
    r4 = r2 * r2;
    r6 = r2 * r4;
    rd = sqrt(r2);     

    x1_E = x1_caustic[i] / parms[P_E];
    x2_E = x2_caustic[i] / parms[P_E];
    
    x1_E2 = x1_E * x1_E;
    x2_E2 = x2_E * x2_E;
    
    r_E2 = x1_E2 + x2_E2;
    r_E  = sqrt(r_E2);
    r_E3 = r_E  * r_E2;
    r_E4 = r_E2 * r_E2;
    r_E6 = r_E2 * r_E4;

    y1cr = A1 * x1_caustic[i] + D * x2_caustic[i];
    y2cr = A2 * x2_caustic[i] + D * x1_caustic[i]; 
    
    models[current_model](x1_caustic[i], x2_caustic[i]);
    
    y1_caustic[i] = y1cr - alpha1;
    y2_caustic[i] = y2cr - alpha2;
  }
}

