#include <stdio.h>
#include <math.h>

double jacobian(double x, double y); 
double caustic(double x1_caustic, double x2_caustic);
void draw_critical();
void draw_caustic();

double jacobian(double x, double y) 
{
  double fnct1, fnct2, Funct2, funck, funchb, funchg;
  double func1, func2, func3, func4, funcnis, funcrl, funcs, funcg, functs;
  double poly1, poly2, poly3, poly4, poly5, poly6, poly7;
  double dipolm1, dipolm2, quadrupolm1, quadrupolm2; 
  double rbeta, s2, rs12;  
  double B1, B2, D2, yp1, yp2, rd2, xd1, xd2, Mkaprp, Mkapto;  
  double alpha0 = 0.0, alpha1 = 0.0;
  double hubble_factor =  2.0;    
  double ring_factor   = 10.0;
  double Epsilon1 = 1.0 - parms[P_EPSILON];
  double Epsilon2 = 1.0 + parms[P_EPSILON];
  double delta    = 1.0 - parms[P_ALPHA];
  double delta2   = 2.0 - parms[P_ALPHA]; 
  double det_J, j11 = A1, j12 = D, j21 = D, j22 = A2;
  double xbar0 = 0.0, xbar1 = 0.0;  
  double radius_E, radius_E2, radius_E3, radius_E4;
  double alpha, dalph;

  /*

  xl = source_position.x;
  yl = source_position.y;

  **/

  xl = 0.0;
  yl = 0.0;

  xbar0 =   (x - xl) * cs + (y - yl) * ss;
  xbar1 = - (x - xl) * ss + (y - yl) * cs;  

  radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / R_E2;
  radius_E  = sqrt(radius_E2);
  radius_E3 = radius_E  * radius_E2;
  radius_E4 = radius_E2 * radius_E2;

  switch(current_model) {
  case M_CHANG_REFSDAL:
    j11 -= (xbar1 * xbar1 - xbar0 * xbar0) / radius_E4;
    j12 -= (2.0 * xbar0 * xbar1) / radius_E4;
    j21  = j12;
    j22 -= (xbar0 * xbar0 - xbar1 * xbar1) / radius_E4;
  
    break;
  case M_SECOND:
    
    break;      
  case M_SING_ISOT_SPHERE:
    rbeta = pow(radius_E, (parms[P_EXP] - 4.0));
    
    j11 -= rbeta * (xbar1 * xbar1 + (parms[P_EXP] - 1.0) * xbar0 * xbar0);
    j12 -= rbeta * (parms[P_EXP] - 2.0) * xbar0 * xbar1;
    j21  = j12;
    j22 -= rbeta * (xbar0 * xbar0 + (parms[P_EXP] - 1.0) * xbar1 * xbar1);

    break;      
  case M_DE_VAUCOULEUR:
    poly1 = 720.0 / tau8 + (720.0 * pow(radius_E, (1.0 / 4.0))) / tau7;
    poly2 = poly1 + pow(radius_E, (3.0 / 2.0)) / tau2;
    poly3 = poly2 + (6.0 * pow(radius_E, (5.0 / 4.0))) / tau3;
    poly4 = poly3 + (30.0 * radius_E) / tau4;
    poly5 = poly4 + (120.0 * pow(radius_E, (3.0 / 4.0))) / tau5;
    poly6 = poly5 + (360.0 * sqrt(radius_E)) / tau6;
    poly7 = 7.0 * poly6 + pow(radius_E, (7.0 / 4.0)) / tau;

    func1 = poly7 * exp(- tau * (pow(radius_E, (1.0 / 4.0)) - 1.0));
    func2 = func1 - (5040.0 * Ftau) / tau8;

    alpha = - (8.0 * parms[P_KAPPA] * func2) / radius_E;
    
    poly1 = pow(radius_E, (3.0 / 4.0)) / tau;
    poly2 = poly1 + 720.0 * pow(radius_E, - (3.0 / 4.0)) / tau7;
    poly3 = poly2 + 6.0 * sqrt(radius_E) / tau2;
    poly4 = poly3 + 30.0 * pow(radius_E, (1.0 / 4.0)) / tau3;
    poly5 = poly4 + 360.0 * pow(radius_E, - (1.0 / 4.0)) / tau5; 
    poly6 = poly5 + 720.0 / (tau6 * sqrt(radius_E));  
    poly7 = poly6 + 120.0 / tau4;  

    func3 = 7.0 * poly7 * exp(- tau * (pow(radius_E, (1.0 / 4.0)) - 1.0));
    func4 = (func3 - tau * func1 * pow(radius_E, - (3.0 / 4.0))) / 4.0;

    dalph = 8.0 * parms[P_KAPPA] * (func2 / radius_E2 - func4 / radius_E);

    j11 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar0 * xbar0) / radius_E3;
    j12 -= ((radius_E * dalph - alpha) * xbar0 * xbar1) / radius_E3;
    j21  = j12;
    j22 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar1 * xbar1) / radius_E3;

    break;      
  case M_KING: 
    /* log1p(x) = log(1.0 + x) */

    funck = (radius_E2 / (1.0 + radius_E2) - log1p(radius_E2)) / radius_E4;
    
    j11 -= (parms[P_KAPPA] * log1p(radius_E2)) / radius_E2 
      + 2.0 * parms[P_KAPPA] * xbar0 * xbar0 * funck;
    j12 -= 2.0 * parms[P_KAPPA] * xbar0 * xbar1 * funck;
    j21 = j12;
    j22 -= (parms[P_KAPPA] * log1p(radius_E2)) / radius_E2 
      + 2.0 * parms[P_KAPPA] * xbar1 * xbar1 * funck;
    
    break;      
  case M_HUBBLE:
    funchb = log1p(radius_E) - radius_E / (1.0 + radius_E);
    funchg = 2.0 * funchb - radius_E2 / ((1.0 + radius_E) * (1.0 + radius_E));

    j11 -= hubble_factor * parms[P_KAPPA] 
      * (funchb - (funchg * xbar0 * xbar0) / radius_E2) / radius_E2;
    j12 += hubble_factor * parms[P_KAPPA] 
      * ((funchg * xbar0 * xbar1) / radius_E4);
    j21 = j12;
    j22 -= hubble_factor * parms[P_KAPPA] 
      * (funchb - (funchg * xbar1 * xbar1) / radius_E2) / radius_E2;

    break;      
  case M_SPIRAL:
    funcs = 2.0 * parms[P_KAPPA] * (1.0 - exp(- parms[P_ALPHA2] * radius_E) 
				    * (parms[P_ALPHA2] * radius_E + 1.0));
    funcg = parms[P_KAPPA] * exp(- parms[P_ALPHA2] * radius_E) 
      - funcs / (parms[P_ALPHA2] * parms[P_ALPHA2] * radius_E2);

    j11 -= (funcs / (parms[P_ALPHA2] * parms[P_ALPHA2]) 
	    + 2.0 * funcg * xbar0 * xbar0) / radius_E2;
    j12 -= 2.0 * funcg * xbar0 * xbar1 / radius_E2;
    j21 = j12;
    j22 -= (funcs / (parms[P_ALPHA2] * parms[P_ALPHA2]) 
	    + 2.0 * funcg * xbar1 * xbar1) / radius_E2;

    break;      
  case M_TRUNC_KING:
    /* hypot(x, y) = sqrt(x * x + y * y) */

    func1 = log1p(radius_E2) + radius_E2 / (1.0 + xc2); 
    func2 = (4.0 * (hypot(1.0, radius_E) - 1.0)) / hypot(1.0, xc);
    func3 = func1 - func2;
    func4 = 1.0 / hypot(1.0, radius_E) + 1.0 / hypot(1.0, xc) 
      - 2.0 / sqrt((1.0 + xc2) * (1.0 + radius_E2));
    
    alpha = (parms[P_KAPPA2] * func3) / radius_E2; 
    dalph = (parms[P_KAPPA2] * (2.0 * func4 - func3 / radius_E2));

    j11 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar0 * xbar0) / radius_E3;
    j12 -= ((radius_E * dalph - alpha) * xbar0 * xbar1) / radius_E3;
    j21  = j12;
    j22 -= alpha / radius_E 
      + ((radius_E * dalph - alpha) * xbar1 * xbar1) / radius_E3;

    break;      
  case M_NSING_ISOT_SPHERE:
     s2 = xc2 + radius_E2;

     funcnis = sqrt(s2) - xc;     

     j11 -= parms[P_KAPPA] 
       * ((funcnis * (xbar1 * xbar1 - xbar0 * xbar0)) / radius_E4 
	  + xbar0 * xbar0 / sqrt(s2));
     j12 -= parms[P_KAPPA] 
       * (1.0 / sqrt(s2) - 2.0 * funcnis / radius_E4) * (xbar0 * xbar1); 
     j21  = j12;
     j22 -= parms[P_KAPPA] 
       * ((funcnis * (xbar0 * xbar0 - xbar1 * xbar1)) / radius_E4 
	  + xbar1 * xbar1 / sqrt(s2));
     
     break;      
  case M_TRANSP_SPHERE:

    break;
  case M_ELLIPTICAL:
    s2 = xc2 + (Epsilon1 * xbar0 * xbar0 + Epsilon2 * xbar1 * xbar1) / R_E2;

    j11 -= parms[P_KAPPA] * (Epsilon1 * s2 - 2.0 * delta * Epsilon1 * Epsilon1 
			     * xbar0 * xbar0) / pow(s2, delta2);
    j12 += 2.0 * delta * parms[P_KAPPA] * Epsilon1 * Epsilon2 
      * xbar0 * xbar1 / pow(s2, delta2);
    j21 = j12; 
    j22 -= parms[P_KAPPA] * (Epsilon2 * s2 - 2.0 * delta * Epsilon2 * Epsilon2 
			     * xbar1 * xbar1) / pow(s2, delta2);

    break;      
  case M_MULTIPOLE:

    break;
  case M_ROTATION_LENS:

    break;
  case M_DOUBLE_LENS:
    
    break;
  case M_UNIFORM_RING:
          
    break;
  default:
    break;
  }  
  
  det_J = j11 * j22 - j12 * j21;

  return(det_J);
}

void draw_critical()
{
  int i, j, kc = 50;
  double x, y, d[Nd];
  
  for(i = 0; i < kc; ++i) {
    for(j = 0; j < kc; ++j) {
      x = ((double) i);
      y = ((double) j);
      
      d[i + j * Np] = jacobian(x, y);
    }
  }
  
  cntr(&c_M, &c_N, &c_10, &c_12, &c_10, &c_12, 
       &d[0], &c_3, &c_0, &c_b11, &c_0);  
}

double caustic(double x1_caustic, double x2_caustic)
{
  double d_0 = (M_PI * parms[P_PHI]) / 180.0;
  double y1_caustic, y2_caustic;
  double y1cr, y2cr, caustic;
  
  R_E2 = parms[P_E] * parms[P_E];
  R_E3 = parms[P_E] * R_E2;  
  R_E4 = R_E2 * R_E2;
  R_E6 = R_E2 * R_E4;

  R_C2 = parms[P_C] * parms[P_C];
  
  xc  = parms[P_C] / parms[P_E];
  xc2 = xc * xc;
  
  xt  = parms[P_XT] / parms[P_E];
  xt2 = xt * xt;
  
  Ri_E = parms[P_Ri] / parms[P_E];
  Ro_E = parms[P_Ro] / parms[P_E];
  
  Ri_E2 = Ri_E * Ri_E;
  Ro_E2 = Ro_E * Ro_E;
  
  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(2.0 * d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(2.0 * d_0);
  D  = - parms[P_GAMMA] * sin(2.0 * d_0);
  
  x12 = x1_caustic * x1_caustic;
  x22 = x2_caustic * x2_caustic;
  
  r2 = x12 + x22;
  r4 = r2 * r2;
  r6 = r2 * r4;
  rd = sqrt(r2);     
  
  x1_E = x1_caustic / parms[P_E];
  x2_E = x2_caustic / parms[P_E];
  
  x1_E2 = x1_E * x1_E;
  x2_E2 = x2_E * x2_E;
  
  r_E2 = x1_E2 + x2_E2;
  r_E  = sqrt(r_E2);
  r_E3 = r_E  * r_E2;
  r_E4 = r_E2 * r_E2;
  r_E6 = r_E2 * r_E4;
  
  y1cr = A1 * x1_caustic + D * x2_caustic;
  y2cr = A2 * x2_caustic + D * x1_caustic; 
  
  models[current_model](x1_caustic, x2_caustic);
  
  y1_caustic = y1cr - alpha1;
  y2_caustic = y2cr - alpha2;

  caustic = y1_caustic * y1_caustic + y2_caustic * y2_caustic;

  return(caustic);
}

void draw_caustic()
{
  int i, j, kc = 50;
  double x, y, d[Nd];
  
  for(i = 0; i < kc; ++i) {
    for(j = 0; j < kc; ++j) {
      x = ((double) i);
      y = ((double) j);
      
      d[i + j * Np] = caustic(x, y);
    }
  }
  
  cntr(&c_M, &c_N, &c_10, &c_12, &c_10, &c_12, 
       &d[0], &c_3, &c_0, &c_b11, &c_0);  
}
