#include <stdio.h>
#include <math.h>
#include "glxf.h"

double csgn(double x1, double x2);

void model_menu(double x1, double x2, double x10, double x20)
{
  glColor3f(0.0, 0.0, 0.0);
}

void uniform_ring(double x1, double x2, double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double ring_factor = 10.0;

  rs12 /= R_E2;

  Mkapr = ring_factor * parms[P_KAPPA] * (r_E2 - Ri_E2);
  Mkapt = ring_factor * parms[P_KAPPA] * (Ro_E2 - Ri_E2);

  if(rd >= parms[P_Ro]) {
    alpha1 = (Mkapt * (x1 - x10)) / rs12;
    alpha2 = (Mkapt * (x2 - x20)) / rs12; 	     
  } else {
    if(rd >= parms[P_Ri]) {      
      alpha1 = (Mkapr * (x1 - x10)) / rs12;
      alpha2 = (Mkapr * (x2 - x20)) / rs12; 	
    } else {
      alpha1 = 0.0;
      alpha2 = 0.0; 
    }
  }
}

void chang_refsdal(double x1, double x2, double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));

  rs12 /= R_E2;

  alpha1 = (x1 - x10) / rs12;
  alpha2 = (x2 - x20) / rs12; 
}

void second_order(double x1, double x2, double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  
  rs12 /= R_E2;  
    
  alpha1 = (x1 - x10) * (1.0 / r_E2 + parms[P_Ko] / r_E3);
  alpha2 = (x2 - x20) * (1.0 / r_E2 + parms[P_Ko] / r_E3); 
}

void double_pmass(double x1, double x2, double x10, double x20)
{
  double rs12, xd1, xd2, rd2, A12, A22, D2, yp1, yp2;
  double d_0 = (M_PI * parms[P_PHI2]) / 180.0;  
  
  rs12 = (x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20);
  
  rs12 /= R_E2;

  A12 = 1.0 - parms[P_SIGMA2] - parms[P_GAMMA2] * cos(2.0 * d_0);
  A22 = 1.0 - parms[P_SIGMA2] + parms[P_GAMMA2] * cos(2.0 * d_0);
  D2  = - parms[P_GAMMA2] * sin(2.0 * d_0);
  
  yp1 = A12 * x1 + D2 * x2;
  yp2 = A22 * x2 + D2 * x1; 
  
  xd1 = yp1 - parms[P_M_1] * parms[P_BETA] * (x1 - x10) / rs12; 
  xd2 = yp2 - parms[P_M_1] * parms[P_BETA] * (x2 - x20) / rs12; 

  rd2 = (xd1 - parms[P_X_2]) * (xd1 - parms[P_X_2])
    + (xd2 - parms[P_Y_2]) * (xd2 - parms[P_Y_2]);
  
  rd2 /= R_E2;

  alpha1 = parms[P_M_1] * (x1 - parms[P_X_1]) / rs12
    + parms[P_M_2] * (xd1 - parms[P_X_2]) / rd2; 
  alpha2 = parms[P_M_1] * (x2 - parms[P_Y_1]) / rs12
    + parms[P_M_2] * (xd2 - parms[P_Y_2]) / rd2;
}

void double_lenses(double x1, double x2, double x10, double x20)
{
  double rs12, xd1, xd2, rd2, A12, A22, D2, yp1, yp2;
  double d_0 = (M_PI * parms[P_PHI2]) / 180.0;  

  /*

  double rs12;

  rs12 = (x1 - parms[P_X_1]) * (x1 - parms[P_X_1])
    + (x2 - parms[P_Y_1]) * (x2 - parms[P_Y_1]);
  
  rs12 /= R_E2;

  **/

  model_1[current_model1](x1, x2, x10, x20);
  
  A12 = 1.0 - parms[P_SIGMA2] - parms[P_GAMMA2] * cos(2.0 * d_0);
  A22 = 1.0 - parms[P_SIGMA2] + parms[P_GAMMA2] * cos(2.0 * d_0);
  D2  = - parms[P_GAMMA2] * sin(2.0 * d_0);
  
  yp1 = A12 * x1 + D2 * x2;
  yp2 = A22 * x2 + D2 * x1; 
  
  xd1 = yp1 - parms[P_M_1] * parms[P_BETA] * alpha1px;
  xd2 = yp2 - parms[P_M_1] * parms[P_BETA] * alpha1py;

  rd2  = (xd1 - parms[P_X_2]) * (xd1 - parms[P_X_2])
    + (xd2 - parms[P_Y_2]) * (xd2 - parms[P_Y_2]);

  // rd2 /= R_E2;

  model_2[current_model2](x1, x2, parms[P_X_2], parms[P_Y_2], rd2);
  
  alpha1 = parms[P_M_1] * alpha1px + parms[P_M_2] * alpha2px; 
  alpha2 = parms[P_M_1] * alpha1py + parms[P_M_2] * alpha2py; 
}

void singular_isothermal_sphere(double x1, double x2, double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double rbeta, r_s; 

  rs12 /= R_E2;
  r_s   = sqrt(rs12);

  rbeta  = pow(r_s, (parms[P_EXP] - 1.0));

  alpha1 = rbeta * (x1 - x10);
  alpha2 = rbeta * (x2 - x20);  
}       

void nonsingular_isothermal_sphere(double x1, double x2, 
				   double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double s2; 

  rs12 /= R_E2;
  
  s2 = xc2 + rs12;

  func = sqrt(s2) - xc; 

  alpha1 = (parms[P_KAPPA] * func * (x1 - x10)) / rs12;
  alpha2 = (parms[P_KAPPA] * func * (x2 - x20)) / rs12;  
}

void transparent_sphere(double x1, double x2, double x10, double x20)
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double s2; 

  if(parms[P_C] > rd) {
    s2 = 1.0 - rs12 / R_C2;
    
    func = 1.0 - pow(s2, (3.0 / 2.0));

    rs12 /= R_E2;
 
    alpha1 = (func * (x1 - x10)) / rs12;
    alpha2 = (func * (x2 - x20)) / rs12;  
  } else {
    rs12 /= R_E2;
    
    alpha1 = (x1 - x10) / rs12;
    alpha2 = (x2 - x20) / rs12; 
  }
}

void elliptical(double x1, double x2, double x10, double x20)
{
  double Epsilon1 = 1.0 - parms[P_EPSILON];
  double Epsilon2 = 1.0 + parms[P_EPSILON];
  double delta    = 1.0 - parms[P_ALPHA];
  double rsx2 = (x1 - x10) * (x1 - x10); 
  double rsy2 = (x2 - x20) * (x2 - x20);
  double s2 = xc2 + (Epsilon1 * rsx2 + Epsilon2 * rsy2) / R_E2; 
  
  alpha1 = (parms[P_KAPPA] * Epsilon1 * (x1 - x10)) / pow(s2, delta);
  alpha2 = (parms[P_KAPPA] * Epsilon2 * (x2 - x20)) / pow(s2, delta);
}

void king(double x1, double x2, double x10, double x20) 
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  
  /* log1p(x) = log(1.0 + x) */
  
  rs12 /= R_E2;

  alpha1 = (parms[P_KAPPA] * (x1 - x10) * log1p(rs12)) / rs12;
  alpha2 = (parms[P_KAPPA] * (x2 - x20) * log1p(rs12)) / rs12; 
}

void truncated_king(double x1, double x2, double x10, double x20) 
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double r_s;

  rs12 /= R_E2;

  r_s = sqrt(rs12);

  /* hypot(x, y) = sqrt(x * x + y * y) */
  
  func  = log1p(rs12) + rs12 / (1.0 + xc2); 
  func -= (4.0 * (hypot(1.0, r_s) - 1.0)) / hypot(1.0, xc);

  alpha1 = (parms[P_KAPPA2] * (x1 - x10) * func) / rs12; 
  alpha2 = (parms[P_KAPPA2] * (x2 - x20) * func) / rs12;
}             

void hubble(double x1, double x2, double x10, double x20)  
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double hubble_factor = 2.0;
  double r_s;

  rs12 /= R_E2;

  r_s = sqrt(rs12);

  func = log1p(r_s) - r_s / (1.0 + r_s); 
   
  alpha1 = (hubble_factor * parms[P_KAPPA] * (x1 - x10) * func) / rs12;
  alpha2 = (hubble_factor * parms[P_KAPPA] * (x2 - x20) * func) / rs12;  
}

void de_vaucouleur(double x1, double x2, double x10, double x20)  
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double hubble_factor = 2.0;
  double r_s, poly;

  rs12 /= R_E2;

  r_s = sqrt(rs12);

  poly  = 720.0 / tau8 + (720.0 * pow(r_s, (1.0 / 4.0))) / tau7;
  poly += pow(r_s, (3.0 / 2.0)) / tau2;
  poly += (6.0 * pow(r_s, (5.0 / 4.0))) / tau3;
  poly += (30.0 * r_s) / tau4;
  poly += (120.0 * pow(r_s, (3.0 / 4.0))) / tau5;
  poly += (360.0 * sqrt(r_s)) / tau6;
  poly  = 7.0 * poly + pow(r_s, (7.0 / 4.0)) / tau;

  func  = poly * exp(- tau * (pow(r_s, (1.0 / 4.0)) - 1.0));
  func -= (5040.0 * Ftau) / tau8;

  alpha1 = - (8.0 * parms[P_KAPPA] * (x1 - x10) * func) / rs12;
  alpha2 = - (8.0 * parms[P_KAPPA] * (x2 - x20) * func) / rs12;  
}

void spiral(double x1, double x2, double x10, double x20)  
{
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double r_s;

  rs12 /= R_E2;

  r_s = sqrt(rs12);

  func = 2.0 * parms[P_KAPPA] 
    * (1.0 - exp(- parms[P_ALPHA2] * r_s) * (parms[P_ALPHA2] * r_s + 1.0));
  
  alpha1 = ((x1 - x10) * func) / (parms[P_ALPHA2] * parms[P_ALPHA2] * rs12);
  alpha2 = ((x2 - x20) * func) / (parms[P_ALPHA2] * parms[P_ALPHA2] * rs12);  
}

void multipole(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);
  double rs12 = (rsx * rsx + rsy * rsy);
  double dipol1, dipol2;
  double quadp1, quadp2, qdipterm1, qdipterm2;
  double rs14, rs16;

  rs12 /= R_E2;
  rs14  = rs12 * rs12;
  rs16  = rs12 * rs14;
  
  dif = rsy * rsy - rsx * rsx;

  dipol1 = (parms[P_DX_QP] * dif 
	    - 2.0 * parms[P_DY_QP] * rsx * rsy) / parms[P_E];
  dipol2 = (- parms[P_DY_QP] * dif 
	    - 2.0 * parms[P_DX_QP] * rsx * rsy) / parms[P_E];

  qdipterm1 = (rsy * (3.0 * rsx * rsx - rsy * rsy)) / R_E2;
  qdipterm2 = (rsx * (3.0 * rsy * rsy - rsx * rsx)) / R_E2;
    
  quadp1 = 2.0 * (parms[P_QXX] * qdipterm2 - parms[P_QXY] * qdipterm1);
  quadp2 = 2.0 * (parms[P_QYY] * qdipterm1 - parms[P_QXY] * qdipterm2);  
  
  alpha1 = (parms[P_KAPPA0] * rsx) / rs12 + (dipol1 / rs14 + quadp1 / rs16);
  alpha2 = (parms[P_KAPPA0] * rsy) / rs12 + (dipol2 / rs14 + quadp2 / rs16); 
}

void rotation_lens(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);  
  double rs12 = ((x1 - x10) * (x1 - x10) + (x2 - x20) * (x2 - x20));
  double rs14;

  rs12 /= R_E2;
  rs14  = rs12 * rs12;

  func = (parms[P_SX] * rsy - parms[P_SY] * rsx) / parms[P_E];
  
  alpha1 = rsx / r_E2 + (2.0 * func * rsx) / rs14
    + parms[P_SY] / (parms[P_E] * rs12);
  alpha2 = rsy / r_E2 + (2.0 * func * rsy) / rs14
    - parms[P_SX] / (parms[P_E] * rs12);
}

void Elliptic_King(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);  
  double rs12 = (rsx * rsx + rsy * rsy);
  double k, x, zm, phi, f, h, fxc;
  double f1_rp, f1_ip, f1m, phi1, f2_rp, f2_ip, f2m, phi2;
  double f3_rp, f3_ip, f3m, phi3, f4_rp, f4_ip, f4m, phi4;
  double f5_rp, f5_ip, f5m, phi5, f6_rp, f6_ip, f6m, phi6;
  double f7_rp, f7_ip, f7m, phi7, f8_rp, f8_ip, f8m, phi8;
  double Fm, phim;
  
  k = parms[P_KAPPA];
  f = parms[P_EPSILON2];   
  h = sqrt(1.0 - f * f);

  rsx = rsx / parms[P_E];
  rsy = rsy / parms[P_E];

  x   = sqrt(rsx * rsx + f * f * rsy * rsy);
  fxc = f * f * (1.0 + (x * x) / (xc * xc));
  
  zm  = sqrt(rsx * rsx + rsy * rsy);
  phi = atan2(rsy, rsx);
  
  f1_rp = f * f * (rsx * rsx - rsy * rsy) - h * h * xc * xc;
  f1_ip = 2 * f * f * rsx * rsy;

  f1m  = sqrt(f1_rp * f1_rp + f1_ip * f1_ip);
  phi1 = atan2(f1_ip, f1_rp);

  f2_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * x * x;
  f2_ip = f1_ip;

  f2m  = sqrt(f2_rp * f2_rp + f2_ip * f2_ip * f2_ip);	
  phi2 = atan2(f2_ip, f2_rp);

  f3_rp = f * f * (rsx * rsx - rsy * rsy); 
  f3_ip = f1_ip + h * h * x * xc;
  
  f3m  = sqrt(f3_rp * f3_rp + f3_ip * f3_ip);
  phi3 = atan2(f3_ip, f3_rp);
  
  f4_rp = f3_rp; 
  f4_ip = f1_ip - h * h * x * xc;
  
  f4m  = sqrt(f4_rp * f4_rp + f4_ip * f4_ip);
  phi4 = atan2(f4_ip, f4_rp); 
    
  f5_rp = f * x1 + sqrt(f1m) * cos(phi / 2.0) * csgn(rsx, rsy);
  f5_ip = f * x2 + sqrt(f1m) * sin(phi / 2.0) * csgn(rsx, rsy);
 
  f5m  = sqrt(f5_rp * f5_rp + f5_ip * f5_ip);
  phi5 = atan2(f5_ip, f5_rp);

  f6_rp = zm * f5m * cos(phi + phi5);
  f6_ip = zm * f5m * sin(phi + phi5);
 
  f6m  = sqrt(f6_rp * f6_rp + f6_ip * f6_ip);
  phi6 = atan2(f6_ip, f6_rp);

  f7_rp = f3m * cos(phi3) + sqrt(f1m * f2m) * cos((phi + phi2) / 2.0);
  f7_ip = f3m * sin(phi3) + sqrt(f1m * f2m) * sin((phi + phi2) / 2.0);
 
  f7m  = sqrt(f7_rp * f7_rp + f7_ip * f7_ip);
  phi7 = atan2(f7_ip, f7_rp);

  f8_rp = f4m * cos(phi4) + sqrt(f1m * f2m) * cos((phi + phi2) / 2.0);
  f8_ip = f4m * sin(phi4) + sqrt(f1m * f2m) * sin((phi + phi2) / 2.0);
 
  f8m  = sqrt(f8_rp * f8_rp + f8_ip * f8_ip);
  phi8 = atan2(f8_ip, f8_rp);

  Fm   = fxc * f6m * f6m / (f7m * f8m);
  phim = 2.0 * phi6 - (phi7 + phi8);

  alpha1 = (k * (log(Fm) * cos(phi1) + phim * sin(phi1))) / sqrt(f1m);
  alpha2 = (k * (log(Fm) * sin(phi1) - phim * cos(phi1))) / sqrt(f1m);
}

void Elliptic_NIS(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);  
  double rs12 = (rsx * rsx + rsy * rsy);
  double k, x, zm, phi, f, h, fxc;
  double f1_rp, f1_ip, f1m, phi1, f2_rp, f2_ip, f2m, phi2;
  double f3_rp, f3_ip, f3m, phi3, f4_rp, f4_ip, f4m, phi4;
  double f5_rp, f5_ip, f5m, phi5, f6_rp, f6_ip, f6m, phi6;
  double f7_rp, f7_ip, f7m, phi7, f8_rp, f8_ip, f8m, phi8;
  double f9_rp, f9_ip, f9m, phi9;

  k = parms[P_KAPPA];
  f = parms[P_EPSILON2];   
  h = sqrt(1.0 - f * f);  

  rsx = rsx / parms[P_E];
  rsy = rsy / parms[P_E];

  x   = sqrt(rsx * rsx + f * f * rsy * rsy);
  fxc = f * f * (1.0 + (x * x) / (xc * xc));
  
  zm  = sqrt(rsx * rsx + rsy * rsy);
  phi = atan2(rsx, rsx);
  
  f1_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * x * x;
  f1_ip = 2 * f * f * rsx * rsy;

  f1m  = sqrt(f1_rp * f1_rp + f1_ip * f1_ip);
  phi1 = atan2(f1_ip, f1_rp);

  f2_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * (2 * x * x + xc * xc);
  f2_ip = f1_ip;

  f2m  = sqrt(f2_rp * f2_rp + f2_ip * f2_ip * f2_ip);	
  phi2 = atan2(f2_ip, f2_rp);

  f3_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * xc * xc; 
  f3_ip = f1_ip;
  
  f3m  = sqrt(f3_rp * f3_rp + f3_ip * f3_ip);
  phi3 = atan2(f3_ip, f3_rp);
  
  f4_rp = f2m * cos(phi2) 
    - 2.0 * h * sqrt(f1m * (xc* xc + x * x)) * cos(phi1 / 2.0); 
  f4_ip = f2m * sin(phi2) 
    - 2.0 * h * sqrt(f1m * (xc* xc + x * x)) * sin(phi1 / 2.0); 
  
  f4m  = sqrt(f4_rp * f4_rp + f4_ip * f4_ip);
  phi4 = atan2(f4_ip, f4_rp); 
    
  f5_rp = f2m * cos(phi2) 
    + 2.0 * h * sqrt(f1m * (xc* xc + x * x)) * cos(phi1 / 2.0); 
  f5_ip = f2m * sin(phi2) 
    + 2.0 * h * sqrt(f1m * (xc* xc + x * x)) * sin(phi1 / 2.0); 
 
  f5m  = sqrt(f5_rp * f5_rp + f5_ip * f5_ip);
  phi5 = atan2(f5_ip, f5_rp);

  f6_rp = (f4m * cos(phi4 - phi1 / 2.0)) / sqrt(f1m * (xc * xc + x * x)); 
  f6_ip = (f4m * sin(phi4 - phi1 / 2.0)) / sqrt(f1m * (xc * xc + x * x)); 
 
  f6m  = sqrt(f6_rp * f6_rp + f6_ip * f6_ip);
  phi6 = atan2(f6_ip, f6_rp);

  f7_rp = (f5m * cos(phi4 - phi1 / 2.0)) / sqrt(f1m * (xc * xc + x * x)); 
  f7_ip = (f5m * sin(phi4 - phi1 / 2.0)) / sqrt(f1m * (xc * xc + x * x)); 
   
  f7m  = sqrt(f7_rp * f7_rp + f7_ip * f7_ip);
  phi7 = atan2(f7_ip, f7_rp);

  f8_rp = (f3m * cos(phi3 - phi) / zm - 2 * h * xc * csgn(rsx, rsy)) 
    * csgn(rsx, rsy);
  f8_ip = (f3m * sin(phi3 - phi) * csgn(rsx, rsy)) / zm;
 
  f8m  = sqrt(f8_rp * f8_rp + f8_ip * f8_ip);
  phi8 = atan2(f8_ip, f8_rp);

  f9_rp = (f3m * cos(phi3 - phi) / zm + 2 * h * xc * csgn(rsx, rsy)) 
    * csgn(rsx, rsy);
  f9_ip = (f3m * sin(phi3 - phi) * csgn(rsx, rsy)) / zm;
 
  f9m  = sqrt(f9_rp * f9_rp + f9_ip * f9_ip);
  phi9 = atan2(f9_ip, f9_rp);

  alpha1 = k * log((f7m * f8m) / (f6m * f9m)) / (2.0 * h);
  alpha2 = k * (phi6 - phi7 + phi9 - phi8) / (2.0 * h);
}

void Elliptic_Hubble(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);  
  double rs12 = (rsx * rsx + rsy * rsy);
  double k, x, zm, phi, f, h, fxc;
  double f1_rp, f1_ip, f1m, phi1, f2_rp, f2_ip, f2m, phi2;
  double f3_rp, f3_ip, f3m, phi3, f4_rp, f4_ip, f4m, phi4;
  double f5_rp, f5_ip, f5m, phi5, f6_rp, f6_ip, f6m, phi6;
  double f7_rp, f7_ip, f7m, phi7, f8_rp, f8_ip, f8m, phi8;
  double Fm, phim, alpha1, alpha2;

  k = parms[P_KAPPA];
  f = parms[P_EPSILON2];   
  h = sqrt(1.0 - f * f);  

  rsx = rsx / parms[P_E];
  rsy = rsy / parms[P_E];

  x   = sqrt(rsx * rsx + f * f * rsy * rsy);
  fxc = f * f * (1.0 + (x * x) / (xc * xc));
  
  zm  = sqrt(rsx * rsx + rsy * rsy);
  phi = atan2(rsy, rsx);
  
  f1_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * xc * xc;
  f1_ip = 2 * f * f * rsx * rsy;

  f1m  = sqrt(f1_rp * f1_rp + f1_ip * f1_ip);
  phi1 = atan2(f1_ip, f1_rp);

  f2_rp = f * f * (rsx * rsx - rsy * rsy) + h * h * x * x;
  f2_ip = f1_ip;

  f2m  = sqrt(f2_rp * f2_rp + f2_ip * f2_ip * f2_ip);	
  phi2 = atan2(f2_ip, f2_rp);

  f3_rp = f * f * (rsx * rsx - rsy * rsy) - h * h * xc * x; 
  f3_ip = f1_ip;
  
  f3m  = sqrt(f3_rp * f3_rp + f3_ip * f3_ip);
  phi3 = atan2(f3_ip, f3_rp);
  
  f4_rp = f3m * cos(phi3) + sqrt(f1m * f2m) * cos((phi1 + phi2) / 2.0); 
  f4_ip = f3m * sin(phi3) + sqrt(f1m * f2m) * sin((phi1 + phi2) / 2.0); 
  
  f4m  = sqrt(f4_rp * f4_rp + f4_ip * f4_ip);
  phi4 = atan2(f4_ip, f4_rp); 
    
  f5_rp = f * rsx + sqrt(f1m) * cos(phi / 2.0) * csgn(rsx, rsy);
  f5_ip = f * rsy + sqrt(f1m) * sin(phi / 2.0) * csgn(rsx, rsy);
 
  f5m  = sqrt(f5_rp * f5_rp + f5_ip * f5_ip);
  phi5 = atan2(f5_ip, f5_rp);

  f6_rp = zm * sqrt(f1m) * cos(phi + phi1 / 2.0);
  f6_ip = zm * sqrt(f1m) * sin(phi + phi1 / 2.0);
 
  f6m  = sqrt(f6_rp * f6_rp + f6_ip * f6_ip);
  phi6 = atan2(f6_ip, f6_rp);

  f7_rp = zm * f5m * cos(phi + phi5);
  f7_ip = zm * f5m * sin(phi + phi5);
   
  f7m  = sqrt(f7_rp * f7_rp + f7_ip * f7_ip);
  phi7 = atan2(f7_ip, f7_rp);

  f8_rp = f * f * zm * zm * (x + xc) * cos(2.0 * phi) 
    * log((xc * f4m) /(f * (x + xc) * f7m)) 
    - xc * sqrt(f1m * f2m) * cos((phi1 + phi2) / 2.0)
    + f * (xc + x) * (f6m * cos(phi6) * csgn(rsx, rsy) 
		      + f * zm * zm * sin(2.0 * phi) *(phi7 - phi4));

  f8_ip = f * f * zm * zm * (x + xc) * sin(2.0 * phi) 
    * log((xc * f4m) /(f * (x + xc) * f7m)) 
    - xc * sqrt(f1m * f2m) * sin((phi1 + phi2) / 2.0)
    + f * (xc + x) * (f6m * sin(phi6) * csgn(x1, x2) 
		      + f * zm * zm * cos(2.0 * phi) *(phi4 - phi7));
 
  f8m  = sqrt(f8_rp * f8_rp + f8_ip * f8_ip);
  phi8 = atan2(f8_ip, f8_rp);

  Fm   = 2.0 * k * f8m / ((xc + x) * sqrt(f1m * f1m * f1m));
  phim = phi8 - 3.0 * phi1 / 2.0;

  alpha1 = - Fm * cos(phim);
  alpha2 =   Fm * sin(phim);

  /*

  printf("alpha1 = %g \n", alpha1);
  printf("alpha2 = %g \n", alpha2);

  **/
}

void Elliptic_Plummer(double x1, double x2)
{
  alpha1 = 0.0;
  alpha2 = 0.0;
}

/*

               /  1    if Re(x) > 0 or Re(x) = 0 and Im(x) > 0
    csgn(x) = <
               \ -1    if Re(x) < 0 or Re(x) = 0 and Im(x) < 0

**/

double csgn(double x1, double x2) 
{
  double csignum = 1.0;

  if((x1 > 0.0 || x1 == 0.0) && x2 > 0) {
    csignum = 1.0;
  } else if((x1 < 0.0 || x1 == 0.0) && x2 < 0) {
    csignum = - 1.0;
  }

  return(csignum);    
}

/* Barkana's Fastell */

#include "Fastell_2.c"
#include "Dqnc79_2.c"

void Barkana(double x1, double x2, double x10, double x20)   
{
  double rsx = (x1 - x10);
  double rsy = (x2 - x20);  
  double q, s2, x1in, x2in, gam, defl[2], arat, magmx[4];
  double Magxx, Magxy, Magyx, Magyy; 

  /* magmx was [2][2] */

  /* Subroutine */ 
  
  extern int fastellmag();
  
  x1in = rsx / parms[P_E];
  x2in = rsy / parms[P_E];
  q    = parms[P_KAPPA];
  gam  = parms[P_ALPHA3];
  arat = parms[P_EPSILON2];
  s2   = xc2;
  
  fastellmag(&x1in, &x2in, &q, &gam, &arat, &s2, defl, magmx);
  
  alpha1 = defl[0];
  alpha2 = defl[1];
  
  /* Jac[0][0] = Magxx, Jac[0][1] = Magxy, 
     Jac[1][0] = Magyx, Jac[1][1] = Magyy */

  Magxx = magmx[0];
  Magxy = magmx[2];
  Magyx = magmx[1];
  Magyy = magmx[3];
} 

void lens_equation(double x1, double x2, double *y1, double *y2)
{
  double d_0 = (M_PI * parms[P_PHI]) / 180.0;

  /*

  x1 = x1 - xl;
  x2 = x2 - yl;

  **/

  x12 = x1 * x1;
  x22 = x2 * x2;
  
  r2 = x12 + x22;
  r4 = r2 * r2;
  r6 = r2 * r4;
  rd = sqrt(r2);     

  x1_E = x1 / parms[P_E];
  x2_E = x2 / parms[P_E];

  x1_E2 = x1_E * x1_E;
  x2_E2 = x2_E * x2_E;
  
  R_E2 = parms[P_E] * parms[P_E];
  R_E3 = parms[P_E] * R_E2;  
  R_E4 = R_E2 * R_E2;
  R_E6 = R_E2 * R_E4;
  
  r_E2 = x1_E2 + x2_E2;
  r_E  = sqrt(r_E2);
  r_E3 = r_E  * r_E2;
  r_E4 = r_E2 * r_E2;
  r_E6 = r_E2 * r_E4;
  
  R_C2 = parms[P_C] * parms[P_C];

  xc  = parms[P_C] / parms[P_E];
  xc2 = xc * xc;

  /*
    
#if 0
  xt  = parms[P_T] / parms[P_C];
  xt2 = xt * xt;
#endif

  **/

  xt  = parms[P_XT] / parms[P_E];
  xt2 = xt * xt;

  Ri_E = parms[P_Ri] / parms[P_E];
  Ro_E = parms[P_Ro] / parms[P_E];

  Ri_E2 = Ri_E * Ri_E;
  Ro_E2 = Ro_E * Ro_E;

  /*
    
  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(parms[P_PHI]);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(parms[P_PHI]);
  D  =                      - parms[P_GAMMA] * sin(parms[P_PHI]);

  A1 = 1.0 - parms[P_SIGMA]
    - parms[P_GAMMA] * cos((M_PI * parms[P_PHI]) / 180.0);
  A2 = 1.0 - parms[P_SIGMA]
    + parms[P_GAMMA] * cos((M_PI * parms[P_PHI]) / 180.0);
  D  = - parms[P_GAMMA] * sin((M_PI * parms[P_PHI]) / 180.0);

  **/

  /*

  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(d_0);
  D  = - parms[P_GAMMA] * sin(d_0);

  **/

  A1 = 1.0 - parms[P_SIGMA] - parms[P_GAMMA] * cos(2.0 * d_0);
  A2 = 1.0 - parms[P_SIGMA] + parms[P_GAMMA] * cos(2.0 * d_0);
  D  = - parms[P_GAMMA] * sin(2.0 * d_0);

  /*
    
  yeq1 = A1 * x1 + D * x2 + xl;
  yeq2 = A2 * x2 + D * x1 + yl; 

  **/
  
  yeq1 = A1 * x1 + D * x2;
  yeq2 = A2 * x2 + D * x1; 

  models[current_model](x1, x2);

  *y1 = yeq1 - alpha1;
  *y2 = yeq2 - alpha2;
}

/* #include "steepest_descent.c" */









