#include <math.h>
#include <stdio.h>
#include <string.h>

void lens_model_praxis(double *x, int i);
void choice(double *x);

void lens_model_praxis(double *x, int i)
{
  double xbar0 = 0.0, xbar1 = 0.0, alpha0 = 0.0, alpha1 = 0.0;

  /* double radius_E, radius_E2, radius_E3, radius_E4; */

  double radius_E, radius_E2, radius_E4; 

  double Mkaprp, Mkapto, rbeta, rs12, xd1, xd2, yp1, yp2, rd2, s2 = 0.0;
  double poly1, poly2, poly3, poly4, poly5, poly6, poly7;
  double funcnis, functs,func1, func2, func3, funchb, funcs, funcrl;
  double dipolm1, dipolm2, quadrupolm1, quadrupolm2; 
  double Epsilon1, Epsilon2, delta, delta2;
  
  double x0 = x_obs[i];
  double x1 = y_obs[i];

  switch(PRAXIS) {
  case FALSE:
    break;
  case TRUE:
    choice(x);
    
    E_rad  = Xprmt[2];
    E2_rad = E_rad * E_rad;
    E3_rad = E_rad * E2_rad;
    E4_rad = E_rad * E3_rad;

    CS = cos((M_PI * Xprmt[3]) / 180.0);
    SS = sin((M_PI * Xprmt[3]) / 180.0);

    B1 = 1.0 - Xprmt[5] - Xprmt[4] * cos((M_PI * Xprmt[6]) / 180.0);
    B2 = 1.0 - Xprmt[5] + Xprmt[4] * cos((M_PI * Xprmt[6]) / 180.0);
    B3 =                - Xprmt[4] * sin((M_PI * Xprmt[6]) / 180.0);

    xbar0 =   (x0 - Xprmt[0]) * CS + (x1 - Xprmt[1]) * SS;
    xbar1 = - (x0 - Xprmt[0]) * SS + (x1 - Xprmt[1]) * CS;
    
    switch(current_model) {
    case M_CHANG_REFSDAL:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
  
      alpha0 = xbar0 / radius_E2;
      alpha1 = xbar1 / radius_E2;

      break;   
    case M_SING_ISOT_SPHERE:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);

      rbeta = pow(radius_E, (Xprmt[7] - 1.0));
    
      alpha0 = rbeta * xbar0;
      alpha1 = rbeta * xbar1;  

      break;
    case M_NSING_ISOT_SPHERE:
      radius_E2 = xbar0 * xbar0 + xbar1 * xbar1;

      s2 = (Xprmt[8] * Xprmt[8] + radius_E2) / E2_rad;
    
      funcnis = sqrt(s2) - Xprmt[8] / Xprmt[2];

      alpha0 = (Xprmt[7] * funcnis * xbar0) / radius_E2;
      alpha1 = (Xprmt[7] * funcnis * xbar1) / radius_E2;

      break;
    case M_TRANSP_SPHERE:
      radius_E2 = xbar0 * xbar0 + xbar1 * xbar1;
      radius_E  = sqrt(radius_E2);

      s2 = 1.0 - radius_E2 / (Xprmt[7] * Xprmt[7]);

      functs = 1.0 - pow(s2, (3.0 / 2.0));

      if(Xprmt[7] > radius_E){
        alpha0 = (functs * xbar0) / (radius_E2 / E2_rad);
        alpha1 = (functs * xbar1) / (radius_E2 / E2_rad);    
      } else {
        alpha0 = xbar0 / (radius_E2 / E2_rad);
        alpha1 = xbar1 / (radius_E2 / E2_rad); 
      }   

      break;
    case M_ELLIPTICAL:
      Epsilon1 = 1.0 - Xprmt[8];
      Epsilon2 = 1.0 + Xprmt[8];

      delta  = 1.0 - Xprmt[10];
      delta2 = 2.0 * Xprmt[10];

      s2 = Xprmt[9] * Xprmt[9]
        + Epsilon1 * xbar0 * xbar0 + Epsilon2 * xbar1 * xbar1;
  
      alpha0 = (Xprmt[7] * Epsilon1 * E2_rad
                * xbar0) / (pow(s2, delta) * pow(Xprmt[9], delta2));
      alpha1 = (Xprmt[7] * Epsilon2 * E2_rad
                * xbar1) / (pow(s2, delta) * pow(Xprmt[9], delta2));

      break;
    case M_KING:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;

      alpha0 = (Xprmt[7] * xbar0 * log1p(radius_E2)) / radius_E2;
      alpha1 = (Xprmt[7] * xbar1 * log1p(radius_E2)) / radius_E2;

      break;
    case M_TRUNC_KING:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);

      func1 = log1p(radius_E2)
        + radius_E2 / (1.0 + (Xprmt[7] * Xprmt[7]) / E2_rad); 
      func2 = (4.0 *
               (hypot(1.0, radius_E) - 1.0)) / hypot(1.0, Xprmt[7] / E_rad);
      func3 = func1 - func2;
  
      alpha0 = (Xprmt[8] * xbar0 * func3) / radius_E2; 
      alpha1 = (Xprmt[8] * xbar1 * func3) / radius_E2;

      break;
    case M_HUBBLE:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);
  
      funchb = log1p(radius_E) - radius_E / (1.0 + radius_E);

      alpha0 = (2.0 * Xprmt[7] * xbar0 * funchb) / radius_E2;
      alpha1 = (2.0 * Xprmt[7] * xbar1 * funchb) / radius_E2;  

      break;
    case M_DE_VAUCOULEUR:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);
 
      poly1 = 720.0 / tau8 + (720.0 * pow(radius_E, (1.0 / 4.0))) / tau7;
      poly2 = poly1 + pow(radius_E, (3.0 / 2.0)) / tau2;
      poly3 = poly2 + (6.0 * pow(radius_E, (5.0 / 4.0))) / tau3;
      poly4 = poly3 + (30.0 * radius_E) / tau4;
      poly5 = poly4 + (120.0 * pow(radius_E, (3.0 / 4.0))) / tau5;
      poly6 = poly5 + (360.0 * sqrt(radius_E)) / tau6;
      poly7 = 7.0 * poly6 + pow(radius_E, (7.0 / 4.0)) / tau;

      func1 = poly7 * exp(- tau * (pow(radius_E, (1.0 / 4.0)) - 1.0));
      func2 = func1 - (5040.0 * Ftau) / tau8;

      alpha0 = - (8.0 * Xprmt[7] * xbar0 * func2) / radius_E2;
      alpha1 = - (8.0 * Xprmt[7] * xbar1 * func2) / radius_E2;

      break;
    case M_SPIRAL:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);

      funcs = 2.0 * Xprmt[7] * (1.0 - exp(- Xprmt[8] * radius_E)
                               * (Xprmt[8] * radius_E + 1.0));

      alpha0 = (funcs * xbar0) / (Xprmt[8] * Xprmt[8] * radius_E2);
      alpha1 = (funcs * xbar1) / (Xprmt[8] * Xprmt[8] * radius_E2);  

      break;
    case M_MULTIPOLE:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E4 = radius_E2 * radius_E2;

      dipolm1 = E3_rad * (Xprmt[8] * (xbar1 * xbar1 - xbar0 * xbar0)
                          - 2.0 * Xprmt[9] * xbar0 *
                          xbar1) / pow((xbar0 * xbar0 + xbar1 * xbar1), 2.0);
    
      dipolm2 = - E3_rad * (Xprmt[9] * (xbar1 * xbar1 - xbar0 * xbar0)
                            + 2.0 * Xprmt[8] * xbar0 *
                            xbar1) / pow((xbar0 * xbar0 + xbar1 * xbar1), 2.0);

      quadrupolm1 = 2.0 * E4_rad *
        (Xprmt[10] * xbar0 * (3.0 * xbar1 * xbar1 - xbar0 * xbar0)
         + Xprmt[11] * xbar1 *
         (xbar1 * xbar1
          - 3.0 * xbar0 * xbar0)) / pow((xbar0 * xbar0 + xbar1 * xbar1), 3.0);

      quadrupolm2 = 2.0 * E4_rad *
        (Xprmt[12] * xbar1 * (3.0 * xbar0 * xbar0 - xbar1 * xbar1)
         + Xprmt[11] * xbar0 *
         (xbar0 * xbar0
          - 3.0 * xbar1 * xbar1)) / pow((xbar0 * xbar0 + xbar1 * xbar1), 3.0);
    
      alpha0 = (Xprmt[7] * xbar0) / radius_E2 + dipolm1 + quadrupolm1; 
      alpha1 = (Xprmt[7] * xbar1) / radius_E2 + dipolm2 + quadrupolm2;

      break;
    case M_ROTATION_LENS:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E4 = radius_E2 * radius_E2;

      funcrl = (Xprmt[7] * xbar1 - Xprmt[8] * xbar0) / E_rad;

      alpha0 = xbar0 / radius_E2 + (2.0 * funcrl * xbar0) / radius_E4
        + Xprmt[8] / (E_rad * radius_E2);
      alpha1 = xbar1 / radius_E2 + (2.0 * funcrl * xbar1) / radius_E4
        - Xprmt[7] / (E_rad * radius_E2);
      
      break;
    case M_DOUBLE_LENS:
      C1 = 1.0 - Xprmt[15] - Xprmt[14] * cos((M_PI * Xprmt[16]) / 180.0);
      C2 = 1.0 - Xprmt[15] + Xprmt[14] * cos((M_PI * Xprmt[16]) / 180.0);
      C3 =                 - Xprmt[14] * sin((M_PI * Xprmt[16]) / 180.0);

      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;

      rs12 = ((xbar0 - Xprmt[7]) * (xbar0 - Xprmt[7])
              + (xbar1 - Xprmt[8]) * (xbar1 - Xprmt[8])) / E2_rad;

      yp1 = C1 * xbar0 + C3 * xbar1;
      yp2 = C2 * xbar1 + C3 * xbar0; 
  
      xd1 = yp1 - Xprmt[11] * Xprmt[13] * (xbar0 - Xprmt[7]) / rs12; 
      xd2 = yp2 - Xprmt[11] * Xprmt[13] * (xbar1 - Xprmt[8]) / rs12; 

      rd2 = ((xd1 - Xprmt[9]) * (xd1 - Xprmt[9])
             + (xd2 - Xprmt[10]) * (xd2 - Xprmt[10])) / E2_rad;

      alpha0 = Xprmt[11] * (xbar0 - Xprmt[7]) / rs12
        + Xprmt[12] * (xd1 - Xprmt[9]) / rd2;  
      alpha1 = Xprmt[11] * (xbar1 - Xprmt[8]) / rs12
        + Xprmt[12] * (xd2 - Xprmt[10]) / rd2;          

      break;
    case M_UNIFORM_RING:
      radius_E2 = (xbar0 * xbar0 + xbar1 * xbar1) / E2_rad;
      radius_E  = sqrt(radius_E2);
  
      Mkaprp = 10.0 * Xprmt[7] * (radius_E2 - Xprmt[8] * Xprmt[8]);
      Mkapto = 10.0 * Xprmt[7] * (Xprmt[9] * Xprmt[9] - Xprmt[8] * Xprmt[8]);
  
      if(radius_E >= Xprmt[9]) {
        alpha0 = (Mkapto * xbar0) / radius_E2;
        alpha1 = (Mkapto * xbar1) / radius_E2; 	     
      } else {
        if(radius_E >= Xprmt[8]) {
          alpha0 = (Mkaprp * xbar0) / radius_E2;
          alpha1 = (Mkaprp * xbar1) / radius_E2; 	
        } else {
          alpha0 = 0.0;
          alpha1 = 0.0; 
        }   
      }

      break; 
    }   

    Y_x[i] = B1 * xbar0 + B3 * xbar1 - alpha0;    
    Y_y[i] = B2 * xbar1 + B3 * xbar0 - alpha1;

    break;
  default:
    break;    
  }
}

/*
  
    switch(current_model) {
    case M_CHANG_REFSDAL:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else {
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
            
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;
          
              Function += (w_a * temp4) / Delta_M2;
            }         
            
            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      } 
    
    break;    
    case M_SING_ISOT_SPHERE:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0 || Xprmt[7] > 2.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }       
        }         
      }
    
      break;    
    case M_NSING_ISOT_SPHERE:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[8] < 0.0 || Xprmt[8] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }               
      }
    
      break;    
    case M_TRANSP_SPHERE:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0 || Xprmt[7] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }               
      }
    
      break;   
    case M_ELLIPTICAL:         
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[8] < 0.0 || Xprmt[8] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[9] < 0.0 || Xprmt[9] > 4.0) {
        Function += pow(10.0, 4.0);         
      } else if(Xprmt[10] < 0.0 || Xprmt[10] > 0.5) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }       
        }         
      }
    
      break;    
    case M_KING:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }               
      }
    
      break;
    case M_TRUNC_KING:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0 || Xprmt[7] > 4.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[8] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }                   
        }                 
      }
    
      break;
    case M_HUBBLE:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      }
    
      break;
    case M_DE_VAUCOULEUR:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      }
    
      break;
    case M_SPIRAL:        
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[8] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      }

      break;
    case M_MULTIPOLE:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      }

      break;
    case M_ROTATION_LENS:
      if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);

            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }                   
        }               
      }

      break;
    case M_DOUBLE_LENS:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[7] < - 4.0 || Xprmt[7] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[8] < - 4.0 || Xprmt[8] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[9] < - 4.0 || Xprmt[9] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[10] < - 4.0 || Xprmt[10] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[11] < 0.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[12] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[13] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[14] < 0.0 || Xprmt[14] > 1.0) {
        Function += pow(10.0, 4.0);         
      } else if(Xprmt[15] < 0.0 || Xprmt[15] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }                 
      }

      break;
    case M_UNIFORM_RING:
      if(Xprmt[0] < - 4.0 || Xprmt[0] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[1] < - 4.0 || Xprmt[1] > 4.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[2] < 0.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[4] < 0.0 || Xprmt[4] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[5] < 0.0 || Xprmt[5] > 1.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[7] < 0.0) {
        Function += pow(10.0, 4.0);
      } else if(Xprmt[8] < 0.0 || Xprmt[8] > 2.0) {
        Function += pow(10.0, 4.0); 
      } else if(Xprmt[9] < 0.0 || Xprmt[9] <= Xprmt[8] || Xprmt[9] > 4.0) {
        Function += pow(10.0, 4.0);         
      } else {        
        for(i = 0; i < Number_of_images; i++) {
          for(j = 0; j < Number_of_images; j++) {
            temp1 = (Y_x[i] - Y_x[j]) * (Y_x[i] - Y_x[j]);
            temp2 = (Y_y[i] - Y_y[j]) * (Y_y[i] - Y_y[j]);
          
            if(w_p != 0.0) {
              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
            } 

            if(w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

            Function += (w_a * temp4) / Delta_M2;
            }         

            if(w_p != 0.0 && w_a != 0.0) {
              temp3 = (M_O[i][j] - M_T[i] / M_T[j]);
              temp4 = temp3 * temp3;

              Function += (w_p * (temp1 + temp2)) / Delta_Y2;
              Function += (w_a * temp4) / Delta_M2;
            }
          }             
        }         
      }
    
      break;
    default:
      break; 
    }           

**/
    
void choice(double *x)
{
  if(sl_position == TRUE && sl_re == TRUE &&
     sl_macro == TRUE && sl_model == TRUE) {
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 9;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      Xprmt[8] = x[8];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 11;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = x[4];
      Xprmt[5]  = x[5];
      Xprmt[6]  = x[6];
      Xprmt[7]  = x[7];
      Xprmt[8]  = x[8];
      Xprmt[9]  = x[9];
      Xprmt[10] = x[10];

      break;
    case M_KING:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];

      break;
    case M_TRUNC_KING:
      // cprax.n = 9;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      Xprmt[8] = x[8];

      break;
    case M_HUBBLE:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];

      break;
    case M_SPIRAL:
      // cprax.n = 9;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      Xprmt[8] = x[8];

      break;
    case M_MULTIPOLE:
      // cprax.n = 13;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = x[4];
      Xprmt[5]  = x[5];
      Xprmt[6]  = x[6];
      Xprmt[7]  = x[7];
      Xprmt[8]  = x[8];
      Xprmt[9]  = x[9];
      Xprmt[10] = x[10];
      Xprmt[11] = x[11];
      Xprmt[12] = x[12];

      break;
    case M_ROTATION_LENS:
      // cprax.n = 9;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      Xprmt[8] = x[8];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 17;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = x[4];
      Xprmt[5]  = x[5];
      Xprmt[6]  = x[6];
      Xprmt[7]  = x[7];
      Xprmt[8]  = x[8];
      Xprmt[9]  = x[9];
      Xprmt[10] = x[10];
      Xprmt[11] = x[11];
      Xprmt[12] = x[12];
      Xprmt[13] = x[13];
      Xprmt[14] = x[14];
      Xprmt[15] = x[15];
      Xprmt[16] = x[16];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 10;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = x[4];
      Xprmt[5] = x[5];
      Xprmt[6] = x[6];
      Xprmt[7] = x[7];
      Xprmt[8] = x[8];
      Xprmt[9] = x[9];

      break;
    default:
      break; 
    }           
  } else if(sl_position == TRUE && sl_re == TRUE &&
            sl_macro == TRUE && sl_model == FALSE) {
    // cprax.n = 6;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];

      break;
    case M_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];
      
      break;
    case M_ROTATION_LENS:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break; 
    }           
  } else if(sl_position == TRUE && sl_re == TRUE &&
            sl_macro == FALSE && sl_model == TRUE) {
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 8;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[4];
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];    

      break;
    case M_KING:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];

      break;
    case M_TRUNC_KING:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_HUBBLE:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];

      break;
    case M_SPIRAL:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_MULTIPOLE:
      // cprax.n = 10;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[4];
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];    
      Xprmt[11] = x[8];    
      Xprmt[12] = x[9];    

      break;
    case M_ROTATION_LENS:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 11;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = x[3]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[4];
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];
      Xprmt[11] = x[8];
      Xprmt[12] = x[9];
      Xprmt[13] = x[10];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = x[3]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];
      Xprmt[9] = x[6];

      break;
    default:
      break; 
    }
  } else if(sl_position == TRUE && sl_re == TRUE &&
            sl_macro == FALSE && sl_model == FALSE) {
    // cprax.n = 3;
    
    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];

      break;
    case M_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = x[2]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = x[2]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break; 
    }
  } else if(sl_position == TRUE && sl_re == FALSE &&
            sl_macro == TRUE && sl_model == TRUE) {
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      Xprmt[8] = x[7];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 10;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2]; 
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = x[6];
      Xprmt[8]  = x[7];
      Xprmt[9]  = x[8];
      Xprmt[10] = x[9];    

      break;
    case M_KING:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];

      break;
    case M_TRUNC_KING:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      Xprmt[8] = x[7];

      break;
    case M_HUBBLE:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 7;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];

      break;
    case M_SPIRAL:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      Xprmt[8] = x[7];

      break;
    case M_MULTIPOLE:
      // cprax.n = 12;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2]; 
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = x[6];
      Xprmt[8]  = x[7];
      Xprmt[9]  = x[8];
      Xprmt[10] = x[9];
      Xprmt[11] = x[10];
      Xprmt[12] = x[11];

      break;
    case M_ROTATION_LENS:
      // cprax.n = 8;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      Xprmt[8] = x[7];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 16;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2]; 
      Xprmt[4]  = x[3];
      Xprmt[5]  = x[4];
      Xprmt[6]  = x[5];
      Xprmt[7]  = x[6];
      Xprmt[8]  = x[7];
      Xprmt[9]  = x[8];
      Xprmt[10] = x[9];
      Xprmt[11] = x[10];
      Xprmt[12] = x[11];
      Xprmt[13] = x[12];
      Xprmt[14] = x[13];
      Xprmt[15] = x[14];
      Xprmt[16] = x[15];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 9;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2]; 
      Xprmt[4] = x[3];
      Xprmt[5] = x[4];
      Xprmt[6] = x[5];
      Xprmt[7] = x[6];
      Xprmt[8] = x[7];
      Xprmt[9] = x[8];

      break;
    default:
      break; 
    }           
  } else if(sl_position == TRUE && sl_re == FALSE &&
            sl_macro == TRUE && sl_model == FALSE) {
    // cprax.n = 5;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_C];
            
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];

      break;
    case M_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[4]  = parms[P_GAMMA2];
      Xprmt[5]  = parms[P_SIGMA2];
      Xprmt[6]  = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break; 
    }
  } else if(sl_position == TRUE && sl_re == FALSE &&
            sl_macro == FALSE && sl_model == TRUE) {
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 3;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      Xprmt[8] = x[4];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 7;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[3];
      Xprmt[8]  = x[4];
      Xprmt[9]  = x[5];
      Xprmt[10] = x[6];        

      break;
    case M_KING:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];

      break;
    case M_TRUNC_KING:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      Xprmt[8] = x[4];

      break;
    case M_HUBBLE:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 4;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];

      break;
    case M_SPIRAL:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      Xprmt[8] = x[4];

      break;
    case M_MULTIPOLE:
      // cprax.n = 9;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[3];
      Xprmt[8]  = x[4];
      Xprmt[9]  = x[5];
      Xprmt[10] = x[6];        
      Xprmt[11] = x[7];        
      Xprmt[12] = x[8];        

      break;
    case M_ROTATION_LENS:
      // cprax.n = 5;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      Xprmt[8] = x[4];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 13;

      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[2];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[3];
      Xprmt[8]  = x[4];
      Xprmt[9]  = x[5];
      Xprmt[10] = x[6];        
      Xprmt[11] = x[7];        
      Xprmt[12] = x[8];        
      Xprmt[13] = x[9];        
      Xprmt[14] = x[10];        
      Xprmt[15] = x[11];        
      Xprmt[16] = x[12];        

      break;
    case M_UNIFORM_RING:
      // cprax.n = 6;

      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[2];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[3];
      Xprmt[8] = x[4];
      Xprmt[9] = x[5];

      break;
    default:
      break; 
    }
  } else if(sl_position == TRUE && sl_re == FALSE &&
            sl_macro == FALSE && sl_model == FALSE) {      
    // cprax.n = 2;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];

      break;
    case M_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = x[0]; 
      Xprmt[1]  = x[1]; 
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = x[0]; 
      Xprmt[1] = x[1]; 
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break; 
    }
  } else if(sl_position == FALSE && sl_re == TRUE &&
            sl_macro == TRUE && sl_model == TRUE) {      
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 7;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];
      Xprmt[8] = x[6];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];

      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 9;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1]; 
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = x[5];
      Xprmt[8]  = x[6];
      Xprmt[9]  = x[7];
      Xprmt[10] = x[8];    

      break;
    case M_KING:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];

      break;
    case M_TRUNC_KING:
      // cprax.n = 7;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];
      Xprmt[8] = x[6];

      break;
    case M_HUBBLE:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];

      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];

      break;
    case M_SPIRAL:
      // cprax.n = 7;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];
      Xprmt[8] = x[6];

      break;
    case M_MULTIPOLE:
      // cprax.n = 11;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1]; 
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = x[5];
      Xprmt[8]  = x[6];
      Xprmt[9]  = x[7];
      Xprmt[10] = x[8];    
      Xprmt[11] = x[9];    
      Xprmt[12] = x[10];    

      break;
    case M_ROTATION_LENS:
      // cprax.n = 7;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];
      Xprmt[8] = x[6];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 15;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1]; 
      Xprmt[4]  = x[2];
      Xprmt[5]  = x[3];
      Xprmt[6]  = x[4];
      Xprmt[7]  = x[5];
      Xprmt[8]  = x[6];
      Xprmt[9]  = x[7];
      Xprmt[10] = x[8];    
      Xprmt[11] = x[9];    
      Xprmt[12] = x[10];    
      Xprmt[13] = x[11];    
      Xprmt[14] = x[12];    
      Xprmt[15] = x[13];    
      Xprmt[16] = x[14];    

      break;
    case M_UNIFORM_RING:
      // cprax.n = 8;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1]; 
      Xprmt[4] = x[2];
      Xprmt[5] = x[3];
      Xprmt[6] = x[4];
      Xprmt[7] = x[5];
      Xprmt[8] = x[6];
      Xprmt[9] = x[7];

      break;
    default:
      break; 
    }
  } else if(sl_position == FALSE && sl_re == TRUE &&
            sl_macro == TRUE && sl_model == FALSE) {
    // cprax.n = 4;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[8] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[9] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];    

      break;
    case M_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];    

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break;     
    }
  } else if(sl_position == FALSE && sl_re == TRUE &&
            sl_macro == FALSE && sl_model == TRUE) { 
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      Xprmt[8] = x[3];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 6;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[2];
      Xprmt[8]  = x[3];
      Xprmt[9]  = x[4];
      Xprmt[10] = x[5];      

      break;
    case M_KING:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];

      break;
    case M_TRUNC_KING:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      Xprmt[8] = x[3];

      break;
    case M_HUBBLE:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];

      break;
    case M_SPIRAL:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      Xprmt[8] = x[3];

      break;
    case M_MULTIPOLE:
      // cprax.n = 8;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[2];
      Xprmt[8]  = x[3];
      Xprmt[9]  = x[4];
      Xprmt[10] = x[5];
      Xprmt[11] = x[6];
      Xprmt[12] = x[7];

      break;
    case M_ROTATION_LENS:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      Xprmt[8] = x[3];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 12;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0]; 
      Xprmt[3]  = x[1];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[2];
      Xprmt[8]  = x[3];
      Xprmt[9]  = x[4];
      Xprmt[10] = x[5];
      Xprmt[11] = x[6];
      Xprmt[12] = x[7];
      Xprmt[13] = x[8];
      Xprmt[14] = x[9];
      Xprmt[15] = x[10];
      Xprmt[16] = x[11];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0]; 
      Xprmt[3] = x[1];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[2];
      Xprmt[8] = x[3];
      Xprmt[9] = x[4];

      break;
    default:
      break; 
    }
  } else if(sl_position == FALSE && sl_re == TRUE &&
            sl_macro == FALSE && sl_model == FALSE) {
    /* We have to avoid this case because cprax.n >= 2 */

    // cprax.n = 1;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0];
      Xprmt[3]  = parms[P_THETA];    
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];    

      break;
    case M_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = x[0];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = parms[P_GAMMA2];
      Xprmt[15] = parms[P_SIGMA2];
      Xprmt[16] = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = x[0];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break;
    }
  } else if(sl_position == FALSE && sl_re == FALSE &&
            sl_macro == TRUE && sl_model == TRUE) {      
    switch(current_model) {
    case M_CHANG_REFSDAL:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 8;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = x[4];
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];        

      break;
    case M_KING:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];

      break;
    case M_TRUNC_KING:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_HUBBLE:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 5;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];

      break;
    case M_SPIRAL:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_MULTIPOLE:
      // cprax.n = 10;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = x[4]; 
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];
      Xprmt[11] = x[8]; 
      Xprmt[12] = x[9];

      break;
    case M_ROTATION_LENS:
      // cprax.n = 6;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 14;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = x[1];
      Xprmt[5]  = x[2];
      Xprmt[6]  = x[3];
      Xprmt[7]  = x[4]; 
      Xprmt[8]  = x[5];
      Xprmt[9]  = x[6];
      Xprmt[10] = x[7];
      Xprmt[11] = x[8]; 
      Xprmt[12] = x[9];
      Xprmt[13] = x[10];
      Xprmt[14] = x[11]; 
      Xprmt[15] = x[12];
      Xprmt[16] = x[13];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 7;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = x[1];
      Xprmt[5] = x[2];
      Xprmt[6] = x[3];
      Xprmt[7] = x[4];
      Xprmt[8] = x[5];
      Xprmt[9] = x[6];

      break;
    default:
      break; 
    }
  } else if(sl_position == FALSE && sl_re == FALSE &&
            sl_macro == TRUE && sl_model == FALSE) {
    // cprax.n = 3;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA]; 
      Xprmt[4]  = x[0];
      Xprmt[5]  = x[1];
      Xprmt[6]  = x[2];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];        
      
      break;
    case M_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA]; 
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[0];
      Xprmt[5]  = x[1];
      Xprmt[6]  = x[2];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_SX];
      Xprmt[8] = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 6;
      
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = x[0];
      Xprmt[5]  = x[1];
      Xprmt[6]  = x[2];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[14] = x[3];
      Xprmt[15] = x[4];
      Xprmt[16] = x[5];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = x[0];
      Xprmt[5] = x[1];
      Xprmt[6] = x[2];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break;
    }
  } else if(sl_position == FALSE && sl_re == FALSE &&
            sl_macro == FALSE && sl_model == TRUE) {
    switch(current_model) {
    case M_CHANG_REFSDAL:
      /* We have to avoid this case because cprax.n >= 2 */
      
      // cprax.n = 1;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];

      break;   
    case M_NSING_ISOT_SPHERE:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      Xprmt[8] = x[2];

      break;   
    case M_TRANSP_SPHERE:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      
      break;   
    case M_ELLIPTICAL:         
      // cprax.n = 5;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[1];
      Xprmt[8]  = x[2];
      Xprmt[9]  = x[3];
      Xprmt[10] = x[4];        

      break;
    case M_KING:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];

      break;
    case M_TRUNC_KING:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      Xprmt[8] = x[2];

      break;
    case M_HUBBLE:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      
      break;
    case M_DE_VAUCOULEUR:
      // cprax.n = 2;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];

      break;
    case M_SPIRAL:
      // cprax.n = 3;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      Xprmt[8] = x[2];

      break;
    case M_MULTIPOLE:
      // cprax.n = 7;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[1];
      Xprmt[8]  = x[2];
      Xprmt[9]  = x[3];
      Xprmt[10] = x[4];
      Xprmt[11] = x[5];
      Xprmt[12] = x[6];

      break;
    case M_ROTATION_LENS:
      // cprax.n = 3;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[1];
      Xprmt[8]  = x[2];

      break;
    case M_DOUBLE_LENS:
      // cprax.n = 11;

      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = x[0]; 
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = x[1];
      Xprmt[8]  = x[2];
      Xprmt[9]  = x[3];
      Xprmt[10] = x[4];
      Xprmt[11] = x[5];
      Xprmt[12] = x[6];
      Xprmt[13] = x[7];
      Xprmt[14] = x[8];
      Xprmt[15] = x[9];
      Xprmt[16] = x[10];

      break;
    case M_UNIFORM_RING:
      // cprax.n = 4;

      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = x[0]; 
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = x[1];
      Xprmt[8] = x[2];
      Xprmt[9] = x[3];

      break;
    default:
      break; 
    }
  } else if(sl_position == FALSE && sl_re == FALSE &&
            sl_macro == FALSE && sl_model == FALSE) {
    /* This case is impossible */
    
    // cprax.n = 0;

    switch(current_model) {
    case M_CHANG_REFSDAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];

      break;   
    case M_SING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_EXP];

      break;   
    case M_NSING_ISOT_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_C];

      break;   
    case M_TRANSP_SPHERE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      
      break;   
    case M_ELLIPTICAL:         
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA];
      Xprmt[8]  = parms[P_EPSILON];
      Xprmt[9]  = parms[P_C];
      Xprmt[10] = parms[P_ALPHA];

      break;
    case M_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_TRUNC_KING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_C];
      Xprmt[8] = parms[P_KAPPA2];

      break;
    case M_HUBBLE:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      
      break;
    case M_DE_VAUCOULEUR:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];

      break;
    case M_SPIRAL:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_ALPHA2];

      break;
    case M_MULTIPOLE:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_KAPPA0];
      Xprmt[8]  = parms[P_DX_QP];
      Xprmt[9]  = parms[P_DY_QP];
      Xprmt[10] = parms[P_QXX];
      Xprmt[11] = parms[P_QXY];
      Xprmt[12] = parms[P_QYY];

      break;
    case M_ROTATION_LENS:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_SX];
      Xprmt[8]  = parms[P_SY];

      break;
    case M_DOUBLE_LENS:
      Xprmt[0]  = xl;
      Xprmt[1]  = yl;
      Xprmt[2]  = parms[P_E];
      Xprmt[3]  = parms[P_THETA];
      Xprmt[4]  = parms[P_GAMMA];
      Xprmt[5]  = parms[P_SIGMA];
      Xprmt[6]  = parms[P_PHI];
      Xprmt[7]  = parms[P_X_1];
      Xprmt[8]  = parms[P_Y_1];
      Xprmt[9]  = parms[P_X_2];
      Xprmt[10] = parms[P_Y_2];
      Xprmt[11] = parms[P_M_1];
      Xprmt[12] = parms[P_M_2];
      Xprmt[13] = parms[P_BETA];
      Xprmt[4]  = parms[P_GAMMA2];
      Xprmt[5]  = parms[P_SIGMA2];
      Xprmt[6]  = parms[P_PHI2];

      break;
    case M_UNIFORM_RING:
      Xprmt[0] = xl;
      Xprmt[1] = yl;
      Xprmt[2] = parms[P_E];
      Xprmt[3] = parms[P_THETA];
      Xprmt[4] = parms[P_GAMMA];
      Xprmt[5] = parms[P_SIGMA];
      Xprmt[6] = parms[P_PHI];
      Xprmt[7] = parms[P_KAPPA];
      Xprmt[8] = parms[P_Ri];
      Xprmt[9] = parms[P_Ro];

      break;
    default:
      break; 
    }
  }
}

